
#include "NIBO_GFX.h"

#include "utility/niboconfig.h"
#include "utility/iodefs.h"
#include "utility/display.h"
#include "utility/gfx.h"


/* ------------------------------------------------ */

GFX_class::GFX_class() {
}

void GFX_class::begin() {
  display_init();
  gfx_init();  
}

unsigned char GFX_class::get_x() {
  return gfx_get_x();
}

unsigned char GFX_class::get_y() {
  return gfx_get_y();
}

void GFX_class::move(unsigned char x, unsigned char y) {
  gfx_move(x, y);
}

void GFX_class::draw_mode(unsigned char mode) {
  gfx_draw_mode(mode);
}

void GFX_class::fill(unsigned char val) {
  gfx_fill(val);
}

unsigned char GFX_class::char_width(char ch) {
  return gfx_char_width(ch);
}

unsigned char GFX_class::text_width(const char *txt) {
  return gfx_text_width(txt);
}

//unsigned char text_width_P(PGM_P txt);

void GFX_class::print_char(char c) {
  gfx_print_char(c);
}

void GFX_class::print_text(const char *txt) {
  gfx_print_text(txt);
}

size_t GFX_class::write(uint8_t c) {
  gfx_print_char(c);
  return 1;
}

size_t GFX_class::write(const uint8_t *buffer, size_t size) {
  size_t s = size;
  while (s--) {
    gfx_print_char(*buffer++);
  }
  return size;
}

//void print_text_P(PGM_P txt);

void GFX_class::draw_xbm(unsigned char width, unsigned char height, unsigned char * data) {
  gfx_draw_xbm(width, height, data);
}

//void draw_xbm_P(unsigned char width, unsigned char height, PGM_P data);

void GFX_class::set_proportional(unsigned char prop) {
  gfx_set_proportional(prop);
}

unsigned char GFX_class::get_proportional() {
  return gfx_get_proportional();
}

void GFX_class::hline(unsigned char length) {
  gfx_hline(length);
}

void GFX_class::vline(unsigned char length) {
  gfx_vline(length);
}

void GFX_class::box(unsigned char width, unsigned char height) {
  gfx_box(width, height);
}

void GFX_class::lineTo(unsigned char x, unsigned char y) {
  gfx_lineTo(x, y);
}

void GFX_class::pixel(unsigned char value) {
  gfx_pixel(value);
}


/* ------------------------------------------------ */

void GFX_term::clear() {
  gfx_term_clear();
}

void GFX_term::clearLine() {
  gfx_term_clearLine();
}

void GFX_term::home() {
  gfx_term_goto(0, 0);
}

void GFX_term::cursor(unsigned char col, unsigned char row) {
  gfx_term_goto(col, row);
}

void GFX_term::cursorRel(signed char col, signed char row) {
  gfx_term_goto(gfx_term_getCol()+col, gfx_term_getRow()+row);
}

void GFX_term::cursorCol(unsigned char col) {
  gfx_term_goto(col, gfx_term_getRow());
}

void GFX_term::cursorRow(unsigned char row) {
  gfx_term_goto(gfx_term_getCol(), row);
}

void GFX_term::scroll(unsigned char lines) {
  gfx_term_scroll(lines);
}

size_t GFX_term::write(uint8_t c) {
  gfx_term_writec(c);
  return 1;
}

size_t GFX_term::write(const uint8_t *buffer, size_t size) {
  gfx_term_write(buffer, size);
  return size;
}

/* ------------------------------------------------ */



GFX_class GFX;