/*
  NIBObee library for ARDUINO
  License: BSD-License
  (c) 2013 by Nils Springob, nicai-systems
*/

#ifndef __NIBOBEE_H__
#define __NIBOBEE_H__


/*! @file    NIBObee.h
 *  @brief   NIBObee library for ARDUINO
 *  @author  Nils Springob (nils@nicai-systems.de)
 *  @date    2013-09-02
 */


#if (!defined (__AVR_ATmega16__)) && (!defined (__AVR_ATmega16A__)) && (!defined(__AVR_ATmega1284P__))
#error "NIBObee library should be used with a NIBObee robot as board (ATmega16(A) or ATmega1284 MCU)"
#endif

#define ON 1
#define OFF 0

#define LED0 0
#define LED1 1
#define LED2 2
#define LED3 3


class NIBObeeClass {
  public:
    NIBObeeClass();
    void begin();
    unsigned int getVoltage();
    void checkVoltage();
    void setLed(int led, int value);
    int getLed(int led);
    unsigned int getAnalog(unsigned char adc_channel, unsigned char active=0);
    unsigned int getRandomSeed();
};

extern NIBObeeClass NIBObee;


/*---------------------------------------------------------------------------*/


class EnginePartClass {
  private:
    unsigned char id;
  public:
    EnginePartClass(unsigned char id);
    void resetTicks();
    int getTicks();
    //int getSpeed();
};


/*---------------------------------------------------------------------------*/


class EngineClass {
  public:
    EnginePartClass left;
    EnginePartClass right;
    EngineClass();
    void begin();
    void setPWM(int left, int right);
    void setSpeed(int left, int right);
    void setTargetAbs(int left, int right, unsigned int speed);
    void setTargetRel(int left, int right, unsigned int speed);
    int getTargetDist();
    void stop();
    void stopImmediate();
};

extern EngineClass Engine;


/*---------------------------------------------------------------------------*/


class LineSensorClass {
  private:
    void calibrateWhite(unsigned char id);
    void calibrateBlack(unsigned char id);
  public:
    LineSensorClass();
    void begin();
    unsigned int get(unsigned char index, unsigned char absolute=0);
    //unsigned int getRaw(unsigned char index, unsigned char variant);
    unsigned int getL(unsigned char absolute=0);
    unsigned int getC(unsigned char absolute=0);
    unsigned int getR(unsigned char absolute=0);
    void calibrateWhite();
    void calibrateBlack();
    void storeCalibration();   
};

extern LineSensorClass LineSensor;


/*---------------------------------------------------------------------------*/


class FeelerSensorClass {
  private:
    unsigned char id;
  public:
    FeelerSensorClass(unsigned char id);
    void begin();
    int get();
};

extern FeelerSensorClass FeelerL;
extern FeelerSensorClass FeelerR;


/*---------------------------------------------------------------------------*/


enum {
  SENSOR_DISABLED = 0,
  SENSOR_PULSE    = 1,
  SENSOR_LOW      = 2,
  SENSOR_HIGH     = 3
};

class ExtSensorClass {
  private:
    unsigned char id;
  public:
    ExtSensorClass(unsigned char id);
    void begin();
    void activate(unsigned char mode);
    void deactivate();
    unsigned int get(unsigned char absolute=0);
    //unsigned int getRaw(unsigned char variant);
};

extern ExtSensorClass ExtSensorL;
extern ExtSensorClass ExtSensorR;


#endif
