/*  BSD-License:

Copyright (c) 2007 by Nils Springob, nicai-systems, Germany

All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.
  * Neither the name nicai-systems nor the names of its contributors may be
    used to endorse or promote products derived from this software without
    specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

/*! @file    base.h
 *  @brief   \~english Supply voltage functions and internal definitions for the NIBO burger Library
 *           \~german Funktionen für die Versorgungsspannung und interne Definitionen der NIBO burger Library
 *  @author  Nils Springob (nils.springob@nicai-systems.com)
 *  @date    2015-03-31
 */

#ifndef _NIBOBURGER_BASE_H_
#define _NIBOBURGER_BASE_H_


#include <stdint.h>

#ifdef __cplusplus
extern "C" {
#endif


/*!
 * \internal
 * \~english Initialization state values for the library modules
 * \~german Werte für die Initialisierungszustände der Module
 */
enum {
  NIBO_ANALOG_INITIALIZED = 0x01,
  NIBO_MOTPWM_INITIALIZED = 0x02,
  NIBO_I2C_INITIALIZED    = 0x04,
};

/*!
 * \internal
 * \~english Initialization state of NIBO burger Library modules
 * \~german Initialisierungszustand der NIBO burger Library Module
 */
extern uint8_t nibo_initialization;


/*!
 * \~english generate value for nibo_assertMinVoltage() function
 * @return value for nibo_assertMinVoltage() function
 * \~german generiert den Wert für die nibo_assertMinVoltage() funktion
 * @return Wert für die nibo_assertMinVoltage() funktion
 */
#define NIBO_ASSERT_VOLTAGE(x) ((uint16_t)(1248/x))


/*!
 * \~english return accumulator voltage, requiers call to function analog_init()
 * @return accumulator voltage in millivolt
 * \~german Akkuspannung zurückgeben, erfordert den Aufruf der Funktion analog_init()
 * @return Akkuspannung in Millivolt
 */
uint16_t nibo_getMillivolt();


/*!
 * \~english check accumulator voltage, blink SOS signal if voltage is below.
 * \par value result from NIBO_ASSERT_VOLTAGE() macro
 * \~german Akkuspannung prüfen, SOS Blinken wenn die Akkuspannung niedriger ist.
 * \par value Ergebnis des NIBO_ASSERT_VOLTAGE() makros
 */
void nibo_assertMinVoltage(uint16_t value);

void nibo_setMonitorVoltage(uint16_t millivolt);

void nibo_checkMonitorVoltage();


#ifdef __cplusplus
} // extern "C"
#endif

#endif // _NIBOBURGER_BASE_H_

