/*  BSD-License:

Copyright (c) 2015 by Nils Springob, nicai-systems, Germany

All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.
  * Neither the name nicai-systems nor the names of its contributors may be
    used to endorse or promote products derived from this software without
    specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

/*!
 *  @file    key.h
 *  \~german
 *  @brief   Routinen zur Abfrage der Tasten 1 - 3
 *  \~english
 *  @brief   functions for keypad
 *  \~
 *  @author  Nils Springob (nils.springob@nicai-systems.com)
 *  @date    2015-03-31
 */

#ifndef _NIBOBURGER_KEY_H_
#define _NIBOBURGER_KEY_H_

#ifdef __cplusplus
extern "C" {
#endif


enum {
  KEY_STATE_NONE    = 0x00, //!< keine Taste gedrückt
  KEY_STATE_1       = 0x01, //!< Taste 1 gedrückt
  KEY_STATE_2       = 0x02, //!< Taste 2 gedrückt
  KEY_STATE_3       = 0x04, //!< Taste 3 gedrückt
  KEY_STATE_INVALID = 0x80  //!< Taste ungültig
};

enum {
  KEY_EVENT_NONE      = '\0', //!< keine Taste gedrückt
  
  KEY_EVENT_1_DOWN    = 'A', //!< Taste 1 gedrückt
  KEY_EVENT_2_DOWN    = 'B', //!< Taste 2 gedrückt
  KEY_EVENT_3_DOWN    = 'C', //!< Taste 3 gedrückt
  
  KEY_EVENT_1_UP      = 'a', //!< Taste 1 losgelassen
  KEY_EVENT_2_UP      = 'b', //!< Taste 2 losgelassen
  KEY_EVENT_3_UP      = 'c', //!< Taste 3 losgelassen
};


/* !
 * \~german
 * aktuellen Zustand der Tasten liefern, die gedrückten Tasten werden ODER-verknüpft. Wenn 
 * der aktuelle Wert ungültig ist liefert die Funktion den Wert KEY_INVALID zurück, ist keine
 * Taste gedrück, liefert die Funktion 0 zurück.
 * \~english
 * current state of keypad, pressed keys are combined by binary OR. If curent value is invalid,
 * the function returns KEY_INVALID, if no key is pressed the function returns 0.
 */
uint8_t key_get_state();


/*
 * liefert das nächste Key Event zurück. Key-Down-Events sind die Großbuchstaben A, B und C,
 * Key-Up-Events sind die Kleinbuchstaben a, b und c.
 * Ein einfacher Tastendruck auf den Button 1 liefert somit nacheinander 'A' und 'a'.
 * Ist kein Key-Event vorhanden liefert die Funktion eine 0 zurück
 */
char key_get_char();


#ifdef __cplusplus
} // extern "C"
#endif



#endif // _NIBOBURGER_KEY_H_
