



#include "niboburger/iodefs.h"
#include "niboburger/analog.h"
#include "niboburger/key.h"



#ifdef __cplusplus
extern "C" {
#endif

/* Key State */
uint8_t _key_unstable_cnt;

/* Key Event Buffer */  
#define _KEY_BUF_SIZE 4
uint8_t _key_buf[_KEY_BUF_SIZE];
uint8_t _key_wpos;

/* Key Event genration */  
uint8_t _key_last;


static uint8_t _key_calc_state() {
  uint16_t val = analog_getValueExt(ANALOG_KEY, 0);
  if (val>800) {
    return 0x00;
  }
  // TODO: optimize code!!
  int16_t val2 = analog_getValueExt(ANALOG_KEY, 1) - val;
  if ((val2>-4) && (val2<4)) {
    // when key signal is stable
    if (val<323) {
      return 0x07;
    } else if (val<349) {
      return 0x03;
    } else if (val<378) {
      return 0x05;
    } else if (val<457) {
      return 0x01;
    } else if (val<556) {
      return 0x06;
    } else if (val<636) {
      return 0x02;
    } else if (val<779) {
      return 0x04;
    } else {
      return 0x00;
    }
  }
  return KEY_STATE_INVALID;
}


uint8_t key_get_state() {
  uint8_t val = _key_calc_state();
  if (val==0) {
    _key_unstable_cnt=3;
    return 0;
  }
  if (val==KEY_STATE_INVALID) {
    _key_unstable_cnt=3;
    return KEY_STATE_INVALID;
  }
  if (_key_unstable_cnt) {
    _key_unstable_cnt--;
    return KEY_STATE_INVALID;
  }
  return val;
}


static void _key_put_char(char c) {
  _key_buf[_key_wpos++] = c;
  if (_key_wpos>(_KEY_BUF_SIZE-1)) {
    _key_wpos=0;
  }
}


static uint8_t _key_get_char() {
  uint8_t rpos=_key_wpos;
  do {
    rpos = rpos?(rpos-1):(_KEY_BUF_SIZE-1);
    char c = _key_buf[rpos];
    if (c) {
      _key_buf[rpos] = 0;
      return c;
    }
  } while (rpos!=_key_wpos);
  return 0;
}



void _key_update() {
  uint8_t key = key_get_state();
  
  if (key==KEY_STATE_INVALID) {
    return;
  }
  
  uint8_t event = key ^ _key_last;
  
  if (event) {
    // a key changed....
    if (event & 0x01) {
      _key_put_char((key & 0x01)?'A':'a');
    }
    if (event & 0x02) {
      _key_put_char((key & 0x02)?'B':'b');
    }
    if (event & 0x04) {
      _key_put_char((key & 0x04)?'C':'c');
    }
    _key_last = key;
  }
}

char key_get_char() {
  _key_update();
  return _key_get_char();
}

#ifdef __cplusplus
} // extern "C"
#endif
