#include "NIBO2.h"

#include "nibo/niboconfig.h"
#include "nibo/iodefs.h"
#include "nibo/leds.h"
#include "nibo/floor.h"
#include "nibo/bot.h"
#include "nibo/spi.h"
#include "nibo/copro.h"
#include "nibo/analog.h"
#include "nibo/pwm.h"


NIBO2Class::NIBO2Class() {
  
}

void NIBO2Class::begin() {
  bot_init();
  leds_init();
  pwm_init();
  spi_init();
  floor_init();
}

void NIBO2Class::enableCopro() {
  activate_output_bit(IO_RESET_CO);
  set_output_bit(IO_RESET_CO);
}
 
void NIBO2Class::disableCopro() {
  activate_output_bit(IO_RESET_CO);
  clear_output_bit(IO_RESET_CO);
}

unsigned char NIBO2Class::getS3() {
  return !get_input_bit(IO_INPUT_1);
}

unsigned char NIBO2Class::getJP1() {
  return !get_input_bit(IO_INPUT_2);
}



unsigned int NIBO2Class::getVoltage() {
  int32_t val = analog_getValue(ANALOG_SUPPLY);
  val *= 133;
  val /= 8;
  val -= 1190;
  return val;
}

void NIBO2Class::setLed(unsigned char led, unsigned char color) {
  led = 1<<led;
  if (color&LED_GREEN) {
    IO_LEDS_GREEN_PORT |= led;
  } else {
    IO_LEDS_GREEN_PORT &= ~led;
  }
  
  if (color&LED_RED) {
    IO_LEDS_RED_PORT |= led;
  } else {
    IO_LEDS_RED_PORT &= ~led;
  }
}

unsigned char NIBO2Class::getLed(unsigned char led) {
  led = 1<<led;
  unsigned char res=0;
  if (IO_LEDS_GREEN_PORT & led) {
    res |= LED_GREEN;
  }
  if (IO_LEDS_RED_PORT & led) {
    res |= LED_RED;
  }
  return res;
}


void NIBO2Class::setLedsIntensity(int light) {
  leds_set_status_intensity(light);
}


void NIBO2Class::setHeadlightsIntensity(int light) {
  leds_set_headlights(light);
}


void NIBO2Class::setDisplayIntensity(int light) {
  leds_set_displaylight(light);
}


unsigned int NIBO2Class::getRandomSeed() {
  return analog_getRandomSeed();
}


unsigned int NIBO2Class::getAnalog(unsigned char channel, unsigned char active) {
  if (active) {
    switch (channel) {
      case 0: channel = ANALOG_FLOOR_R1; break;
      case 1: channel = ANALOG_FLOOR_L1; break;
      case 2: channel = ANALOG_LINE_L1; break;
      case 3: channel = ANALOG_LINE_R1; break;
      case 4: channel = ANALOG_INPUT31; break;
      case 5: channel = ANALOG_INPUT21; break;
      default: return 0;
    }
  } else {
    switch (channel) {
      case 0: channel = ANALOG_FLOOR_R0; break;
      case 1: channel = ANALOG_FLOOR_L0; break;
      case 2: channel = ANALOG_LINE_L0; break;
      case 3: channel = ANALOG_LINE_R0; break;
      case 4: channel = ANALOG_INPUT30; break;
      case 5: channel = ANALOG_INPUT20; break;
      case 7: channel = ANALOG_SUPPLY; break;
      default: return 0;
    }
  }
  return analog_getValue(channel);
}


NIBO2Class NIBO2;


/*--------------------------------------------------*/



FloorSensorClass::FloorSensorClass() {
  
}

void FloorSensorClass::begin() {
  
  
}


unsigned int FloorSensorClass::get(unsigned char ch, unsigned char absolute) {
  if (absolute) {
    return floor_getAbs(ch);
  } else {
    return floor_get(ch);
  }
}


unsigned int FloorSensorClass::getRaw(unsigned char ch, unsigned char variant) {
  if (variant==2) {
    return floor_getRawDiff(ch);
  } else if (variant==1) {
    return floor_getRaw1(ch);
  } else if (variant==0) {
    return floor_getRaw0(ch);
  }
  return 0;
}


void FloorSensorClass::calibrateWhite() {
  floor_calibrateWhite();
}

void FloorSensorClass::calibrateBlack() {
  floor_calibrateBlack();  
}

void FloorSensorClass::storeCalibration() {
  floor_writePersistent();
}   

FloorSensorClass FloorSensor;


/*--------------------------------------------------*/

EnginePartClass::EnginePartClass(unsigned char _id) {
  id=_id;
}

int EnginePartClass::getTicks() {
  copro_update();
  return id?copro_ticks_r:copro_ticks_l;
}

int EnginePartClass::getSpeed() {
  copro_update();
  return id?copro_speed_r:copro_speed_l;
}

int EnginePartClass::getCurrent() {
  copro_update();
  return id?copro_current_r:copro_current_l;
}

/*--------------------------------------------------*/


EngineClass::EngineClass() : left(0), right(1) {
  
}

void EngineClass::begin() {
}


void EngineClass::setPWM(int left, int right) {
  copro_setPWM(left, right);
}

void EngineClass::setSpeed(int left, int right) {
  copro_setSpeed(left, right);  
}

void EngineClass::setTargetAbs(int left, int right, unsigned int speed) {
  copro_setTargetAbs(left, right, speed);  
}

void EngineClass::setTargetRel(int left, int right, unsigned int speed) {
  copro_setTargetRel(left, right, speed);
}

int EngineClass::getTargetDist() {
  copro_update();
  return copro_target_distance;
}

void EngineClass::stop() {
  copro_stop();
}

void EngineClass::stopImmediate() {
  copro_stopImmediate();
}


EngineClass Engine;



/*--------------------------------------------------*/

DistanceSensorClass::DistanceSensorClass() {
  
}

void DistanceSensorClass::begin() {
  beginMeasure();
}

void DistanceSensorClass::beginMeasure() {
  copro_ir_startMeasure();
}

void DistanceSensorClass::beginReceiveRC5() {
  copro_ir_stop();
}

unsigned int DistanceSensorClass::get(unsigned char dir) {
  copro_update();
  uint16_t dist = copro_distance[dir];
  return dist;
}

unsigned char DistanceSensorClass::transmitRC5(unsigned int code) {
  return copro_transmitRC5(code);
}

unsigned int DistanceSensorClass::getLastRC5() {
  copro_update();
  unsigned int result = copro_rc5_cmd;
  copro_rc5_cmd = 0;
  return result;
}

DistanceSensorClass DistSensor;





