/*  BSD-License:

Copyright (c) 2007 by Nils Springob, nicai-systems, Germany

All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.
  * Neither the name nicai-systems nor the names of its contributors may be
    used to endorse or promote products derived from this software without
    specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

/*! @file   base.c
 *  @brief  Interne Definitionen für die NIBObee Lib
 *  @author Nils Springob
 *  @date   2009-09-26
 */

#include "nibobee/base.h"
#include "nibobee/analog.h"
#include "nibobee/iodefs.h"
#include "nibobee/led.h"
#include "nibobee/motpwm.h"
#include <util/delay.h>

#define MIN_VOLTAGE 4.2

#ifdef __cplusplus
extern "C" {
#endif


uint8_t nibobee_initialization;

uint16_t nibobee_getMillivolt() {
  uint16_t voltage = analog_getValue(ANALOG_VOLT);
  voltage = (4*12480U)/voltage;
  voltage *= 25;
  return voltage;
}

static void delay(uint8_t count) {
  while (count--) {
    _delay_ms(100);
  }
}

static void blink_all(uint8_t on, uint8_t off) {
  led_set(LED_L_RD, 1);
  led_set(LED_R_RD, 1);
  led_set(LED_L_YE, 1);
  led_set(LED_R_YE, 1);
  delay(on);
  led_set(LED_L_RD, 0);
  led_set(LED_R_RD, 0);
  led_set(LED_L_YE, 0);
  led_set(LED_R_YE, 0);
  delay(off);
}

void nibobee_assertMinVoltage(uint16_t value) {
  uint16_t voltage = analog_getValue(ANALOG_VOLT);
  if (voltage>value) {
    activate_output_group(IO_LEDS);  // LED bits als Output
    motpwm_stop();
    disable_interrupts();
    deactivate_output_group(IO_MOTOR);

    while (1) {
      blink_all(1,1); // S
      blink_all(1,1);
      blink_all(1,1);
      blink_all(3,1); // O
      blink_all(3,1);
      blink_all(3,1);
      blink_all(1,1); // S
      blink_all(1,1);
      blink_all(1,3);
    }
  }
}


#ifdef __cplusplus
} // extern "C"
#endif
