/*  BSD-License:

Copyright (c) 2007 by Nils Springob, nicai-systems, Germany

All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.
  * Neither the name nicai-systems nor the names of its contributors may be
    used to endorse or promote products derived from this software without
    specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

/*! @file    feeler.c
 *  @brief   Zuordnung der physikalischen Pins zu symbolischen Namen
 *  @author  Nils Springob (nils@nicai-systems.de)
 *  @date    2014-10-31
 */

#include "nibobee/iodefs.h"
#include "nibobee/feeler.h"
#include "nibobee/delay.h"

#ifdef __cplusplus
extern "C" {
#endif



void feeler_init() {
  set_output_group(IO_SENS);
}

int8_t feeler_getLeft () {
  uint8_t val = IO_SENS_PIN & (_BV(IO_SENS_BIT_LEFT1) | _BV(IO_SENS_BIT_LEFT2));
  if (val==_BV(IO_SENS_BIT_LEFT1)) return -1;
  if (val==_BV(IO_SENS_BIT_LEFT2)) return 1;
  return 0;
}

int8_t feeler_getRight () {
  uint8_t val = IO_SENS_PIN & (_BV(IO_SENS_BIT_RIGHT1) | _BV(IO_SENS_BIT_RIGHT2));
  if (val==_BV(IO_SENS_BIT_RIGHT1)) return -1;
  if (val==_BV(IO_SENS_BIT_RIGHT2)) return 1;
  return 0;
}

  
int8_t feeler_getAll() {
  return ((IO_SENS_PIN & 0xf0)>>4);
}

int8_t feeler_waitInput() {
  uint8_t result = 0;
  uint8_t val = 0;
  
  do {
    val = feeler_getAll();
    delay(1);
  } while (val==0);
  
  result |= val;
  
  do {
    val = feeler_getAll();
    result |= (val<<4);
    delay(1);
  } while (val!=0);
  
  return result & (~(result<<4));
}



#ifdef __cplusplus
} // extern "C"
#endif



