/*  BSD-License:

Copyright (c) 2007 by Nils Springob, nicai-systems, Germany

All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.
  * Neither the name nicai-systems nor the names of its contributors may be
    used to endorse or promote products derived from this software without
    specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

/*! @file    led.c
 *  @brief   Routinen zur Ansteuerung der LEDs
 *  @author  Nils Springob (nils@nicai-systems.de)
 *  @date    2009-09-26
 */

#include "nibobee/iodefs.h"
#include "nibobee/led.h"

#ifdef __cplusplus
extern "C" {
#endif


void led_init() {
  activate_output_group(IO_LEDS);  // LED bits als Output
}

void led_set(uint8_t led, uint8_t status) {
  if (status) {
    switch (led) {
      case 0: set_output_groupbit(IO_LEDS, L_YE); break;
      case 1: set_output_groupbit(IO_LEDS, L_RD); break;
      case 2: set_output_groupbit(IO_LEDS, R_RD); break;
      case 3: set_output_groupbit(IO_LEDS, R_YE); break;
    }
  } else {
    switch (led) {
      case 0: clear_output_groupbit(IO_LEDS, L_YE); break;
      case 1: clear_output_groupbit(IO_LEDS, L_RD); break;
      case 2: clear_output_groupbit(IO_LEDS, R_RD); break;
      case 3: clear_output_groupbit(IO_LEDS, R_YE); break;
    }
  }
}

uint8_t led_get(uint8_t led) {
  switch (led) {
    case 0: return get_output_groupbit(IO_LEDS, L_YE)?1:0;
    case 1: return get_output_groupbit(IO_LEDS, L_RD)?1:0;
    case 2: return get_output_groupbit(IO_LEDS, R_RD)?1:0;
    case 3: return get_output_groupbit(IO_LEDS, R_YE)?1:0;
  }
  return 0;
}

#ifdef __cplusplus
} // extern "C"
#endif

