#include "nibo/niboconfig.h"
#include "nibo/iodefs.h"
#include "nibo/delay.h"
#include "nibo/display.h"
#include "nibo/gfx.h"
#include "nibo/i2cmaster.h"
#include "nibo/motco.h"
#include "nibo/irco.h"
#include "nibo/adc.h"


#include <avr/interrupt.h>
#include <avr/pgmspace.h>


#define LOBYTE(x)        (uint8_t)((uint16_t)x)
#define HIBYTE(x)        (uint8_t)(((uint16_t)x)>>8)
#define MAKE_WORD(hi,lo) ((hi*0x100)+lo)

int16_t tspeed_l = 0;
int16_t tspeed_r = 0;

void show_boot_message_gfx(void);


void print_hex (uint8_t val)
{
  char c1=val/16;
  char c2=val%16;
  if (c1<10) c1+='0'; else c1+='a'-10;
  if (c2<10) c2+='0'; else c2+='a'-10;
  gfx_print_char(c1, 0);
  gfx_print_char(c2, 0);
}


void output_status(char c)
{
  uint8_t x = gfx_get_x();
  uint8_t y = gfx_get_y();
  gfx_move(0, 0);
  gfx_print_char(c, 0);
  gfx_move(x, y);
}

void nibo_test_nav() {
  gfx_fill(0x00);
  gfx_move(5, 5);
  gfx_print_text("nibo navigation test", 1);
  i2c_init();

  adc_init(AN_FLOOR_L);
  adc_init(AN_FLOOR_R);
  adc_init(AN_LINE_L);
  adc_init(AN_LINE_R);
  activate_output_bit(IO_LINE_FLOOR_EN);

  int8_t mode=2;
  //int8_t old_mode=2;
  uint8_t error_motco = 0;
  uint8_t error_distco = 0;

  uint8_t c='0';

  irco_startMeasure();
  motco_setParameters(5, 4, 6); // ki, kp, kd
  
  while (1)
  {
    delay(10);
//    motco_setParameters(1, 1, 1); // ki, kp, kd
    delay(10);
  //  motco_setParameters(1, 1, 1); // ki, kp, kd
    if (error_motco>10)
    {
      output_status('h');
      activate_output_bit(IO_RESET_5);
      activate_output_bit(IO_RESET_3);
      clear_output_bit(IO_RESET_5);
      clear_output_bit(IO_RESET_3);
      delay(10);
      i2c_init();
      set_output_bit(IO_RESET_5);
      set_output_bit(IO_RESET_3);
      delay(100);
      error_motco=0;
      error_distco=0;
    }
    if (error_distco>10)
    {
      output_status('i');
      activate_output_bit(IO_RESET_5);
      activate_output_bit(IO_RESET_3);
      clear_output_bit(IO_RESET_5);
      clear_output_bit(IO_RESET_3);
      delay(10);
      i2c_init();
      set_output_bit(IO_RESET_5);
      set_output_bit(IO_RESET_3);
      delay(100);
      error_motco=0;
      error_distco=0;
    }
    delay(90);
    int16_t speed_l = 0;
    int16_t speed_r = 0;

    // Request distance data
    output_status('a');
    if (irco_update())
    {
      
      uint8_t dist_l  = irco_distance[4];
      uint8_t dist_vl = irco_distance[3];
      uint8_t dist_v  = irco_distance[2];
      uint8_t dist_vr = irco_distance[1];
      uint8_t dist_r  = irco_distance[0];

      output_status('c');
      error_distco=0;
      gfx_move(25, 35);
      print_hex(i2c_status());
      gfx_move(25, 45);
      print_hex(irco_distance[0]);
      gfx_print_char(' ', 0);
      print_hex(irco_distance[1]);
      gfx_print_char(' ', 0);
      print_hex(irco_distance[2]);
      gfx_print_char(' ', 0);
      print_hex(irco_distance[3]);
      gfx_print_char(' ', 0);
      print_hex(irco_distance[4]);

      uint8_t ledr = 0x00;
      uint8_t ledg = 0x00;

      if (irco_distance[0]<0x30) ledg|=0x80;
      if (irco_distance[1]<0x30) ledg|=0x40;
      if (irco_distance[2]<0x30) ledg|=0x30;
      if (irco_distance[3]<0x30) ledg|=0x08;
      if (irco_distance[4]<0x30) ledg|=0x04;

      if (irco_distance[0]>0xa0) ledr|=0x80;
      if (irco_distance[1]>0xa0) ledr|=0x40;
      if (irco_distance[2]>0xa0) ledr|=0x30;
      if (irco_distance[3]>0xa0) ledr|=0x08;
      if (irco_distance[4]>0xa0) ledr|=0x04;

      uint16_t sum = irco_distance[0] + irco_distance[1]
                   + irco_distance[2] + irco_distance[3]
                   + irco_distance[4];
      uint8_t dmax = (2*sum)/5;
      uint8_t dmin = sum/(2*5);

      if (irco_distance[0]<dmin) ledg|=0x80;
      if (irco_distance[1]<dmin) ledg|=0x40;
      if (irco_distance[2]<dmin) ledg|=0x30;
      if (irco_distance[3]<dmin) ledg|=0x08;
      if (irco_distance[4]<dmin) ledg|=0x04;

      if (irco_distance[0]>dmax) ledr|=0x80;
      if (irco_distance[1]>dmax) ledr|=0x40;
      if (irco_distance[2]>dmax) ledr|=0x30;
      if (irco_distance[3]>dmax) ledr|=0x08;
      if (irco_distance[4]>dmax) ledr|=0x04;

      ledr=0;
      
      if ((dist_l<254) && (dist_l<=dist_vl) && (dist_l<=dist_v) && (dist_l<=dist_vr) && (dist_l<=dist_r))
        speed_l=-25, speed_r=25, ledr= 0x04, mode=-1; // links frei!
      if ((dist_r<254) && (dist_r<=dist_l) && (dist_r<=dist_vl) && (dist_r<=dist_v) && (dist_r<=dist_vr))
        speed_l=25, speed_r=-25, ledr= 0x80, mode=1; // rechts frei!
      if ((dist_vl<254) && (dist_vl<=dist_l) && (dist_vl<=dist_v) && (dist_vl<=dist_vr) && (dist_vl<=dist_r))
        speed_l=130, speed_r=160, ledr= 0x08, mode=0; // vorne links frei!
      if ((dist_vr<254) && (dist_vr<=dist_l) && (dist_vr<=dist_vl) && (dist_vr<=dist_v) && (dist_vr<=dist_r))
        speed_l=160, speed_r=130, ledr= 0x40, mode=0; // vorne rechts frei!

      if (dist_v<254)
      {
        if (dist_v>5) dist_v-=5;
        if ((dist_v<=dist_l) && (dist_v<=dist_vl) && (dist_v<=dist_vr) && (dist_v<=dist_r))
          speed_l=180, speed_r=180, ledr= 0x30, mode=0; // vorne frei!
      }
      
      IO_LEDS_RED_PORT=ledr;
      IO_LEDS_GREEN_PORT=ledg;

      if (ledr&0x30)
        set_output_bit(IO_LED_WHITE);
      else
        clear_output_bit(IO_LED_WHITE);
      
      uint16_t lval=(dist_vl+dist_l)/2;
      uint16_t rval=(dist_vr+dist_r)/2;

      if (lval>rval) speed_r-=15; // rechts mehr Platz
      if (rval>lval) speed_l-=15; // links mehr Platz


      //speed_l=20;
      //speed_r=0x80;
    }
    else
    {
      error_distco++;
      continue;
    }
    output_status('d');

    gfx_move(25, 55);
    gfx_print_text("r:", 1);
    print_hex(HIBYTE(speed_r));
    print_hex(LOBYTE(speed_r));
    gfx_print_text(" l:", 1);
    print_hex(HIBYTE(speed_l));
    print_hex(LOBYTE(speed_l));


    if (((tspeed_l<0) && (speed_l>0)) ||
        ((tspeed_l>0) && (speed_l<0)) ||
        ((tspeed_r<0) && (speed_r>0)) ||
        ((tspeed_r>0) && (speed_r<0)))
    {
      motco_stop();
      tspeed_l=0;
      tspeed_r=0;
      speed_l=0;
      speed_r=0;
    }
        
    tspeed_l = (3*tspeed_l+speed_l)/4;
    tspeed_r = (3*tspeed_r+speed_r)/4;
    
    output_status('e');
    if (motco_setSpeed(tspeed_l, tspeed_r))
    {
      if (c=='9')
        c='0';
      else
        c++;
      gfx_move(25, 25);
      gfx_print_char(c, 0);
      gfx_move(45, 25);
      print_hex(TWCR);
      gfx_print_char(' ', 0);
      print_hex(i2c_last_TWSR);
    }
    else
    {
      output_status('g');
      gfx_move(45, 25);
      print_hex(TWCR);
      gfx_print_char(' ', 0);
      print_hex(i2c_last_TWSR);
      error_motco++;
    }
    gfx_move(25, 35);
    print_hex(i2c_status());

    // Analog values:
    int16_t value_fl = adc_read(AN_FLOOR_L);
    int16_t value_fr = adc_read(AN_FLOOR_R);
    int16_t value_ll = adc_read(AN_LINE_L);
    int16_t value_lr = adc_read(AN_LINE_R);
    set_output_bit(IO_LINE_FLOOR_EN);
    delay(1);
    value_fl -= adc_read(AN_FLOOR_L);
    value_fr -= adc_read(AN_FLOOR_R);
    value_ll -= adc_read(AN_LINE_L);
    value_lr -= adc_read(AN_LINE_R);
    clear_output_bit(IO_LINE_FLOOR_EN);

    if (value_fl<0) value_fl=0;
    if (value_fr<0) value_fr=0;
    if (value_ll<0) value_ll=0;
    if (value_lr<0) value_lr=0;
    if (value_fl>255) value_fl=255;
    if (value_fr>255) value_fr=255;
    if (value_ll>255) value_ll=255;
    if (value_lr>255) value_lr=255;
    
    
    gfx_move(100, 10);
    print_hex(value_fl);
    gfx_move(100, 20);
    print_hex(value_fr);
    gfx_move(100, 30);
    print_hex(value_ll);
    gfx_move(100, 40);
    print_hex(value_lr);
    
  }

}




void nibo_test_com() {
  gfx_fill(0x00);
  gfx_move(5, 5);
  gfx_print_text("nibo communication test", 1);
  i2c_init();

  uint8_t c='0';
  while (1)
  {
    delay(1000);
    
    // write value
    i2c_size = 6;
    i2c_buf[0] = ((44<<1));
    i2c_buf[1] = 0xee;
    i2c_buf[2] = 0x22;
    i2c_buf[3] = 0x33;
    i2c_buf[4] = 0x44;
    i2c_buf[5] = 0x55;
    
    gfx_move(0, 15);
    gfx_print_text("tx:", 1);
    i2c_start_transmission();
    if (i2c_wait_transmission())
    {
      if (c=='9')
        c='0';
      else
        c++;
      gfx_move(25, 15);
      gfx_print_char(c, 0);
    }
    gfx_move(45, 15);
    print_hex(TWCR);
    gfx_print_char(' ', 0);
    print_hex(i2c_last_TWSR);
    gfx_print_char(' ', 0);
    print_hex(i2c_status());
    
    // read value
    i2c_size = 4;
    i2c_buf[0] = ((44<<1)+1);
    
    gfx_move(0, 25);
    gfx_print_text("rx:", 1);
    i2c_start_transmission();
    if (i2c_wait_transmission())
    {
      if (c=='9')
        c='0';
      else
        c++;
      gfx_move(25, 25);
      gfx_print_char(c, 0);
    }
    gfx_move(45, 25);
    print_hex(TWCR);
    gfx_print_char(' ', 0);
    print_hex(i2c_last_TWSR);
    gfx_print_char(' ', 0);
    print_hex(i2c_status());

    gfx_move(0, 35);
    gfx_print_text("s:", 1);
    print_hex(i2c_buf[1]);
    gfx_print_text(" v:", 1);
    print_hex(i2c_buf[2]);
    gfx_print_text(" ~v:", 1);
    print_hex(i2c_buf[3]);
    

  }
}

void nibo_test_odometry() {
  gfx_print_text("nibo odometry test", 1);

  uint8_t c0='0';
  while (1)
  {
    i2c_size=5;
    i2c_buf[0]=((44<<1)+1);
    i2c_start_transmission();
    i2c_wait_transmission();
    if (c0=='9')
      c0='0';
    else
      c0++;
    gfx_move(25, 25);
    gfx_print_char(c0, 0);
    gfx_move(25, 35);
    print_hex(i2c_status());
    gfx_move(25, 45);
    print_hex(i2c_buf[1]);
    print_hex(i2c_buf[2]);
    gfx_print_char(' ', 0);
    print_hex(i2c_buf[3]);
    print_hex(i2c_buf[4]);
    delay(100);
  }
  
}

void nibo_test_distance() {
  gfx_print_text("nibo distance test", 1);

  uint8_t c='0';
  irco_startMeasure();
  while (1)
  {
    delay(50);
    i2c_size=8;
    i2c_buf[0]=((42<<1)+1);
    i2c_buf[1]=0xaf;
    i2c_buf[2]=0xfe;
    i2c_buf[3]=0xaf;
    i2c_buf[4]=0xfe;
    i2c_buf[5]=0xaf;
    i2c_buf[6]=0xfe;
    i2c_buf[7]=0xaf;
    if (!i2c_start_transmission())
    {
      gfx_move(10, 25);
      gfx_print_text(" cr=", 1);
      print_hex(TWCR);
      gfx_print_text(" sr=", 1);
      print_hex(i2c_last_TWSR);
      gfx_print_text(" pos=", 1);
      print_hex(i2c_pos);
      gfx_move(25, 35);
      print_hex(i2c_status());
//      while(1)
//        gfx_move(10, 25);
    }
    if (i2c_wait_transmission())
    {
      if (c=='9')
        c='0';
      else
        c++;
      gfx_move(0, 25);
      gfx_print_char(c, 0);
    }
//    else
//    {
      gfx_move(10, 25);
      gfx_print_text(" cr=", 1);
      print_hex(TWCR);
      gfx_print_text(" sr=", 1);
      print_hex(i2c_last_TWSR);
      gfx_print_text(" pos=", 1);
      print_hex(i2c_pos);
      gfx_move(25, 35);
      print_hex(i2c_status());
//      while(1)
//        gfx_move(10, 25);
//    }
    gfx_move(25, 45);
    print_hex(i2c_buf[1]);
    gfx_print_char(' ', 0);
    print_hex(i2c_buf[2]);
    gfx_print_char(' ', 0);
    print_hex(i2c_buf[3]);
    gfx_print_char(' ', 0);
    print_hex(i2c_buf[4]);
    gfx_print_char(' ', 0);
    print_hex(i2c_buf[5]);
    gfx_move(25, 55);
    print_hex(i2c_buf[6]);
    print_hex(i2c_buf[7]);

    uint8_t ledr = 0x00;
    uint8_t ledg = 0x00;

    if (i2c_buf[1]<0x30) ledg|=0x80;
    if (i2c_buf[2]<0x30) ledg|=0x40;
    if (i2c_buf[3]<0x30) ledg|=0x30;
    if (i2c_buf[4]<0x30) ledg|=0x08;
    if (i2c_buf[5]<0x30) ledg|=0x04;

    if (i2c_buf[1]>0xa0) ledr|=0x80;
    if (i2c_buf[2]>0xa0) ledr|=0x40;
    if (i2c_buf[3]>0xa0) ledr|=0x30;
    if (i2c_buf[4]>0xa0) ledr|=0x08;
    if (i2c_buf[5]>0xa0) ledr|=0x04;

    uint16_t sum = i2c_buf[1]+i2c_buf[2]+i2c_buf[3]+i2c_buf[4]+i2c_buf[5];
    uint8_t dmax = (2*sum)/5;
    uint8_t dmin = sum/(2*5);

    if (i2c_buf[1]<dmin) ledg|=0x80;
    if (i2c_buf[2]<dmin) ledg|=0x40;
    if (i2c_buf[3]<dmin) ledg|=0x30;
    if (i2c_buf[4]<dmin) ledg|=0x08;
    if (i2c_buf[5]<dmin) ledg|=0x04;

    if (i2c_buf[1]>dmax) ledr|=0x80;
    if (i2c_buf[2]>dmax) ledr|=0x40;
    if (i2c_buf[3]>dmax) ledr|=0x30;
    if (i2c_buf[4]>dmax) ledr|=0x08;
    if (i2c_buf[5]>dmax) ledr|=0x04;

    IO_LEDS_RED_PORT=ledr;
    IO_LEDS_GREEN_PORT=ledg;

  }
  
}

enum
{
  BHV_STOP       = 0,
  BHV_OBSTACLE   = 1,
  BHV_OBSTACLE_L = 2,
  BHV_OBSTACLE_R = 3,
  BHV_SPACE      = 4,
  BHV_SPACE_L    = 5,
  BHV_SPACE_R    = 6,
  BHV_GO         = 7
};

uint8_t calc_rel_dist(uint8_t min_d, uint8_t dist, uint8_t max_d)
{
  if (dist == 0x00) return 0x00;
  if (dist == 0xff) return 0xff;
  uint16_t w = max_d-min_d;
  uint16_t x = dist-min_d;
  x *= 0xf0;
  x /= w;
  x += 0x08;
  return x;
}



void nibo_test_behaviour() {
  uint16_t max_filter = 0;
  uint16_t min_filter = 0;
  uint8_t behaviour = BHV_STOP;
  uint8_t counter = 0;
  
  gfx_fill(0x00);
  gfx_move(5, 5);
  gfx_print_text("nibo behaviour test", 1);
  i2c_init();

  adc_init(AN_FLOOR_L);
  adc_init(AN_FLOOR_R);
  adc_init(AN_LINE_L);
  adc_init(AN_LINE_R);
  activate_output_bit(IO_LINE_FLOOR_EN);

  int8_t mode=2;
  //int8_t old_mode=2;
  uint8_t error_motco = 0;
  uint8_t error_distco = 0;
  irco_startMeasure();

  while (1)
  {
    if (error_motco>10)
    {
      //output_status('h');
      activate_output_bit(IO_RESET_5);
      activate_output_bit(IO_RESET_3);
      clear_output_bit(IO_RESET_5);
      clear_output_bit(IO_RESET_3);
      delay(10);
      i2c_init();
      set_output_bit(IO_RESET_5);
      set_output_bit(IO_RESET_3);
      delay(100);
      error_motco=0;
      error_distco=0;
    }
    if (error_distco>10)
    {
      //output_status('i');
      activate_output_bit(IO_RESET_5);
      activate_output_bit(IO_RESET_3);
      clear_output_bit(IO_RESET_5);
      clear_output_bit(IO_RESET_3);
      delay(10);
      i2c_init();
      set_output_bit(IO_RESET_5);
      set_output_bit(IO_RESET_3);
      delay(100);
      error_motco=0;
      error_distco=0;
    }
    delay(100);
    int16_t speed_l = 0;
    int16_t speed_r = 0;

    // Request distance data
    //output_status('a');
    if (irco_update())
    {
      
      uint8_t dist_l  = irco_distance[4];
      uint8_t dist_vl = irco_distance[3];
      uint8_t dist_v  = irco_distance[2];
      uint8_t dist_vr = irco_distance[1];
      uint8_t dist_r  = irco_distance[0];
      
      error_distco=0;
      gfx_move(0, 35);
      print_hex(dist_r);
      gfx_print_char(' ', 0);
      print_hex(dist_vr);
      gfx_print_char(' ', 0);
      print_hex(dist_v);
      gfx_print_char(' ', 0);
      print_hex(dist_vl);
      gfx_print_char(' ', 0);
      print_hex(dist_l);

      
      uint8_t max_l = (dist_l>dist_vl)?dist_l:dist_vl;
      uint8_t min_l = (dist_l<dist_vl)?dist_l:dist_vl;
      uint8_t max_r = (dist_r>dist_vr)?dist_r:dist_vr;
      uint8_t min_r = (dist_r<dist_vr)?dist_r:dist_vr;
      uint8_t max_d = (max_l>max_r)?max_l:max_r;
      max_d = (max_d>dist_v)?max_d:dist_v;
      uint8_t min_d = (min_l<min_r)?min_l:min_r;
      min_d = (min_d<dist_v)?min_d:dist_v;

      if (min_d>0x40) min_d=0x40;
      if (max_d>0xc0) max_d=0xc0;

      //  max_filter = 7/8*max_filter + 1/8 max_d*8
      max_filter *= 7;
      max_filter /= 8;
      max_filter += max_d;
      uint8_t max_df = max_filter/8;
      if (max_df>max_d) max_d = max_df;

      min_filter *= 7;
      min_filter /= 8;
      min_filter += min_d;
      uint8_t min_df = min_filter/8;
      if (min_df>min_d) min_d = min_df;

      uint8_t fdist_r  = calc_rel_dist(min_d, dist_r,  max_d);
      uint8_t fdist_vr = calc_rel_dist(min_d, dist_vr, max_d);
      uint8_t fdist_v  = calc_rel_dist(min_d, dist_v,  max_d);
      uint8_t fdist_vl = calc_rel_dist(min_d, dist_vl, max_d);
      uint8_t fdist_l  = calc_rel_dist(min_d, dist_l,  max_d);

      gfx_move(0, 45);
      print_hex(fdist_r);
      gfx_print_char(' ', 0);
      print_hex(fdist_vr);
      gfx_print_char(' ', 0);
      print_hex(fdist_v);
      gfx_print_char(' ', 0);
      print_hex(fdist_vl);
      gfx_print_char(' ', 0);
      print_hex(fdist_l);

      uint8_t ledr = 0x00;
      uint8_t ledg = 0x00;

      if (fdist_r <0x30) ledg|=0x80;
      if (fdist_vr<0x30) ledg|=0x40;
      if (fdist_v <0x30) ledg|=0x30;
      if (fdist_vl<0x30) ledg|=0x08;
      if (fdist_l <0x30) ledg|=0x04;

      if (fdist_r >0xa0) ledr|=0x80;
      if (fdist_vr>0xa0) ledr|=0x40;
      if (fdist_v >0xa0) ledr|=0x30;
      if (fdist_vl>0xa0) ledr|=0x08;
      if (fdist_l >0xa0) ledr|=0x04;

      // Verhalten auswählen
      switch(behaviour)
      {
        case BHV_STOP:
          if (counter==0)
            counter=10;
          counter--;
          if (counter==0)
            behaviour=BHV_OBSTACLE;
          break;
          
        case BHV_OBSTACLE:
          if (fdist_l<0x90)
            behaviour=BHV_OBSTACLE_L;
          else if (fdist_r<0x90)
            behaviour=BHV_OBSTACLE_R;
          break;
          
        case BHV_OBSTACLE_L:
          if (fdist_l>0xa0)
            behaviour=BHV_OBSTACLE;
          else if ((fdist_vl<0x90) && (fdist_v<0x90))
            behaviour=BHV_SPACE_L;
          break;
        
        case BHV_OBSTACLE_R:
          if (fdist_r>0xa0)
            behaviour=BHV_OBSTACLE;
          else if ((fdist_vr<0x90) && (fdist_v<0x90))
            behaviour=BHV_SPACE_R;
          break;
        
        case BHV_SPACE:
          if (fdist_vr<fdist_vl)
            behaviour=BHV_SPACE_R;
          else
            behaviour=BHV_SPACE_L;
          if (fdist_v<0x90)
            behaviour=BHV_GO;
          break;
          
        case BHV_SPACE_L:
          if (fdist_v<fdist_vl)
            behaviour=BHV_SPACE;
          if (fdist_v<0x90)
            behaviour=BHV_GO;
          break;
        
        case BHV_SPACE_R:
          if (fdist_v<fdist_vr)
            behaviour=BHV_SPACE;
          if (fdist_v<0x90)
            behaviour=BHV_GO;
          break;
        
        case BHV_GO:
          if (fdist_v>0xa0)
            behaviour=BHV_SPACE;
          break;
      }

      if (behaviour>=BHV_SPACE)
      {
        if (fdist_v >0xe0)
          behaviour=BHV_STOP;
        if (dist_vl>0xf0)
          behaviour=BHV_STOP;
        if (dist_vr>0xf0)
          behaviour=BHV_STOP;
        //if (dist_l >0xfa)
        //  behaviour=BHV_STOP;
        //if (dist_r >0xfa)
        //  behaviour=BHV_STOP;
      }

      uint16_t lval=((uint16_t)dist_vl+dist_l)/2;
      uint16_t rval=((uint16_t)dist_vr+dist_r)/2;
      
      // Verhalten anwenden
      switch(behaviour)
      {
        case BHV_STOP:
          speed_l=0, speed_r=0, mode=0;
          break;
          
        case BHV_OBSTACLE:
          speed_l=-10, speed_r=-10, mode=1;
          break;
          
        case BHV_OBSTACLE_L:
          speed_l=-10, speed_r=0, mode=1;
          break;
        
        case BHV_OBSTACLE_R:
          speed_l=0, speed_r=-10, mode=1;
          break;
        
        case BHV_SPACE:
          speed_l=40, speed_r=40, mode=1;
          if (lval>rval) speed_r-=15; // rechts mehr Platz
          if (rval>lval) speed_l-=15; // links mehr Platz
          break;
          
        case BHV_SPACE_L:
          speed_l=30, speed_r=50, mode=1;
          break;
        
        case BHV_SPACE_R:
          speed_l=50, speed_r=30, mode=1;
          break;
        
        case BHV_GO:
          speed_l=180, speed_r=180, mode=1;
          if (lval>rval) speed_r-=10; // rechts mehr Platz
          if (rval>lval) speed_l-=10; // links mehr Platz
          break;
      }
      
      IO_LEDS_RED_PORT=ledr;
      IO_LEDS_GREEN_PORT=ledg;

      if (speed_l+speed_r>100)
        set_output_bit(IO_LED_WHITE);
      else
        clear_output_bit(IO_LED_WHITE);
    
    }
    else
    {
      error_distco++;
      continue;
    }
    output_status('0'+behaviour);

    gfx_move(5, 55);
    gfx_print_text("r:", 1);
    print_hex(HIBYTE(speed_r));
    print_hex(LOBYTE(speed_r));
    gfx_print_text(" l:", 1);
    print_hex(HIBYTE(speed_l));
    print_hex(LOBYTE(speed_l));

    if (mode)
    {
      if (speed_l > tspeed_l+10)
        tspeed_l+=10;
      else if (speed_l < tspeed_l-10)
        tspeed_l-=10;
      else
        tspeed_l=speed_l;
      
      if (speed_r > tspeed_r+10)
        tspeed_r+=10;
      else if (speed_r < tspeed_r-10)
        tspeed_r-=10;
      else
        tspeed_r=speed_r;
        
      motco_setSpeed(tspeed_l, tspeed_r);
    }
    else
    {
      motco_stop();
    }
    
    // Analog values:
    int16_t value_fl = adc_read(AN_FLOOR_L);
    int16_t value_fr = adc_read(AN_FLOOR_R);
    int16_t value_ll = adc_read(AN_LINE_L);
    int16_t value_lr = adc_read(AN_LINE_R);
    set_output_bit(IO_LINE_FLOOR_EN);
    delay(1);
    value_fl -= adc_read(AN_FLOOR_L);
    value_fr -= adc_read(AN_FLOOR_R);
    value_ll -= adc_read(AN_LINE_L);
    value_lr -= adc_read(AN_LINE_R);
    clear_output_bit(IO_LINE_FLOOR_EN);

    if (value_fl<0) value_fl=0;
    if (value_fr<0) value_fr=0;
    if (value_ll<0) value_ll=0;
    if (value_lr<0) value_lr=0;
    if (value_fl>255) value_fl=255;
    if (value_fr>255) value_fr=255;
    if (value_ll>255) value_ll=255;
    if (value_lr>255) value_lr=255;
    
    gfx_move(100, 10);
    print_hex(value_fl);
    gfx_move(100, 20);
    print_hex(value_fr);
    gfx_move(100, 30);
    print_hex(value_ll);
    gfx_move(100, 40);
    print_hex(value_lr);
    
  }

}

int main(void) {
  sei(); // enable interrupts
  activate_output_bit(IO_LED_WHITE);
  activate_output_bit(IO_DISP_LIGHT);
  
  IO_LEDS_RED_DDR = IO_LEDS_RED_MASK;
  IO_LEDS_GREEN_DDR = IO_LEDS_GREEN_MASK;

/* <TEST> */
  activate_output_bit(IO_RESET_5);
  activate_output_bit(IO_RESET_3);
  clear_output_bit(IO_RESET_5);
  clear_output_bit(IO_RESET_3);
  delay(10);
  set_output_bit(IO_RESET_5);
  set_output_bit(IO_RESET_3);

  delay(100);

  if (display_init()!=0)
  {
    IO_LEDS_RED_PORT=0x00;
    IO_LEDS_GREEN_PORT=0x00;
    set_output_bit(IO_DISP_LIGHT);
    if (display_type==2)
      show_boot_message_gfx();
    delay(500);
    delay(500);
  }

  //nibo_test_prog();

  gfx_fill(0x00);
  gfx_move(5, 5);
  i2c_init();

  //nibo_test_com();
  nibo_test_nav();
  //nibo_test_behaviour();
  //nibo_test_odometry();
  //nibo_test_distance();
  
/* </TEST> */



  while (1)
  {
    set_output_bit(IO_LED_WHITE);
    delay(500);
    clear_output_bit(IO_LED_WHITE);
    
    delay(100);
    
    set_output_bit(IO_DISP_LIGHT);
    delay(1000);
    clear_output_bit(IO_DISP_LIGHT);
    
    delay(100);
    
    uint8_t x=4;
    for (int i=0; i<6; ++i)
    {
      IO_LEDS_RED_PORT=x;
      x<<=1;
      delay(80);
    }
    IO_LEDS_RED_PORT=0;
    delay(80);
    
    x=4;
    for (int i=0; i<6; ++i)
    {
      IO_LEDS_GREEN_PORT=x;
      x<<=1;
      delay(80);
    }
    IO_LEDS_GREEN_PORT=0;
    delay(100);
  }
}

void show_boot_message_gfx(void) {
  display_init();
  display_wait(0x00);
  display_write(0x00, 0x3f);
  display_wait(0x00);
  display_write(0x00, 0xc0);
  display_wait(0x00);
  display_write(0x00, 0x40);
  display_wait(0x00);
  display_write(0x00, 0xb8);

  display_wait(0x01);
  display_write(0x01, 0x3f);
  display_wait(0x01);
  display_write(0x01, 0xc0);
  display_wait(0x01);
  display_write(0x01, 0x40);
  display_wait(0x01);
  display_write(0x01, 0xb8);

  gfx_fill(0x00);

  gfx_move(40, 0);
  gfx_print_text("nibo 1.06", 1);
  gfx_move(5, 12);
  gfx_print_text("(c) 2007 by nicai-systems", 1);

  uint8_t prop = 0;

  gfx_move(15, 24);
  gfx_print_text("0123456789:;<=>?", prop);
  gfx_move(15, 32);
  gfx_print_text("@ABCDEFGHIJKLMNO", prop);
  gfx_move(15, 40);
  gfx_print_text("PQRSTUVWXYZ[\\]^_", prop);
  gfx_move(15, 48);
  gfx_print_text("`abcdefghijklmno", prop);
  gfx_move(15, 56);
  gfx_print_text("pqrstuvwxyz{|}~", prop);
  gfx_print_char(127, prop);
}
