/*
*/

#include <nibo/niboconfig.h>            /* muss immer als erste Datei
                                           eingebunden werden */
#include <nibo/display.h>               /* stellt die grundlegenden Routinen
                                           zur Displayansteuerung zur
                                           Verfügung */
#include <nibo/gfx.h>                   /* stellt die Routinen zur Ansteuerung
                                           eines Grafikdisplays zur Verfügung */
#include <nibo/irco.h>
#include <nibo/delay.h>
#include <nibo/iodefs.h>
#include <nibo/adc.h>
#include <nibo/bot.h>
#include <avr/interrupt.h>
#include <nibo/i2cmaster.h>
#include <nibo/floor.h>
#include <stdio.h>


uint16_t value_off[4];
uint16_t value_on[4];

void measure_ground() {
  floor_disable_ir();
  delay(1);
  floor_update();
  value_off[0] = floor_r;
  value_off[1] = line_r;
  value_off[2] = line_l;
  value_off[3] = floor_r;

  floor_enable_ir();
  delay(1);
  floor_update();
  value_on[0] = floor_r;
  value_on[1] = line_r;
  value_on[2] = line_l;
  value_on[3] = floor_l;
  floor_disable_ir();
}


uint8_t diff_ground(int16_t val)
{
  if(val>255)
    val=255;
  if (val<0)
    val=0;
  return val;
}


int main()
{
  sei(); // enable interrupts
  bot_init();
  i2c_init();
  floor_init();
  display_init();                       /* Displayansteuerung initialisieren */
  gfx_init();                           /* Grafikdisplay initialisieren */

  gfx_move(62, 0);
  gfx_set_proportional(1);
  gfx_print_text("nibo test");
  gfx_set_proportional(0);

  gfx_move(5, 0);
  gfx_print_char('R');

  gfx_move(118, 0);
  gfx_print_char('L');

  delay(50);
  irco_startMeasure();

  while (1)
  {
    // IR Controller
    delay(10);

	char text[]="--  --  --  --  --";
    if (irco_update())
	{
	  sprintf(text, "%02x  %02x  %02x  %02x  %02x", 
	      (int)irco_distance[0], 
		  (int)irco_distance[1],
	      (int)irco_distance[2], 
		  (int)irco_distance[3],
	      (int)irco_distance[4]);
	}
    gfx_move(10, 55);
    gfx_print_text(text);

    // Bodensensoren
    measure_ground();

    sprintf(text, "%02x  %02x  %02x  %02x",
        (int)(value_on[0]>>2),	   
        (int)(value_on[1]>>2),	   
        (int)(value_on[2]>>2),	   
        (int)(value_on[3]>>2));	   
    gfx_move(22, 30);
    gfx_print_text(text);
    
    sprintf(text, "%02x  %02x  %02x  %02x",
        (int)(diff_ground(value_on[0]-value_off[0])),	   
        (int)(diff_ground(value_on[1]-value_off[1])),	   
        (int)(diff_ground(value_on[2]-value_off[2])),	   
        (int)(diff_ground(value_on[3]-value_off[3])));	   
    gfx_move(22, 40);
    gfx_print_text(text);

    // Spannung
    bot_update();
    float volt = 0.0166 * bot_supply - 1.19;
    sprintf(text, "%2.1fV ", (double)volt);
    gfx_move(25, 0);
    gfx_print_text(text);
    
  }

  return 0;                             /* Fertig.... */
}
