/*  BSD-License:

Copyright (c) 2007 by Nils Springob, nicai-systems, Germany

All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.
  * Neither the name nicai-systems nor the names of its contributors may be
    used to endorse or promote products derived from this software without
    specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <inttypes.h>
#include <stdio.h>

const uint8_t irco_hex[] PROGMEM =
#include "irco_hex.h"

const uint8_t motco_hex[] PROGMEM =
#include "motco_hex.h"


#include "nibo/niboconfig.h"

#include "nibo/iodefs.h"
#include "nibo/delay.h"
#include "nibo/display.h"
#include "nibo/gfx.h"
#include "nibo/i2cmaster.h"
#include "avrisp.h"
#include "nibo/motco.h"
#include "nibo/irco.h"
#include "nibo/floor.h"
#include "nibo/bot.h"
#include "nibo/adc.h"
#include "nibo/pwm.h"
#include "nibo/leds.h"

uint16_t value_off_ll;
uint16_t value_off_rl;
uint16_t value_off_lf;
uint16_t value_off_rf;
uint16_t value_on_ll;
uint16_t value_on_rl;
uint16_t value_on_lf;
uint16_t value_on_rf;

uint16_t old_rc5_cmd;
uint8_t sequence_state;

#define IC_FLASH_SIZE 4096

#define LOBYTE(x)        (uint8_t)((uint16_t)x)
#define HIBYTE(x)        (uint8_t)(((uint16_t)x)>>8)
#define MAKE_WORD(hi,lo) ((hi*0x100)+lo)

void start_rc_mode();

void print_hexdigit(uint8_t val)
{
  if (val<10)
    val+='0';
  else
    val+='a'-10;
  gfx_print_char(val);
}


void print_hex (uint8_t val)
{
  char c1=val/16;
  char c2=val%16;
  if (c1<10) c1+='0'; else c1+='a'-10;
  if (c2<10) c2+='0'; else c2+='a'-10;
  gfx_print_char(c1);
  gfx_print_char(c2);
}




void test_leds(void);
void test_bot(void);
void show_error(void);
void ic_init(void);
void ic3_spi_init(void);
void ic5_spi_init(void);
void show_boot_message_gfx(void);
uint8_t check_i2c_dev(uint8_t dev);

void ic3_check_flash(void);
void ic3_program_flash(void);
void ic5_check_flash(void);
void ic5_program_flash(void);


uint8_t ic3_status;
uint8_t ic5_status;

int main (void)
{
  sei(); // enable interrupts

  activate_output_bit(IO_LED_WHITE);
  activate_output_bit(IO_DISP_LIGHT);
  IO_LEDS_RED_DDR = IO_LEDS_RED_MASK;
  IO_LEDS_GREEN_DDR = IO_LEDS_GREEN_MASK;
  IO_LEDS_RED_PORT=0xff;
  IO_LEDS_GREEN_PORT=0xff;
  delay(50);

  if (display_init()!=0)
  {
    IO_LEDS_RED_PORT=0x00;
    IO_LEDS_GREEN_PORT=0x00;
    set_output_bit(IO_DISP_LIGHT);
    if (display_type==2)
      show_boot_message_gfx();
    delay(500);
    delay(500);
  }
  ic_init();
  ic5_spi_init();
  ic3_spi_init();

  gfx_draw_mode(GFX_DM_JAM2);

  gfx_move(15, 24);
  gfx_print_text("                ");
  gfx_move(15, 32);
  gfx_print_text("                ");
  gfx_move(15, 40);
  gfx_print_text("                ");
  gfx_move(15, 48);
  gfx_print_text("                ");
  gfx_move(15, 56);
  gfx_print_text("                ");


  ic_init();
  delay(100);
  if (check_i2c_dev(MOTCO_I2C_ID))
    ic3_status=2;

  ic_init();
  delay(100);
  if (check_i2c_dev(IRCO_I2C_ID))
    ic5_status=2;


  gfx_move(15, 24);
  gfx_print_char('0'+ic3_status);
  gfx_move(30, 24);
  gfx_print_char('0'+ic5_status);

  IO_LEDS_RED_PORT=0x00;
  IO_LEDS_GREEN_PORT=0x00;
  delay(50);
  if ((ic3_status==0) && (ic5_status==0))
    test_leds();

  if ((ic3_status==0) && (ic5_status!=0))
  {
    IO_LEDS_GREEN_PORT = 0;
    IO_LEDS_RED_PORT = 0x1c;
    delay(1000);
    ic5_check_flash();

    gfx_move(30, 34);
    gfx_set_proportional(1);
    gfx_print_text("IC5: ");
    gfx_set_proportional(0);
    gfx_print_char('0'+ic5_status);
    

    if (ic5_status==1)
    {
      // program device
      ic5_program_flash();
      while(1)
      {
        IO_LEDS_GREEN_PORT = IO_LEDS_GREEN_MASK;
        IO_LEDS_RED_PORT = 0;
        delay(950);
        IO_LEDS_GREEN_PORT = 0;
        delay(50);
      }
    }
    
    if (ic5_status==2)
    {
      // run device test
    }
    
  }
    
  if ((ic3_status!=0) && (ic5_status==0))
  {
    IO_LEDS_GREEN_PORT = 0;
    IO_LEDS_RED_PORT = 0xe0;
    delay(1000);
    ic3_check_flash();
    
    gfx_move(30, 54);
    gfx_set_proportional(1);
    gfx_print_text("IC3: ");
    gfx_set_proportional(0);
    gfx_print_char('0'+ic3_status);

    if (ic3_status==1)
    {
      ic3_program_flash();
      while(1)
      {
        IO_LEDS_GREEN_PORT = IO_LEDS_GREEN_MASK;
        IO_LEDS_RED_PORT = 0;
        delay(950);
        IO_LEDS_GREEN_PORT = 0;
        delay(50);
      }
    }
    
    if (ic3_status==2)
    {
      // run device test
    }
  
  }

  if ((ic3_status==2) && (ic5_status==2))
  {
    test_bot();
  }
    
  show_error();
}

void test_leds()
{
  activate_output_bit(IO_LED_WHITE);
  activate_output_bit(IO_DISP_LIGHT);
  IO_LEDS_RED_DDR = IO_LEDS_RED_MASK;
  IO_LEDS_GREEN_DDR = IO_LEDS_GREEN_MASK;
  
  delay(100);

  while (1)
  {
    delay(100);
    uint8_t x=4;
    for (int i=0; i<6; ++i)
    {
      IO_LEDS_RED_PORT=x;
      x<<=1;
      delay(80);
    }
    IO_LEDS_RED_PORT=0;
    delay(80);
    
    x=4;
    for (int i=0; i<6; ++i)
    {
      IO_LEDS_GREEN_PORT=x;
      x<<=1;
      delay(80);
    }
    IO_LEDS_GREEN_PORT=0;
    delay(100);
    
    set_output_bit(IO_LED_WHITE);
    set_output_bit(IO_DISP_LIGHT);
    delay(500);
    clear_output_bit(IO_LED_WHITE);
    clear_output_bit(IO_DISP_LIGHT);
    
    delay(100);
  }
}


void ic_init()
{
  activate_output_bit(IO_RESET_5);
  activate_output_bit(IO_RESET_3);
  activate_output_bit(IO_RESET_A);
  activate_output_bit(IO_RESET_B);

  clear_output_bit(IO_RESET_5);
  clear_output_bit(IO_RESET_3);
  clear_output_bit(IO_RESET_A);
  clear_output_bit(IO_RESET_B);
  
  delay(10);
  i2c_init();

  set_output_bit(IO_RESET_5);
  set_output_bit(IO_RESET_3);
  set_output_bit(IO_RESET_A);
  set_output_bit(IO_RESET_B);
  delay(20);
}


uint8_t ic_check_flash(uint32_t ptr)
{
  uint8_t result=0;
  ic_init();
  enterProgrammingMode(_BV(IO_RESET_3_BIT) | _BV(IO_RESET_5_BIT));
  setAddress(0);
  for (uint16_t addr=0; addr<IC_FLASH_SIZE; addr+=2)
  {
    uint16_t data1 = readProgramMemory();
    uint16_t data2 = pgm_read_word_far(ptr+addr);
    if (data1!=data2)
    {
      result=1;
      gfx_move(0, 44);
      gfx_set_proportional(1);
      gfx_print_text("ERR addr=");
      gfx_set_proportional(0);
      print_hexdigit((addr>>9)&0x0f);
      print_hexdigit((addr>>5)&0x0f);
      print_hexdigit((addr>>1)&0x0f);
      gfx_move(0, 54);
      gfx_set_proportional(1);
      gfx_print_text("r=");
      gfx_set_proportional(0);
      print_hexdigit((data1>>12)&0x0f);
      print_hexdigit((data1>>8)&0x0f);
      print_hexdigit((data1>>4)&0x0f);
      print_hexdigit(data1&0x0f);
      gfx_set_proportional(1);
      gfx_print_text(" w=");
      gfx_set_proportional(0);
      print_hexdigit((data2>>12)&0x0f);
      print_hexdigit((data2>>8)&0x0f);
      print_hexdigit((data2>>4)&0x0f);
      print_hexdigit(data2&0x0f);
      //delay(1000);
    }
  }
  leaveProgrammingMode(_BV(IO_RESET_3_BIT) | _BV(IO_RESET_5_BIT));
  delay(100);
  return result;
}


//
// IC3 - Motco
//

void ic3_spi_init(void)
{
  if (enterProgrammingMode(_BV(IO_RESET_3_BIT)))
    ic3_status=1;
  leaveProgrammingMode(_BV(IO_RESET_3_BIT));
  delay(100);
}

void ic3_check_flash(void)
{
  if (ic_check_flash((uint32_t) motco_hex))
    ic3_status=1;
  else
    ic3_status=2;
}

void ic3_program_flash(void)
{
  ic_init();
  enterProgrammingMode(_BV(IO_RESET_3_BIT) | _BV(IO_RESET_5_BIT));
  chipErase();
  writeLoFuse(0xe2);
  setAddress(0);
  uint32_t ptr = (uint32_t) motco_hex;
  for (uint16_t addr=0; addr<sizeof(motco_hex); addr+=2)
  {
    uint16_t data = pgm_read_word_far(ptr+addr);
    writeProgramMemory(data);
    if (((addr>>1)&0x1f)==0x1f)
      issuePageWrite();
  }
  ic3_status=2;
  leaveProgrammingMode(_BV(IO_RESET_3_BIT) | _BV(IO_RESET_5_BIT));
  delay(100);
}


//
// IC5 - Distco
//

void ic5_spi_init(void)
{
  if (enterProgrammingMode(_BV(IO_RESET_5_BIT)))
    ic5_status=1;
  leaveProgrammingMode(_BV(IO_RESET_5_BIT));
  delay(100);
}

void ic5_check_flash(void)
{
  if (ic_check_flash((uint32_t) irco_hex))
    ic5_status=1;
  else
    ic5_status=2;
}

void ic5_program_flash(void)
{
  ic_init();
  enterProgrammingMode(_BV(IO_RESET_3_BIT) | _BV(IO_RESET_5_BIT));
  chipErase();
  writeLoFuse(0xe2);
  setAddress(0);
  uint32_t ptr = (uint32_t) irco_hex;
  for (uint16_t addr=0; addr<sizeof(irco_hex); addr+=2)
  {
    uint16_t data = pgm_read_word_far(ptr+addr);
    writeProgramMemory(data);
    if (((addr>>1)&0x001f)==0x001f)
      issuePageWrite();
  }
  ic5_status=2;
  leaveProgrammingMode(_BV(IO_RESET_3_BIT) | _BV(IO_RESET_5_BIT));
  delay(100);
}


void show_boot_message_gfx(void) {
  display_init();
  display_wait(0x00);
  display_write(0x00, 0x3f);
  display_wait(0x00);
  display_write(0x00, 0xc0);
  display_wait(0x00);
  display_write(0x00, 0x40);
  display_wait(0x00);
  display_write(0x00, 0xb8);

  display_wait(0x01);
  display_write(0x01, 0x3f);
  display_wait(0x01);
  display_write(0x01, 0xc0);
  display_wait(0x01);
  display_write(0x01, 0x40);
  display_wait(0x01);
  display_write(0x01, 0xb8);

  gfx_fill(0x00);

  gfx_move(40, 0);
  gfx_set_proportional(1);
  gfx_print_text("nibo 1.06");
  gfx_move(5, 12);
  gfx_print_text("(c) 2007 by nicai-systems");

  gfx_set_proportional(0);

  gfx_move(15, 24);
  gfx_print_text("0123456789:;<=>?");
  gfx_move(15, 32);
  gfx_print_text("@ABCDEFGHIJKLMNO");
  gfx_move(15, 40);
  gfx_print_text("PQRSTUVWXYZ[\\]^_");
  gfx_move(15, 48);
  gfx_print_text("`abcdefghijklmno");
  gfx_move(15, 56);
  gfx_print_text("pqrstuvwxyz{|}~");
  gfx_print_char(127);
  
}


uint8_t check_i2c_dev(uint8_t dev)
{
  i2c_size = 2;
  i2c_buf[0] = I2C_RX(dev);

  if (!i2c_start_transmission())
    return 0;
  if (!i2c_wait_transmission())
    return 0;
  if (i2c_status()!=I2C_SUCCESS)
    return 0;
  return 1;
}


void show_error(void)
{
  activate_output_bit(IO_LED_WHITE);
  activate_output_bit(IO_DISP_LIGHT);
  IO_LEDS_RED_DDR = IO_LEDS_RED_MASK;
  IO_LEDS_GREEN_DDR = IO_LEDS_GREEN_MASK;

  while (1)
  {
    delay(50);
    IO_LEDS_RED_PORT = IO_LEDS_RED_MASK;
    delay(50);
    IO_LEDS_RED_PORT = 0;
    delay(50);
    IO_LEDS_RED_PORT = IO_LEDS_RED_MASK;
    delay(50);
    IO_LEDS_RED_PORT = 0;
    delay(50);
    IO_LEDS_RED_PORT = IO_LEDS_RED_MASK;
    delay(50);
    IO_LEDS_RED_PORT = 0;
    delay(50);
    IO_LEDS_RED_PORT = IO_LEDS_RED_MASK;
    delay(50);
    IO_LEDS_RED_PORT = 0;
    delay(600);
  }

}

void measure_ground() {
    clear_output_bit(IO_LINE_FLOOR_EN);
    delay_us(25);
    value_off_ll = 1023-adc_read(AN_LINE_L);
    value_off_rl = 1023-adc_read(AN_LINE_R);
    value_off_lf = 1023-adc_read(AN_FLOOR_L);
    value_off_rf = 1023-adc_read(AN_FLOOR_R);
    value_off_rf += 1023-adc_read(AN_FLOOR_R);
    value_off_lf += 1023-adc_read(AN_FLOOR_L);
    value_off_rl += 1023-adc_read(AN_LINE_R);
    value_off_ll += 1023-adc_read(AN_LINE_L);
    set_output_bit(IO_LINE_FLOOR_EN);
    delay_us(25);
    value_on_ll = 1023-adc_read(AN_LINE_L);
    value_on_rl = 1023-adc_read(AN_LINE_R);
    value_on_lf = 1023-adc_read(AN_FLOOR_L);
    value_on_rf = 1023-adc_read(AN_FLOOR_R);
    value_on_rf += 1023-adc_read(AN_FLOOR_R);
    value_on_lf += 1023-adc_read(AN_FLOOR_L);
    value_on_rl += 1023-adc_read(AN_LINE_R);
    value_on_ll += 1023-adc_read(AN_LINE_L);
    clear_output_bit(IO_LINE_FLOOR_EN);
}

uint8_t check_ground(int16_t val)
{
  return val<200;
}

uint8_t diff_ground(int16_t val)
{
  if(val>255)
    val=255;
  if (val<0)
    val=0;
  return val;
}


void test_bot(void)
{
  pwm_init();
  //display_clear();
  gfx_fill(0);
  gfx_move(62, 0);
  gfx_set_proportional(1);
  gfx_print_text("nibo test");
  gfx_set_proportional(0);

  gfx_move(5, 0);
  gfx_print_char('R');

  gfx_move(118, 0);
  gfx_print_char('L');

  uint8_t c='0';
  while (1)
  {

    // IR Controller
    delay(50);
    irco_startMeasure();
    delay(10);
    
    if (!irco_update())
      continue;
    
    gfx_move(10, 55);
    print_hex(irco_distance[0]);
    gfx_print_char(' ');
    gfx_print_char(' ');
    print_hex(irco_distance[1]);
    gfx_print_char(' ');
    gfx_print_char(' ');
    print_hex(irco_distance[2]);
    gfx_print_char(' ');
    gfx_print_char(' ');
    print_hex(irco_distance[3]);
    gfx_print_char(' ');
    gfx_print_char(' ');
    print_hex(irco_distance[4]);
    
    gfx_move(42, 16);
    gfx_set_proportional(1);
    gfx_print_text("RC5: ");
    gfx_set_proportional(0);
    print_hex(HIBYTE(irco_rc5_cmd));
    print_hex(LOBYTE(irco_rc5_cmd));

    if (irco_rc5_cmd!=old_rc5_cmd) {
      old_rc5_cmd = irco_rc5_cmd;
      if ((HIBYTE(irco_rc5_cmd)&0x07)==0x02) {
        switch(sequence_state) {
          case 0: sequence_state = (LOBYTE(irco_rc5_cmd)==0x81)?1:0; break;
          case 1: sequence_state = (LOBYTE(irco_rc5_cmd)==0x83)?2:0; break;
          case 2: sequence_state = (LOBYTE(irco_rc5_cmd)==0x88)?3:0; break;
          default: sequence_state = 0;
        }
        if (sequence_state==3) {
          start_rc_mode();
        }
      } else {
        sequence_state = 0;
      }
    }

    uint8_t ledr = 0x00;
    uint8_t ledg = 0x00;

    if (irco_distance[0]<0x30) ledg|=0x80;
    if (irco_distance[1]<0x30) ledg|=0x40;
    if (irco_distance[2]<0x30) ledg|=0x30;
    if (irco_distance[3]<0x30) ledg|=0x08;
    if (irco_distance[4]<0x30) ledg|=0x04;

    if (irco_distance[0]>0xa0) ledr|=0x80;
    if (irco_distance[1]>0xa0) ledr|=0x40;
    if (irco_distance[2]>0xa0) ledr|=0x30;
    if (irco_distance[3]>0xa0) ledr|=0x08;
    if (irco_distance[4]>0xa0) ledr|=0x04;

    uint16_t sum = irco_distance[0]+irco_distance[1]+irco_distance[2]+irco_distance[3]+irco_distance[4];
    uint8_t dmax = (2*sum)/5;
    uint8_t dmin = sum/(2*5);

    if (irco_distance[0]<dmin) ledg|=0x80;
    if (irco_distance[1]<dmin) ledg|=0x40;
    if (irco_distance[2]<dmin) ledg|=0x30;
    if (irco_distance[3]<dmin) ledg|=0x08;
    if (irco_distance[4]<dmin) ledg|=0x04;

    if (irco_distance[0]>dmax) ledr|=0x80;
    if (irco_distance[1]>dmax) ledr|=0x40;
    if (irco_distance[2]>dmax) ledr|=0x30;
    if (irco_distance[3]>dmax) ledr|=0x08;
    if (irco_distance[4]>dmax) ledr|=0x04;

    IO_LEDS_RED_PORT=ledr;
    IO_LEDS_GREEN_PORT=ledg;

    // Bodensensoren
    measure_ground();
    if (check_ground(value_on_ll-value_off_ll) &&
        check_ground(value_on_rl-value_off_rl) &&
        check_ground(value_on_lf-value_off_lf) &&
        check_ground(value_on_rf-value_off_rf))
    {
      leds_set_headlights(1024);
    }
    else
    {
      leds_set_headlights(0);
    }

    gfx_move(22, 30);
    print_hex(value_on_rf>>2);
    gfx_print_char(' ');
    gfx_print_char(' ');
    print_hex(value_on_rl>>2);
    gfx_print_char(' ');
    gfx_print_char(' ');
    print_hex(value_on_ll>>2);
    gfx_print_char(' ');
    gfx_print_char(' ');
    print_hex(value_on_lf>>2);
    
    gfx_move(22, 40);
    print_hex(diff_ground(value_on_rf-value_off_rf));
    gfx_print_char(' ');
    gfx_print_char(' ');
    print_hex(diff_ground(value_on_rl-value_off_rl));
    gfx_print_char(' ');
    gfx_print_char(' ');
    print_hex(diff_ground(value_on_ll-value_off_ll));
    gfx_print_char(' ');
    gfx_print_char(' ');
    print_hex(diff_ground(value_on_lf-value_off_lf));

    // Spannung

    bot_update();
    float volt = 0.0166 * bot_supply - 1.19;
    char str[10];
    sprintf(str, "%2.1fV", volt);

    gfx_move(25, 0);
    gfx_print_text(str);

    // MOTOREN
    
    motco_update();

    gfx_move(100, 16);
    print_hex(HIBYTE(motco_ticks_l));
    print_hex(LOBYTE(motco_ticks_l));
    gfx_move(5, 16);
    print_hex(HIBYTE(motco_ticks_r));
    print_hex(LOBYTE(motco_ticks_r));

    motco_setSpeedParameters(1, 1, 1);
    delay(10);

    uint8_t phase = (pwm_timer_sec/5)%4; // jeweils 5 Sekunden lang
/*
    if (phase==0)
      motco_setSpeed(0, 0);
    else if (phase==1)
      motco_setSpeed(+36, +36); // +20cm/s
    else if (phase==2)
      motco_setSpeed(0, 0);
    else if (phase==3)
      motco_setSpeed(-36, -36); // -20cm/s
*/
    if (phase==0)
      motco_setPosition(200, 200, 36);
    else if (phase==1)
      motco_setPosition(200, 200, 36);
    else if (phase==2)
      motco_setPosition(0, 0, 36);
    else if (phase==3)
      motco_setPosition(0, 0, 36);

  }

  
}

