#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <inttypes.h>
#include <stdio.h>

#include "nibo/niboconfig.h"

#include "nibo/iodefs.h"
#include "nibo/delay.h"
#include "nibo/display.h"
#include "nibo/gfx.h"
#include "nibo/i2cmaster.h"
#include "avrisp.h"
#include "nibo/motco.h"
#include "nibo/irco.h"
#include "nibo/floor.h"
#include "nibo/bot.h"
#include "nibo/adc.h"
#include "nibo/pwm.h"
#include "nibo/leds.h"

#define LOBYTE(x)        (uint8_t)((uint16_t)x)
#define HIBYTE(x)        (uint8_t)(((uint16_t)x)>>8)
#define MAKE_WORD(hi,lo) ((hi*0x100)+lo)

uint16_t value_off_ll;
uint16_t value_off_rl;
uint16_t value_off_lf;
uint16_t value_off_rf;
uint16_t value_on_ll;
uint16_t value_on_rl;
uint16_t value_on_lf;
uint16_t value_on_rf;

int16_t speed_l;
int16_t speed_r;


uint16_t old_rc5_cmd;
uint8_t sequence_state;

void print_hex (uint8_t val);


void start_rc_mode() {
  gfx_fill(0);
  gfx_move(62, 0);
  gfx_set_proportional(1);
  gfx_print_text("nibo rc");
  gfx_set_proportional(0);

  gfx_move(5, 0);
  gfx_print_char('R');

  gfx_move(118, 0);
  gfx_print_char('L');

  uint8_t c='0';
  while (1)
  {

    // IR Controller
    delay(50);
    irco_startMeasure();
    delay(10);
    
    if (!irco_update())
      continue;
    
    gfx_move(10, 55);
    print_hex(irco_distance[0]);
    gfx_print_char(' ');
    gfx_print_char(' ');
    print_hex(irco_distance[1]);
    gfx_print_char(' ');
    gfx_print_char(' ');
    print_hex(irco_distance[2]);
    gfx_print_char(' ');
    gfx_print_char(' ');
    print_hex(irco_distance[3]);
    gfx_print_char(' ');
    gfx_print_char(' ');
    print_hex(irco_distance[4]);
    
    gfx_move(42, 16);
    gfx_set_proportional(1);
    gfx_print_text("RC5: ");
    gfx_set_proportional(0);
    print_hex(HIBYTE(irco_rc5_cmd));
    print_hex(LOBYTE(irco_rc5_cmd));

    if (irco_rc5_cmd!=old_rc5_cmd) {
      old_rc5_cmd = irco_rc5_cmd;
      if ((HIBYTE(irco_rc5_cmd)&0x07)==0x02) {
        switch(LOBYTE(irco_rc5_cmd)) {
          case 0x81: speed_l++; break;
          case 0x87: speed_l--; break;
          case 0x83: speed_r++; break;
          case 0x89: speed_r--; break;

          case 0x90:
          case 0x82: speed_l++; speed_r++; break;
          case 0x91:
          case 0x88: speed_l--; speed_r--; break;

          case 0x96:
          case 0x84: speed_l--; speed_r++; break;
          case 0x95:
          case 0x86: speed_l++; speed_r--; break;

          case 0x97:
          case 0x85: speed_l=0; speed_r=0; break;
          case 0x8c:
          case 0x80: speed_l=0; speed_r=0; break;
        }
      }
    }

    uint8_t ledr = 0x00;
    uint8_t ledg = 0x00;

    if (irco_distance[0]<0x30) ledg|=0x80;
    if (irco_distance[1]<0x30) ledg|=0x40;
    if (irco_distance[2]<0x30) ledg|=0x30;
    if (irco_distance[3]<0x30) ledg|=0x08;
    if (irco_distance[4]<0x30) ledg|=0x04;

    if (irco_distance[0]>0xa0) ledr|=0x80;
    if (irco_distance[1]>0xa0) ledr|=0x40;
    if (irco_distance[2]>0xa0) ledr|=0x30;
    if (irco_distance[3]>0xa0) ledr|=0x08;
    if (irco_distance[4]>0xa0) ledr|=0x04;

    uint16_t sum = irco_distance[0]+irco_distance[1]+irco_distance[2]+irco_distance[3]+irco_distance[4];
    uint8_t dmax = (2*sum)/5;
    uint8_t dmin = sum/(2*5);

    if (irco_distance[0]<dmin) ledg|=0x80;
    if (irco_distance[1]<dmin) ledg|=0x40;
    if (irco_distance[2]<dmin) ledg|=0x30;
    if (irco_distance[3]<dmin) ledg|=0x08;
    if (irco_distance[4]<dmin) ledg|=0x04;

    if (irco_distance[0]>dmax) ledr|=0x80;
    if (irco_distance[1]>dmax) ledr|=0x40;
    if (irco_distance[2]>dmax) ledr|=0x30;
    if (irco_distance[3]>dmax) ledr|=0x08;
    if (irco_distance[4]>dmax) ledr|=0x04;

    IO_LEDS_RED_PORT=ledr;
    IO_LEDS_GREEN_PORT=ledg;

    // Bodensensoren
    measure_ground();
    if (check_ground(value_on_ll-value_off_ll) &&
        check_ground(value_on_rl-value_off_rl) &&
        check_ground(value_on_lf-value_off_lf) &&
        check_ground(value_on_rf-value_off_rf))
    {
      leds_set_headlights(1024);
    }
    else
    {
      leds_set_headlights(0);
    }

    gfx_move(22, 30);
    print_hex(value_on_rf>>2);
    gfx_print_char(' ');
    gfx_print_char(' ');
    print_hex(value_on_rl>>2);
    gfx_print_char(' ');
    gfx_print_char(' ');
    print_hex(value_on_ll>>2);
    gfx_print_char(' ');
    gfx_print_char(' ');
    print_hex(value_on_lf>>2);
    
    gfx_move(22, 40);
    print_hex(diff_ground(value_on_rf-value_off_rf));
    gfx_print_char(' ');
    gfx_print_char(' ');
    print_hex(diff_ground(value_on_rl-value_off_rl));
    gfx_print_char(' ');
    gfx_print_char(' ');
    print_hex(diff_ground(value_on_ll-value_off_ll));
    gfx_print_char(' ');
    gfx_print_char(' ');
    print_hex(diff_ground(value_on_lf-value_off_lf));

    // Spannung

    bot_update();
    float volt = 0.0166 * bot_supply - 1.19;
    char str[10];
    sprintf(str, "%2.1fV", volt);

    gfx_move(25, 0);
    gfx_print_text(str);

    // MOTOREN
    
    motco_update();

    gfx_move(100, 16);
    print_hex(HIBYTE(motco_ticks_l));
    print_hex(LOBYTE(motco_ticks_l));
    gfx_move(5, 16);
    print_hex(HIBYTE(motco_ticks_r));
    print_hex(LOBYTE(motco_ticks_r));

    motco_setSpeedParameters(1, 1, 1);
    delay(10);

    motco_setSpeed(speed_l, speed_r);

  }

  

}
