/*  BSD-License:

Copyright (c) 2007 by Nils Springob, nicai-systems, Germany

All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.
  * Neither the name nicai-systems nor the names of its contributors may be
    used to endorse or promote products derived from this software without
    specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

#include "iodefs.h"
#include "distance.h"
#include <util/delay.h>
#include <avr/interrupt.h>

#define ANALOG_MASK 0x8f

int illumination;

void distance_init(void)
{
  PORTA &= ~ANALOG_MASK;
  clear_output_bit(IO_IR_TX);
  activate_output_bit(IO_IR_TX);
}


unsigned int measure_b(uint8_t portval, uint8_t channel)
{
    unsigned int value=0;
    cli(); // disable interrupts
    PORTA &= ~ANALOG_MASK;
    DDRA &= ~ANALOG_MASK;
    DDRA |= 1 << channel;
    sei(); // enable interrupts
    _delay_us(1);
    cli(); // disable interrupts
    DDRA &= ~ANALOG_MASK;
    sei(); // enable interrupts
    if (portval)
      set_output_bit(IO_IR_TX);
    _delay_ms(0.100);

    ADCSRA = _BV(ADPS2)  // prescale faktor = 128 ADC laeuft
           | _BV(ADPS1)  // mit 8MHz/ 64 = 125kHz
           /*| _BV(ADPS0)*/
           | _BV(ADEN)   // ADC an
           | _BV(ADSC);  // Beginne mit der Konvertierung
    loop_until_bit_is_clear(ADCSRA, ADSC);

    uint8_t i;
    for (i=0; i<8; ++i)
    {
      ADCSRA |= _BV(ADEN) | _BV(ADSC);
      loop_until_bit_is_clear(ADCSRA, ADSC);
      value += ADC;
    }
    clear_output_bit(IO_IR_TX);
    return value;
}


int measure_a(uint8_t ch)
{
    int val0 = measure_b(0, ch);
    int val1 = measure_b(1, ch);
    illumination += val0/8;
    return (val1-val0)/2;
}


uint8_t encode_val(int16_t val) {
  uint8_t res=0;
  if (val<=0) {
    return 0;
  }
  val+=0x10;

  while(val>=0x20) {
    val /= 2;
    res+=0x10;
  }
  val&=0x0f;
  res+=val;
  return res;
}

uint8_t distance_measure(uint8_t ch)
{
  ADMUX = ch;
  int value=-24;
  uint8_t i;
  illumination = 0;
  for (i=0; i<8; ++i) {
    value += measure_a(ch);
  }
  return encode_val(value);
  //return encode_val(illumination);
  /*
  value /= 16;
  if (value<0)
    value=0;
  if (value>=0x80)
    value=0x40+(value/2);
  if (value>=0xff)
    value=0xff;
  return value;
  */
}
