/*  BSD-License:

Copyright (c) 2007 by Nils Springob, nicai-systems, Germany

All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.
  * Neither the name nicai-systems nor the names of its contributors may be
    used to endorse or promote products derived from this software without
    specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

#include <avr/io.h>

/*! IR Receiver */
#define IO_IR_RX       IO_IR_RX
#define IO_IR_RX_PORT  PORTB
#define IO_IR_RX_PIN   PINB
#define IO_IR_RX_BIT   2
#define IO_IR_RX_DDR   DDRB

/*! IR Transmitter/Emitter */
#define IO_IR_TX       IO_IR_TX
#define IO_IR_TX_PORT  PORTB
#define IO_IR_TX_BIT   0
#define IO_IR_TX_DDR   DDRB

/*! I2C bus clock */
#define IO_I2C_SCL       IO_I2C_SCL
#define IO_I2C_SCL_PORT  PORTA
#define IO_I2C_SCL_PIN   PINA
#define IO_I2C_SCL_BIT   4
#define IO_I2C_SCL_DDR   DDRA

/*! I2C bus data */
#define IO_I2C_SDA       IO_I2C_SDA
#define IO_I2C_SDA_PORT  PORTA
#define IO_I2C_SDA_PIN   PINA
#define IO_I2C_SDA_BIT   6
#define IO_I2C_SDA_DDR   DDRA


/*! I2C driver configuration */
#define I2C_ADDRESS 42
#define I2C_RX_BUF_SIZE 9
#define I2C_TX_BUF_SIZE 9

// I2C timer clock = 1MHz (factor 1/8), clock time = 1us
#define I2C_TIMER_TCCRA        TCCR0A
#define I2C_TIMER_TCCRA_INIT   0x00
#define I2C_TIMER_TCCRB        TCCR0B
#define I2C_TIMER_TCCRB_INIT   _BV(CS01)
#define I2C_TIMER_TIMSK        TIMSK0
#define I2C_TIMER_TIMSK_INIT   0x00
#define I2C_TIMER_TIMSK_ACTIVE _BV(OCIE0A)
#define I2C_TIMER_TIFR         TIFR0

#define I2C_TIMER_TCNT         TCNT0
#define I2C_TIMER_OCRA         OCR0A
#define I2C_TIMER_TIMEOUT      200
#define I2C_TIMER_OCRA_vect    TIM0_COMPA_vect


/* Hilfs-Macro */
#define _PPCAT(a,b) a##b

/* Macro-Funktionen für IO-Bits */
#define set_output_bit(NAME) _PPCAT(NAME,_PORT)|=_BV(_PPCAT(NAME,_BIT))
#define clear_output_bit(NAME) _PPCAT(NAME,_PORT)&=~(unsigned char)_BV(_PPCAT(NAME,_BIT))
#define set_output_bitval(NAME, VAL) _PPCAT(NAME,_PORT)=(_PPCAT(NAME,_PORT)&~(unsigned char)_BV(_PPCAT(NAME,_BIT)))|((VAL)?_BV(_PPCAT(NAME,_BIT)):0)

#define activate_output_bit(NAME) _PPCAT(NAME,_DDR)|=_BV(_PPCAT(NAME,_BIT))
#define deactivate_output_bit(NAME) _PPCAT(NAME,_DDR)&=~(unsigned char)_BV(_PPCAT(NAME,_BIT))

#define get_input_bit(NAME) (_PPCAT(NAME,_PIN)&_BV(_PPCAT(NAME,_BIT)))

