/*  BSD-License:

Copyright (c) 2007 by Nils Springob, nicai-systems, Germany

All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.
  * Neither the name nicai-systems nor the names of its contributors may be
    used to endorse or promote products derived from this software without
    specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

/*! @file       pid.h
 * @brief       Modul zur PID-Regelung der Motoren
 * @author      Nils Springob (nils@nicai-systems.de)
 * @date        2007-01-05
*/

#ifndef _CTBOT_PID_
#define _CTBOT_PID_

enum {
  PID_MODE_OFF = 0x00,
  PID_MODE_SPEED = 0x03,
  PID_MODE_POS = 0x04,
  PID_MODE_STOP_IMM = 0x05
};

// Soll Werte für DIP Controller:
extern int16_t pid_left_value;
extern int16_t pid_right_value;
extern int16_t pid_position_speed;

// Ist Werte der PID Controller:
extern int16_t pid_left_speed;
extern int16_t pid_right_speed;
extern int16_t pid_left_speed2;
extern int16_t pid_right_speed2;

// Konstanten
extern volatile int8_t pid_ki;
extern volatile int8_t pid_kd;
extern volatile int8_t pid_kp;

// Konfiguration:
extern uint16_t pid_timeout;
extern uint8_t pid_mode;
#define PID_ENABLED 0x01

// These values will be incremented by wheel encoders
extern int8_t pid_left_cnt;
extern int8_t pid_right_cnt;

// This value will try to keep direction
extern int16_t pid_dir_sum;


void pid_change_mode(uint8_t new_mode);
void pid_reset(void);
void pid_left_timer_int(void);
void pid_right_timer_int(void);

#endif
