/*  BSD-License:

Copyright (c) 2007 by Nils Springob, nicai-systems, Germany

All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.
  * Neither the name nicai-systems nor the names of its contributors may be
    used to endorse or promote products derived from this software without
    specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

#include "iodefs.h"
#include "distance.h"
#include <util/delay.h>
#include <avr/interrupt.h>
#include "utils.h"
#include "ir_rc5.h"

#define ANALOG_MASK 0xdf


#define analog_enable() ADCSRA |= _BV(ADEN)
#define analog_restart() ADCSRA |= _BV(ADSC)

#define analog_select_mux_vcc(ch) ADMUX = ( _BV(REFS0) | (ch))
//#define analog_select_mux_bg(ch) ADMUX = ( _BV(REFS1) | _BV(REFS0) | (ch))

int16_t distance_value;
int16_t distance_values[DISTANCE_CHANNELS];
uint8_t distance_cnt;

int16_t analog_current_left;
int16_t analog_current_right;


uint8_t analog_ignore;
uint8_t analog_channel;

int illumination;

void distance_init(void) {
  // disable digital inputs
  DIDR0 = 0x3f & ANALOG_MASK;
  // disable digital outputs
  DDRC &= ~ANALOG_MASK;
  // setup output value for pin-discharge
  PORTC &= ~ANALOG_MASK;
  
  clear_output_bit(IO_IR_TX);
  activate_output_bit(IO_IR_TX);
  
  ADCSRA = _BV(ADPS2)  // prescale faktor = 128
         | _BV(ADPS1)  // ADC laeuft mit 16MHz / 128 = 125kHz
         | _BV(ADPS0)
         | _BV(ADIE)    // Interrupt an
         | _BV(ADEN);   // ADC an

  analog_ignore = 1;
  distance_value = 0;
  distance_cnt = 0;
  analog_channel = 0;
  analog_select_mux_vcc(0);
  analog_restart();
}


ISR (ADC_vect) {
  if (analog_ignore) {
    analog_ignore--;
    analog_restart();
    return;
  }
  
  schedule_irq();

  if (analog_channel<5) {
    // distance measure
    if (distance_cnt&0x08) {
      distance_value += ADC;
    } else {
      distance_value -= ADC;
    }
 
    // next cycle
    distance_cnt++;

    if ((distance_cnt&0x0007) != 0x00) {
      // setup unchanged, immediatly start next cycle!
      analog_restart();
      return;
    }

    // setup needs to be changed, ignore next cycle!
    analog_ignore = 1;
    schedule_irq();
  
    // switch lamp
    if (ir_tx_mode==IR_TX_DISTANCE) {
      if ((distance_cnt&0x0008) != 0x00) {
        set_output_bit(IO_IR_TX);
      } else {
        clear_output_bit(IO_IR_TX);
      }
      activate_output_bit(IO_IR_TX);
    }

    if (distance_cnt&0x80) {
      // store result
      distance_values[analog_channel] = distance_value;
      distance_value = 0;
      distance_cnt = 0;

      schedule_irq();
    
      // select next channel
      switch (analog_channel) {
        case 0: analog_channel = 4; break;
        case 1: analog_channel = 3; break;
        case 2: analog_channel = 6; break;
        case 3: analog_channel = 2; break;
        case 4: analog_channel = 1; break;
        default: analog_channel = 0; break;
      }
    }
    
  } else {
    // current measure
    analog_ignore = 1;
    switch (analog_channel) {
      case 6: analog_current_left = ADC; analog_channel = 7; break;
      case 7: analog_current_right = ADC; analog_channel = 0; break;
      default: analog_channel = 0; break;
    }
  }

  if (analog_channel<5) {
    // distance measure
    analog_select_mux_vcc(analog_channel);
    // discharge port pin
    PORTC &= ~ANALOG_MASK;
    DDRC &= ~ANALOG_MASK;
    DDRC |= (1 << analog_channel);
    sei(); // enable interrupts
    _delay_us(1);
    cli(); // disable interrupts
    DDRC &= ~ANALOG_MASK;
    
  } else {
    // current measure
    analog_select_mux_vcc(analog_channel);
    
  }
  analog_restart();
}



/*
uint8_t encode_val(int16_t val) {
  uint8_t res=0;
  if (val<=0) {
    return 0;
  }
  val+=0x10;

  while(val>=0x20) {
    val /= 2;
    res+=0x10;
  }
  val&=0x0f;
  res+=val;
  return res;
}
*/





