/*  BSD-License:

Copyright (c) 2008 by Nils Springob, nicai-systems, Germany

All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.
  * Neither the name nicai-systems nor the names of its contributors may be
    used to endorse or promote products derived from this software without
    specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

#include <avr/io.h>

/*! IR Receiver */
#define IO_IR_RX       IO_IR_RX
#define IO_IR_RX_PORT  PORTB
#define IO_IR_RX_PIN   PINB
#define IO_IR_RX_BIT   0
#define IO_IR_RX_DDR   DDRB

/*! IR Transmitter/Emitter */
#define IO_IR_TX       IO_IR_TX
#define IO_IR_TX_PORT  PORTD
#define IO_IR_TX_BIT   7
#define IO_IR_TX_DDR   DDRD

// *********************************************
//   MOTOR
// *********************************************

/*! IO_MOTOR_L_DIR */
#define IO_MOTOR_L_DIR       IO_MOTOR_L_DIR
#define IO_MOTOR_L_DIR_PORT  PORTD
#define IO_MOTOR_L_DIR_BIT   4
#define IO_MOTOR_L_DIR_DDR   DDRD

/*! IO_MOT_R_DIR */
#define IO_MOTOR_R_DIR       IO_MOTOR_R_DIR
#define IO_MOTOR_R_DIR_PORT  PORTD
#define IO_MOTOR_R_DIR_BIT   5
#define IO_MOTOR_R_DIR_DDR   DDRD

/*! IO_MOT_L_PWM */
#define IO_MOTOR_L_PWM       IO_MOTOR_L_PWM
#define IO_MOTOR_L_PWM_PORT  PORTB
#define IO_MOTOR_L_PWM_BIT   1
#define IO_MOTOR_L_PWM_DDR   DDRB

/*! IO_MOT_R_PWM */
#define IO_MOTOR_R_PWM       IO_MOTOR_R_PWM
#define IO_MOTOR_R_PWM_PORT  PORTB
#define IO_MOTOR_R_PWM_BIT   2
#define IO_MOTOR_R_PWM_DDR   DDRB

// *********************************************
//   ODOMETRY
// *********************************************

/*! IO_ODO_EN */
#define IO_ODO_EN       IO_ODO_EN
#define IO_ODO_EN_PORT  PORTD
#define IO_ODO_EN_PIN   PIND
#define IO_ODO_EN_BIT   6
#define IO_ODO_EN_DDR   DDRD

/*! IO_ODO_L_A */
#define IO_ODO_L_A       IO_ODO_L_A
#define IO_ODO_L_A_PORT  PORTD
#define IO_ODO_L_A_PIN   PIND
#define IO_ODO_L_A_BIT   0
#define IO_ODO_L_A_DDR   DDRD

/*! IO_ODO_L_B */
#define IO_ODO_L_B       IO_ODO_L_B
#define IO_ODO_L_B_PORT  PORTD
#define IO_ODO_L_B_PIN   PIND
#define IO_ODO_L_B_BIT   1
#define IO_ODO_L_B_DDR   DDRD

/*! IO_ODO_R_A */
#define IO_ODO_R_A       IO_ODO_R_A
#define IO_ODO_R_A_PORT  PORTD
#define IO_ODO_R_A_PIN   PIND
#define IO_ODO_R_A_BIT   2
#define IO_ODO_R_A_DDR   DDRD

/*! IO_ODO_R_B */
#define IO_ODO_R_B       IO_ODO_R_B
#define IO_ODO_R_B_PORT  PORTD
#define IO_ODO_R_B_PIN   PIND
#define IO_ODO_R_B_BIT   3
#define IO_ODO_R_B_DDR   DDRD


// *********************************************
//   SPI
// *********************************************

/*! SPI bus clock */
#define IO_SPI_SCK       IO_SPI_SCK
#define IO_SPI_SCK_PORT  PORTB
#define IO_SPI_SCK_PIN   PINB
#define IO_SPI_SCK_BIT   5
#define IO_SPI_SCK_DDR   DDRB

/*! SPI MOSI */
#define IO_SPI_MOSI       IO_SPI_MOSI
#define IO_SPI_MOSI_PORT  PORTB
#define IO_SPI_MOSI_PIN   PINB
#define IO_SPI_MOSI_BIT   3
#define IO_SPI_MOSI_DDR   DDRB

/*! SPI MISO */
#define IO_SPI_MISO       IO_SPI_MISO
#define IO_SPI_MISO_PORT  PORTB
#define IO_SPI_MISO_PIN   PINB
#define IO_SPI_MISO_BIT   4
#define IO_SPI_MISO_DDR   DDRB

/*! SPI SS */
#define IO_SPI_SS       IO_SPI_SS
#define IO_SPI_SS_PORT  PORTC
#define IO_SPI_SS_PIN   PINC
#define IO_SPI_SS_BIT   5
#define IO_SPI_SS_DDR   DDRC


/* Hilfs-Macro */
#define _PPCAT(a,b) a##b

/* Macro-Funktionen für IO-Bits */
#define set_output_bit(NAME) _PPCAT(NAME,_PORT)|=_BV(_PPCAT(NAME,_BIT))
#define clear_output_bit(NAME) _PPCAT(NAME,_PORT)&=~(unsigned char)_BV(_PPCAT(NAME,_BIT))
#define set_output_bitval(NAME, VAL) _PPCAT(NAME,_PORT)=(_PPCAT(NAME,_PORT)&~(unsigned char)_BV(_PPCAT(NAME,_BIT)))|((VAL)?_BV(_PPCAT(NAME,_BIT)):0)
#define get_output_bit(NAME) (_PPCAT(NAME,_PORT)&_BV(_PPCAT(NAME,_BIT)))

#define activate_output_bit(NAME) _PPCAT(NAME,_DDR)|=_BV(_PPCAT(NAME,_BIT))
#define deactivate_output_bit(NAME) _PPCAT(NAME,_DDR)&=~(unsigned char)_BV(_PPCAT(NAME,_BIT))

#define get_input_bit(NAME) (_PPCAT(NAME,_PIN)&_BV(_PPCAT(NAME,_BIT)))
#define get_activation_bit(NAME) (_PPCAT(NAME,_DDR)&_BV(_PPCAT(NAME,_BIT)))
