/*  BSD-License:

Copyright (c) 2008 by Nils Springob, nicai-systems, Germany

All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.
  * Neither the name nicai-systems nor the names of its contributors may be
    used to endorse or promote products derived from this software without
    specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

#include "iodefs.h"
#include "odometry.h"
#include "pid.h"
#include <avr/interrupt.h>
#include <util/delay.h>

int8_t odometry_state_right;
int8_t odometry_state_left;

int16_t odometry_right;
int16_t odometry_left;

int8_t odometry_dir_left;
int8_t odometry_dir_right;

uint8_t is_nibo_2_5 = 0;

#define IO_ODOMETRY_PIN IO_ODO_L_A_PIN

#define IO_ODOMETRY_LEFT_00 (0)
#define IO_ODOMETRY_LEFT_01 _BV(IO_ODO_L_A_BIT)
#define IO_ODOMETRY_LEFT_10 _BV(IO_ODO_L_B_BIT)
#define IO_ODOMETRY_LEFT_11 (IO_ODOMETRY_LEFT_01|IO_ODOMETRY_LEFT_10)
#define IO_ODOMETRY_LEFT_MASK IO_ODOMETRY_LEFT_11

#define IO_ODOMETRY_RIGHT_00 (0)
#define IO_ODOMETRY_RIGHT_01 _BV(IO_ODO_R_A_BIT)
#define IO_ODOMETRY_RIGHT_10 _BV(IO_ODO_R_B_BIT)
#define IO_ODOMETRY_RIGHT_11 (IO_ODOMETRY_RIGHT_01|IO_ODOMETRY_RIGHT_10)
#define IO_ODOMETRY_RIGHT_MASK IO_ODOMETRY_RIGHT_11


void odometry_init() {
  deactivate_output_bit(IO_ODO_EN);
  _delay_us(10);
  set_output_bit(IO_ODO_EN);
  if (get_input_bit(IO_ODO_EN)) {
    is_nibo_2_5 = 0;
  } else {
    is_nibo_2_5 = 1;
  }  
  activate_output_bit(IO_ODO_EN);
  clear_output_bit(IO_ODO_EN);
  PCMSK2 = _BV(IO_ODO_L_A_BIT) | _BV(IO_ODO_R_A_BIT);
  PCICR |= _BV(PCIE2);
}

void odometry_reset() {
  odometry_left=0;
  odometry_right=0;
}


ISR(PCINT2_vect) {
  uint8_t val;
  int8_t inc;

  // Left side
  val = IO_ODOMETRY_PIN & IO_ODOMETRY_LEFT_MASK;
  if      ((val==IO_ODOMETRY_LEFT_00) && (odometry_state_left))  inc = -1, odometry_state_left=0;
  else if ((val==IO_ODOMETRY_LEFT_01) && (!odometry_state_left)) inc = +1, odometry_state_left=val;
  else if ((val==IO_ODOMETRY_LEFT_10) && (odometry_state_left))  inc = +1, odometry_state_left=0;
  else if ((val==IO_ODOMETRY_LEFT_11) && (!odometry_state_left)) inc = -1, odometry_state_left=val;
  else inc=0;

  pid_left_cnt += inc;
  odometry_left += inc;
  if (inc != 0) {
    odometry_dir_left = inc;
  }
  
  // Right side
  val = IO_ODOMETRY_PIN & IO_ODOMETRY_RIGHT_MASK;
  if      ((val==IO_ODOMETRY_RIGHT_00) && (odometry_state_right))  inc = +1, odometry_state_right=0;
  else if ((val==IO_ODOMETRY_RIGHT_01) && (!odometry_state_right)) inc = -1, odometry_state_right=val;
  else if ((val==IO_ODOMETRY_RIGHT_10) && (odometry_state_right))  inc = -1, odometry_state_right=0;
  else if ((val==IO_ODOMETRY_RIGHT_11) && (!odometry_state_right)) inc = +1, odometry_state_right=val;
  else inc=0;
  
  pid_right_cnt += inc;
  odometry_right += inc;
  if (inc != 0) {
    odometry_dir_right = inc;
  }

}

