#define NIBO_USE_UART0
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include "nibo/niboconfig.h"
#include "nibo/iodefs.h"
#include "nibo/uart0.h"
#include "nibo/leds.h"
#include "nibo/pwm.h"
#include "nibo/bot.h"
#include "nibo/copro.h"
#include "nibo/spi.h"
#include "nibo/floor.h"
#include "niboprot.h"

static char buffer[256];
static uint16_t pos;
static uint16_t len;
static uint8_t mode;
static uint8_t echo;

#define READ_REG(x) nbp_register_flags[x]&(NBP_FLAG_REQUESTED|NBP_FLAG_REPORTED)
#define WRITE_REG(x) nbp_register_flags[x]&NBP_FLAG_MODIFIED_REMOTE



void update() {
  bot_update();
  copro_update();
  floor_update();
  
  
  if (READ_REG(NBPREG_BOTID)) {
    nbp_registers[NBPREG_BOTID] = 0x4e32;   // N2 NIBO 2
    //nbp_registers[NBPREG_BOTID] = 0x4e62; // Nb NIBObee
  }

  if (READ_REG(NBPREG_VERSION)) {
    nbp_registers[NBPREG_VERSION] = 2;
  }
  
  if (READ_REG(NBPREG_VSUPPLY)) {
    uint32_t supply = bot_supply;
    supply *= 166;
    supply /= 10;
    supply -= 1190;
    nbp_registers[NBPREG_VSUPPLY] = supply; // [mV]
  }
  
  if (WRITE_REG(NBPREG_LEDSR)) {
    IO_LEDS_RED_PORT = nbp_registers[NBPREG_LEDSR];
  }
  
  if (WRITE_REG(NBPREG_LEDSG)) {
    IO_LEDS_GREEN_PORT = nbp_registers[NBPREG_LEDSG];
  }  
  
  if (WRITE_REG(NBPREG_LEDSWPWM)) {
    leds_set_headlights (nbp_registers[NBPREG_LEDSWPWM]);
  }

  if (WRITE_REG(NBPREG_MOTPWML) || WRITE_REG(NBPREG_MOTPWMR) || WRITE_REG(NBPREG_MOTPIDL) || WRITE_REG(NBPREG_MOTPIDR) || WRITE_REG(NBPREG_MOTMODE) ) {
    switch (nbp_registers[NBPREG_MOTMODE]) {
      case 0: copro_stopImmediate(); break;
      case 1: copro_stop(); break;
      case 2: copro_setPWM (nbp_registers[NBPREG_MOTPWML], nbp_registers[NBPREG_MOTPWMR]);
      case 3: copro_setSpeed (nbp_registers[NBPREG_MOTPIDL], nbp_registers[NBPREG_MOTPIDR]);
    }
  }
  
  if (READ_REG(NBPREG_MOTCURL)) {
    nbp_registers[NBPREG_MOTCURL] = copro_current_l;
  }

  if (READ_REG(NBPREG_MOTCURR)) {
    nbp_registers[NBPREG_MOTCURR] = copro_current_r;
  }

  if (READ_REG(NBPREG_ODOL)) {
    nbp_registers[NBPREG_ODOL] = copro_ticks_l;
  }

  if (READ_REG(NBPREG_ODOR)) {
    nbp_registers[NBPREG_ODOR] = copro_ticks_r;
  }

  if (READ_REG(NBPREG_FLOORL)) {
    nbp_registers[NBPREG_FLOORL] = floor_relative[FLOOR_LEFT];
  }
  
  if (READ_REG(NBPREG_FLOORR)) {
    nbp_registers[NBPREG_FLOORR] = floor_relative[FLOOR_RIGHT];
  }
  
  if (READ_REG(NBPREG_LINEL)) {
    nbp_registers[NBPREG_LINEL] = floor_relative[LINE_LEFT];
  }

  if (READ_REG(NBPREG_LINER)) {
    nbp_registers[NBPREG_LINER] = floor_relative[LINE_RIGHT];
  }
  
  if (READ_REG(NBPREG_DIST_L)) {
    nbp_registers[NBPREG_DIST_L] = copro_distance[0];
  }
  if (READ_REG(NBPREG_DIST_FL)) {
    nbp_registers[NBPREG_DIST_FL] = copro_distance[1];
  }
  if (READ_REG(NBPREG_DIST_F)) {
    nbp_registers[NBPREG_DIST_F] = copro_distance[2];
  }
  if (READ_REG(NBPREG_DIST_FR)) {
    nbp_registers[NBPREG_DIST_FR] = copro_distance[3];
  }
  if (READ_REG(NBPREG_DIST_R)) {
    nbp_registers[NBPREG_DIST_R] = copro_distance[4];
  }
  
}


static void send_help() {
  PGM_P text = PSTR(NBP_HELP_TEXT NBP_REGISTER_TEXT);
  char c=1;
  while (c) {
    if (!uart0_txfull()) {
      c=pgm_read_byte(text++);
      uart0_putchar(c);
    }
  }
  
}

int main(int argc, char * argv[]) {
  sei(); // enable interrupts
  bot_init();
  spi_init();
  pwm_init();
  leds_init();
  floor_init();
  
  uart0_set_baudrate(9600);
  uart0_enable();
  mode = 1;
  uart0_putchar('N');
  uart0_putchar('I');
  pos=0;
  
  uart0_putchar('B');
  uart0_putchar('O');
  copro_ir_startMeasure();
  uart0_putchar('2');
  uart0_putchar('\r');
  uart0_putchar('\n');
  
  while (1) {
    if (mode==1) {
      if (!uart0_rxempty()) {
        char c = uart0_getchar();
	if (pos==0) {
	  echo = (c!='$');
	}
	if (echo) {
	  uart0_putchar(c);
	}
	if (c!='\r') {
          buffer[pos++] = c;
	}
        if (c=='\n') {
          nbp_request(buffer, pos-1);
	  update();
	  if (nbp_show_help()) {
	    send_help();
	    

	  }
          len = nbp_reply(buffer, sizeof(buffer));
	  pos = 0;
	  mode = 2;
        }
      }
    }  
    if (mode==2) {
      if (pos<len) {
	if (!uart0_txfull()) {
	  uart0_putchar(buffer[pos++]);
	}
      } else {
	pos = 0;
	len = 0;
	mode = 1;
      }
    }
  }
  
  return 0;
}
