/*  BSD-License:

Copyright (c) 2007 by Nils Springob, nicai-systems, Germany

All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.
  * Neither the name nicai-systems nor the names of its contributors may be
    used to endorse or promote products derived from this software without
    specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

#include "iodefs.h"
#include "i2c.h"
#include "comm.h"
#include <avr/interrupt.h>


enum {
  COMM_STATE_IDLE,
  COMM_STATE_RX,
  COMM_STATE_TX
};

uint8_t comm_pos;
uint8_t comm_buf[COMM_BUF_SIZE];
uint8_t comm_cmd;
uint8_t comm_size;
uint8_t comm_state;


uint8_t i2c_onStart(uint8_t address) {
  if ((address>>1)!=I2C_ADDRESS) {
    return 0;
  }

  if (address&0x01) {
    if (comm_state==COMM_STATE_IDLE) {
      // error: Tx only after Repeated Stop! Send NAK, reset
      comm_state = COMM_STATE_IDLE;
      return 0;
      
    } else if (comm_state==COMM_STATE_RX) {
      // switch to transmit mode
      comm_size = 1 + comm_onTransaction(comm_pos);
      comm_pos = 1;
      comm_state = COMM_STATE_TX;
      return 1;
      
    }
  
  } else {
    // start receive mode
    comm_pos = 0;
    comm_state = COMM_STATE_RX;
    return 1;
  }

  // error: send NAK, reset
  comm_state = COMM_STATE_IDLE;
  return 0;
}

uint8_t i2c_onRxData(uint8_t data) {
  if (comm_pos<COMM_BUF_SIZE) {
    comm_buf[comm_pos++] = data;
    return 1;
  } else {
  
    return 0;
  }
}

uint8_t i2c_onTxData() {
  comm_pos++;
  if (comm_pos<comm_size) {
    return 1;
  } else {
    return 0;
  }
}

uint8_t i2c_onTxReload() {
  return comm_buf[comm_pos];
}

void i2c_onStop() {
  if (comm_state==COMM_STATE_RX) {
    if (comm_pos>0) {
      // master sent a transaction
      comm_onTransaction(comm_pos);
    }
  }
  comm_state = COMM_STATE_IDLE;
}

