/*  BSD-License:

Copyright (c) 2007 by Nils Springob, nicai-systems, Germany

All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.
  * Neither the name nicai-systems nor the names of its contributors may be
    used to endorse or promote products derived from this software without
    specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

#include <avr/io.h>


/*! A0 i/o */
#define IO_A0       IO_A0
#define IO_A0_PORT  PORTA
#define IO_A0_PIN   PINA
#define IO_A0_BIT   0
#define IO_A0_DDR   DDRA

/*! A1 i/o */
#define IO_A1       IO_A1
#define IO_A1_PORT  PORTA
#define IO_A1_PIN   PINA
#define IO_A1_BIT   1
#define IO_A1_DDR   DDRA

/*! A2 i/o */
#define IO_A2       IO_A2
#define IO_A2_PORT  PORTA
#define IO_A2_PIN   PINA
#define IO_A2_BIT   2
#define IO_A2_DDR   DDRA

/*! A3 i/o */
#define IO_A3       IO_A3
#define IO_A3_PORT  PORTA
#define IO_A3_PIN   PINA
#define IO_A3_BIT   3
#define IO_A3_DDR   DDRA


/*! B0 output */
#define IO_B0       IO_B0
#define IO_B0_PORT  PORTB
#define IO_B0_PIN   PINB
#define IO_B0_BIT   0
#define IO_B0_DDR   DDRB

/*! B2 output */
#define IO_B2       IO_B2
#define IO_B2_PORT  PORTB
#define IO_B2_PIN   PINB
#define IO_B2_BIT   2
#define IO_B2_DDR   DDRB


/*! Servo output */
#define IO_SERVO       IO_SERVO
#define IO_SERVO_PORT  PORTA
#define IO_SERVO_PIN   PINA
#define IO_SERVO_BIT   5
#define IO_SERVO_DDR   DDRA

/*! Distance enable */
#define IO_DIST_EN       IO_DIST_EN
#define IO_DIST_EN_PORT  PORTB
#define IO_DIST_EN_PIN   PINB
#define IO_DIST_EN_BIT   1
#define IO_DIST_EN_DDR   DDRB



/*! I2C bus clock */
#define IO_I2C_SCL       IO_I2C_SCL
#define IO_I2C_SCL_PORT  PORTA
#define IO_I2C_SCL_PIN   PINA
#define IO_I2C_SCL_BIT   4
#define IO_I2C_SCL_DDR   DDRA

/*! I2C bus data */
#define IO_I2C_SDA       IO_I2C_SDA
#define IO_I2C_SDA_PORT  PORTA
#define IO_I2C_SDA_PIN   PINA
#define IO_I2C_SDA_BIT   6
#define IO_I2C_SDA_DDR   DDRA






/*! I2C driver configuration */
#define I2C_ADDRESS 42
#define I2C_RX_BUF_SIZE 9
#define I2C_TX_BUF_SIZE 9

// I2C timer clock = 125kHz (factor 1/256), clock time = 8us
#define I2C_TIMER_TCCRA        TCCR0A
#define I2C_TIMER_TCCRA_INIT   0x00
#define I2C_TIMER_TCCRB        TCCR0B
#define I2C_TIMER_TCCRB_INIT   _BV(CS02)
#define I2C_TIMER_TIMSK        TIMSK0
#define I2C_TIMER_TIMSK_INIT   0x00
#define I2C_TIMER_TIMSK_ACTIVE _BV(OCIE0A)
#define I2C_TIMER_TIFR         TIFR0

#define I2C_TIMER_TCNT         TCNT0
#define I2C_TIMER_OCRA         OCR0A
#define I2C_TIMER_TIMEOUT      200
#define I2C_TIMER_OCRA_vect    TIM0_COMPA_vect


/* Hilfs-Macro */
#define _PPCAT(a,b) a##b

/* Macro-Funktionen für IO-Bits */
#define set_output_bit(NAME) _PPCAT(NAME,_PORT)|=_BV(_PPCAT(NAME,_BIT))
#define clear_output_bit(NAME) _PPCAT(NAME,_PORT)&=~(unsigned char)_BV(_PPCAT(NAME,_BIT))
#define set_output_bitval(NAME, VAL) _PPCAT(NAME,_PORT)=(_PPCAT(NAME,_PORT)&~(unsigned char)_BV(_PPCAT(NAME,_BIT)))|((VAL)?_BV(_PPCAT(NAME,_BIT)):0)

#define activate_output_bit(NAME) _PPCAT(NAME,_DDR)|=_BV(_PPCAT(NAME,_BIT))
#define deactivate_output_bit(NAME) _PPCAT(NAME,_DDR)&=~(unsigned char)_BV(_PPCAT(NAME,_BIT))

#define get_input_bit(NAME) (_PPCAT(NAME,_PIN)&_BV(_PPCAT(NAME,_BIT)))

