/*  BSD-License:

Copyright (c) 2007 by Nils Springob, nicai-systems, Germany

All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.
  * Neither the name nicai-systems nor the names of its contributors may be
    used to endorse or promote products derived from this software without
    specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

#include "iodefs.h"
#include "servo.h"
#include "transaction.h"
#include "i2c.h"
#include "analog.h"
#include "persistent.h"
#include <util/delay.h>
#include <util/atomic.h>
#include <avr/interrupt.h>

#define LOBYTE(x)        (uint8_t)((uint16_t)x)
#define HIBYTE(x)        (uint8_t)(((uint16_t)x)>>8)
#define MAKE_WORD(hi,lo) (((uint8_t)hi*(uint16_t)0x100)+(uint8_t)lo)

#define CNT_RELOAD 5

uint8_t dist[5];

int32_t pos;
uint8_t mode;

uint16_t cnt;

void servo_set_dir(uint8_t dir) {
  int32_t pos=dir;
  if (dir<90) {
    pos *= (persistent.servo_mid-persistent.servo_lo);
    pos /= 90;
    pos += persistent.servo_lo;
  } else {
    pos -= 90;
    pos *= (persistent.servo_hi-persistent.servo_mid);
    pos /= 90;
    pos += persistent.servo_mid;
  }
  servo_set(pos);
}

inline static void do_move() {
  if (global_pos==global_target) {
    global_mode = MODE_IDLE;
    return;
  }
  if (global_pos>global_target) {
    global_pos--;
  } else {
    global_pos++;
  }
  servo_set_dir(global_target);
}

inline static void do_measure() {
  if (cnt) {
    cnt--;
    return;
  }
  cnt = CNT_RELOAD;
  cnt *= global_step;
  uint8_t delta = (global_pos>global_target)?(global_pos-global_target):(global_target-global_pos);

  // store measurement
  global_distances[global_pos] = global_actdist;
  if (global_pos>global_target) {
    global_pos -= global_step;
  } else {
    global_pos += global_step;
  }
  
  if (delta < global_step) {
    global_mode = MODE_IDLE;
    global_pos = global_target;
  }
  
  servo_set_dir(global_pos);
}


int main(void) {
  persistent_update();
  servo_init();
  i2c_init();
  analog_init();
  activate_output_bit(IO_DIST_EN);
  clear_output_bit(IO_DIST_EN);
  
  while(1) {
    //if (i2c_lock()) {
    //  i2c_unlock();
    //}
    sei();
    while(!servo_get_sync()) ; // wait for next 20 ms frame

    uint16_t val = analog_get_distance();
    ATOMIC_BLOCK(ATOMIC_FORCEON) {
      global_actdist = val;
    }
    
    switch (global_mode) {
      case MODE_IDLE:      break;
      case MODE_MOVE:      do_move(); break;
      case MODE_MEASURE:   do_measure(); break;
      case MODE_MOVESERVO: break;
    }
    if (persistent_dirty) {
      persistent_commit();
    }
  }
}

