/*  BSD-License:

Copyright (c) 2007 by Nils Springob, nicai-systems, Germany

All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.
  * Neither the name nicai-systems nor the names of its contributors may be
    used to endorse or promote products derived from this software without
    specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

#include "iodefs.h"
#include "servo.h"
#include <util/delay.h>
#include <avr/interrupt.h>

volatile uint8_t servo_active;
volatile uint16_t servo_us;
volatile uint8_t servo_sync;

void servo_init() {
  activate_output_bit(IO_SERVO);
  set_output_bit(IO_SERVO);
  OCR1A = 10000;
  OCR1B =  1000;
  TCCR1A = _BV(COM1B1);
  TCCR1B = _BV(WGM12)|_BV(CS11);
  TIMSK1 = _BV(OCIE1B)|_BV(OCIE1A);
}

void servo_set_irq(uint16_t value) {
  servo_us = value;
}

void servo_set(uint16_t value) {
  cli();
  servo_us = value;
  sei();
}

uint8_t servo_get_sync() {
  cli();
  uint8_t res = servo_sync;
  servo_sync = 0;
  sei();
  return res;
}

ISR(TIM1_COMPB_vect) {
  if (servo_active) {
    OCR1B = 1000U;
    TCCR1A = _BV(COM1B1)|_BV(COM1B0); // set to 1 on match
    servo_active = 0;
  } else {
    OCR1B = 1000U+servo_us;
    TCCR1A = _BV(COM1B1); // set to 0 on match
    servo_active = 1;
  }
}

ISR(TIM1_COMPA_vect) {
  if (servo_sync!=255) {
    servo_sync++;
  }
}

