#include <nibobee/iodefs.h>
#include <nibobee/i2cmaster.h>
#include <nibobee/bgx1.h>
#include <nibobee/analog.h>
#include <nibobee/led.h>
#include <nibobee/delay.h>
#include <nibobee/clock.h>
#include <nibobee/odometry.h>
#include <nibobee/motpwm.h>
#include <nibobee/motpid.h>

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdio.h>

#include "images.h"

// Dieses Programm gibt nacheinander drei Bilder auf dem Display aus, zwischen den Bildern gibt es jeweils 2 Sekunden Pause.

// Bild mit dem angegebenen Index zentriert ausgeben, vorher den Bildschirm löschen.
void center_image(uint8_t image) {
  uint8_t x = (128 - image_getWidth(image))/2;
  uint8_t y = (64 - image_getHeight(image))/2;
  bgx1_fillAll(0);
  bgx1_move(x, y);
  bgx1_drawBitmap_P(image_getWidth(image), image_getHeight(image), image_getBits(image));
}


int main() {
  activate_output_group(IO_LEDS);  // LED bits als Output
  i2c_init();

  led_set(LED_L_YE, 0);
  led_set(LED_R_YE, 0);
  led_set(LED_L_RD, 1);
  led_set(LED_R_RD, 1);
  while (!bgx1_init());
  led_set(LED_L_YE, 1);
  led_set(LED_R_YE, 1);
  led_set(LED_L_RD, 0);
  led_set(LED_R_RD, 0);

  bgx1_setStatus(0xff);

  motpwm_init();
  motpid_init();

  // Hauptschleife:
  while(1) {
    sei();

    if (bgx1_getStatus()==0xff) {
      // wenn das Display noch nicht initialisiert wurde, dann jetzt!
      delay(1500); // 2000ms = 1.5s
      bgx1_reset();
      delay(500); // 500ms = 0.5s      
    }
    center_image(IMG_NOSE);
    delay(2000); // 2000ms = 2s
    center_image(IMG_BEE);
    delay(2000); // 2000ms = 2s
    center_image(IMG_MARSROVER);
    delay(2000); // 2000ms = 2s
      
  }
  return 0;
}

