/* NIBObee C project */

/*
***********************************
******* Akku-Spannung Biene *******
***********************************
****** Ursprung von Rabenauge *****
** Überarbeitung von elektrolutz **
**** Überarbeitung von workwind ***
***********************************

***************************************************************************************************
*
* Die aktuelle Akku-Spannung wird in Volt mit 1 Nachkommastelle auf dem Display BGX1 ausgegeben.
*
* Die Messung wird alle ??? Sekunden wiederholt
*
* Die Displaybeleuchtung wird nach einer Zeit ausgeschaltet und kann mit dem rechten Fühler,
* nach hinten gedrückt, für eine Zeit wieder eingeschalten werden.
*
* Als Betriebsanzeige blinken die gelben LED`s abwechselnd.
*
* Unterschreitet die Akkuspannung einen bestimmten Wert (x.xV), wird die Displaybeleuchtung
* eingeschaltet, ein Achtungssignal ausgegeben und die beiden roten LED´s eingeschaltet.
*
* Die Zeiten für das Blinken und die Displaybeleuchtung laufen timergesteuert über Timer 0.
*
***************************************************************************************************
*/

#include <nibobee/iodefs.h>
#include <nibobee/i2cmaster.h>
#include <nibobee/bgx1.h>
#include <nibobee/analog.h>
#include <nibobee/led.h>
#include <nibobee/delay.h>
#include <nibobee/clock.h>
#include <nibobee/odometry.h>
#include <nibobee/motpwm.h>
#include <nibobee/motpid.h>
#include <nibobee/sens.h>
#include <nibobee/base.h>

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdio.h>

// - Variablen ------------------------------------------------------------------------------------
int16_t count_backLight = 0;                          // Zeitzähler Displaybeleuchtung
int8_t count = 0;                                     // Zeitzähler Blinker 
char text[32];                                        // Ausgabehilfe
uint16_t count_messung = 0;                           // Zähler Messungen

// - Spannungsüberwachung -------------------------------------------------------------------------
void Batterie()
{
  uint16_t millivolt = nibobee_getMillivolt();
  uint16_t volt_vk = millivolt / 1000;       // Vorkommawert Spannung
  uint16_t volt_nk = millivolt % 1000;       // Nachkommawert Spannung
  
  // beliebige Anzahl von Stellen vor dem Komma und
  // 3 Stellen nach dem Komma mit führenden Nullen ausgeben
  sprintf(text," %d.%03d Volt",(int)volt_vk, (int)volt_nk); 
  bgx1_termGoto(0,1);                                 // Akkuspannung in Volt ausgeben
  bgx1_termPrint(text);


  if (millivolt < 4600)                    // Falls Spannung zu niedrig wird 
     { bgx1_setIllumination (1023);                   // Licht an
       bgx1_termGoto(0,6);                            // Alarm schlagen
       bgx1_termPrint("  ACHTUNG  ");
       led_set(LED_L_RD,1);
       led_set(LED_R_RD,1);
       count_backLight=0;
     }
     else
     { bgx1_termGoto(0,6);                            // Akku ist OK
       bgx1_termPrint("Akku ist OK");
     }   
}

// - Beleuchtung aus/Stromsparmode ----------------------------------------------------------------
void Beleuchtung()                                    // Unterprogramm Displaybeleuchtung
{ if (count_backLight > 499)                          // schaltet nach ??? Sekunden die Display-
  { bgx1_setIllumination(0);                          // beleuchtung ab
  }
}  

// - Hauptprogramm --------------------------------------------------------------------------------
int main()
{
#if defined(TCCR0)
  TCCR0 = (1 << CS02) | (1 << CS00);                  // Normal Mode, kein OC0-Pin, prescaler /1024 
  TIMSK |= (1 << TOIE0);                              // Timer0 Overflow-Interrupt erlauben 
#else
  TCCR0B = (1 << CS02) | (1 << CS00);                 // Normal Mode, kein OC0-Pin, prescaler /1024 
  TIMSK0 |= (1 << TOIE0);                             // Timer0 Overflow-Interrupt erlauben
#endif  
 
  i2c_init();                                         // I2C initialisieren
  bgx1_init();                                        // LCD initialisieren
  analog_init();                                      // Analogeingänge initialisieren
  led_init();                                         // LED`s initialisieren
  sens_init();                                        // Fühler initialisieren 
  set_output_group(IO_SENS);                          // Pull-ups aktivieren
  sei();                                              // Interrupts einschalten
  delay(1500);                                        // Wartezeit Initialisierung BGX1
  bgx1_reset();                                       // Displayanzeige löschen
 
  bgx1_setIllumination(1023);                         // Licht an
  bgx1_termGoto(0,0);
  bgx1_termPrint("Bordspannung");                     // Programmname
  bgx1_termGoto(0,5);
  bgx1_termPrint("<< Lichtschalter");                 // Hinweis Lichtschalter
 
  while(1)
    { int8_t status_R = sens_getRight();
      if(status_R < 0)                                // mit rechtem Fühler nach hinten
      { bgx1_setIllumination (1023);                  // Licht wieder anmachen
        count_backLight=0;
      }   
      if(count == 50)                                 // nach ??? Sekunden
      { led_set(LED_R_YE,1);                          // LED an 
        led_set(LED_L_YE,0);                          // andere aus
        Batterie();                                   // Akkuspannung messen
 
        count_messung++;                              // Anzahlmessungen nach Programmstart
        bgx1_termGoto(0,3);
        bgx1_termPrint("Messung: ");
        sprintf(text," %06d",(int)count_messung);
        bgx1_termPrint(text);
      } 
      if(count > 99 )                                 // nach ??? Sekunden
      { led_set(LED_R_YE,0);                          // LED aus 
        led_set(LED_L_YE,1);                          // andere an
        Beleuchtung();                                // Licht aus?
        count=0;                                      // Sekundentimer zurücksetzen
      } 
    }            
  return 0;                             
}
  
// - Timer-ISR ------------------------------------------------------------------------------------
ISR(TIMER0_OVF_vect)                     
{
  count++;                                            // fürs blinken
  count_backLight++;                                  // für HG-Licht ausschalten
}
  
// - ENDE -----------------------------------------------------------------------------------------
