/*
 * Mit diesem Programm kann der NIBObee umherfahren. Wenn mit den Fühlern
 * Hindernisse detektiert werden, versucht der NIBObee diesen auszuweichen.
 */


#include <nibobee/iodefs.h>
#include <nibobee/motpwm.h>
#include <nibobee/delay.h>
#include <nibobee/sens.h>
#include <nibobee/led.h>
#include <stdlib.h>

enum {
  MODE_STOP,
  MODE_DRIVE,
  MODE_BACK,
  MODE_STEER_R,
  MODE_STEER_L,
  MODE_AVOID_R,
  MODE_AVOID_L
};

int16_t speed_l;
int16_t speed_r;
uint16_t counter_ms;


uint8_t perform_check(uint8_t mode);

uint8_t do_stop();
uint8_t do_drive();
uint8_t do_back();
uint8_t do_steer_r();
uint8_t do_steer_l();
uint8_t do_avoid_r();
uint8_t do_avoid_l();

uint16_t randomval;

enum {
  LEDS_STOP    = 0x0,
  LEDS_DRIVE   = 0x9,
  LEDS_BACK    = 0x6,
  LEDS_STEER_R = 0x1,
  LEDS_STEER_L = 0x8,
  LEDS_AVOID_R = 0x2,
  LEDS_AVOID_L = 0x4
};

void leds(uint8_t val) {
  if (val&0x01) IO_LEDS_PORT |= 0x01; else IO_LEDS_PORT &= (uint8_t)~0x01;
  if (val&0x02) IO_LEDS_PORT |= 0x02; else IO_LEDS_PORT &= (uint8_t)~0x02;
  if (val&0x04) IO_LEDS_PORT |= 0x04; else IO_LEDS_PORT &= (uint8_t)~0x04;
  if (val&0x08) IO_LEDS_PORT |= 0x08; else IO_LEDS_PORT &= (uint8_t)~0x08;
}

int main() {
  motpwm_init();
  sens_init();
  led_init();
  uint8_t mode;

  while(1==1) {
    enable_interrupts();
    delay(1);
    randomval = random();

    mode = perform_check(mode);
    
    switch (mode) {
      case MODE_STOP:    mode = do_stop(); break;
      case MODE_DRIVE:   mode = do_drive(); break;
      case MODE_BACK:    mode = do_back(); break;
      case MODE_STEER_R: mode = do_steer_r(); break;
      case MODE_STEER_L: mode = do_steer_l(); break;
      case MODE_AVOID_R: mode = do_avoid_r(); break;
      case MODE_AVOID_L: mode = do_avoid_l(); break;
    }

    switch (mode) {
      case MODE_STOP:    speed_l =    0; speed_r =    0; leds(LEDS_STOP); break;
      case MODE_DRIVE:   speed_l =  500; speed_r =  500; leds(LEDS_DRIVE); break;
      case MODE_BACK:    speed_l = -500; speed_r = -500; leds(LEDS_BACK); break;
      case MODE_STEER_R: speed_l =  600; speed_r =  400; leds(LEDS_STEER_R); break;
      case MODE_STEER_L: speed_l =  400; speed_r =  600; leds(LEDS_STEER_L); break;
      case MODE_AVOID_R: speed_l = -400; speed_r = -600; leds(LEDS_AVOID_R); break;
      case MODE_AVOID_L: speed_l = -600; speed_r = -400; leds(LEDS_AVOID_L); break;
    }
    motpwm_setLeft(speed_l);
    motpwm_setRight(speed_r);
  }
  return 0;
}

uint8_t perform_check(uint8_t mode) {
  if (sens_getLeft() && sens_getRight()) {
    if ((sens_getLeft()==-1) && (sens_getRight()==-1)) {
      mode = MODE_BACK;
    } else {
      mode = MODE_STOP;
    }
  }
  return mode;
}


uint8_t do_stop() {
  if ((sens_getLeft()==0) && (sens_getRight()==0)) {
    return MODE_DRIVE;
  }
  return MODE_STOP;
}

uint8_t do_back() {
  if (sens_getLeft()==0) {
    return MODE_AVOID_L;
  }
  if (sens_getRight()==0) {
    return MODE_AVOID_R;
  }
  return MODE_BACK;
}

uint8_t do_drive() {
  if (sens_getRight()==1) {
    return MODE_STEER_L;
  }
  if (sens_getLeft()==1) {
    return MODE_STEER_R;
  }
  if (sens_getRight()==-1) {
    return MODE_AVOID_L;
  }
  if (sens_getLeft()==-1) {
    return MODE_AVOID_R;
  }
  return MODE_DRIVE;
}

uint8_t do_steer_r() {
  if (sens_getLeft()==0) {
    return MODE_DRIVE;
  }
  return MODE_STEER_R;
}

uint8_t do_steer_l() {
  if (sens_getRight()==0) {
    return MODE_DRIVE;
  }
  return MODE_STEER_L;
}

uint8_t do_avoid_r() {
  if (counter_ms==0) {
    counter_ms = 750 + (randomval&0x1ff);
  } else {
    counter_ms--;
  }
  if (counter_ms) {
    return MODE_AVOID_R;
  } else {
    return MODE_DRIVE;
  }
}

uint8_t do_avoid_l() {
  if (counter_ms==0) {
    counter_ms=750 + (randomval&0x1ff);
  } else {
    counter_ms--;
  }
  if (counter_ms) {
    return MODE_AVOID_L;
  } else {
    return MODE_DRIVE;
  }
}

