/*
  Dieses Testprogramm laesst beim Nibo eine Status-LED abwechselnd rot und gruen
  blinken.
*/


//#include <nibo/niboconfig.h>            /* muss immer als erste Datei
//                                           eingebunden werden */
//#include <nibo/leds.h>                  /* stellt die leds_xxx Routinen zur
//                                           Verfuegung */
//#include <nibo/delay.h>                 /* stellt die delay Routine zur
//                                           Verfuegung */

#include <nibobee/iodefs.h>
#include <nibobee/usart.h>
#include <nibobee/motpwm.h>
#include <nibobee/analog.h>
#include <nibobee/odometry.h>

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

enum {
  COLL_NONE = 0xF0,
  COLL_RIGHT_BLOCKED = 0x70,
  COLL_LEFT_MOVE = 0xB0,
  COLL_LEFT_BLOCKED = 0xD0,
  COLL_RIGHT_MOVE = 0xE0,
  COLL_BLOCKED = 0x50,
  COLL_TRAPPED = 0xA0,
  COLL_LEFT_TURN = 0x90,
  COLL_RIGHT_TURN = 0x60
};


void print_hex (uint8_t val) {
  char c1=val/16;
  char c2=val%16;
  if (c1<10) c1+='0'; else c1+='a'-10;
  if (c2<10) c2+='0'; else c2+='a'-10;
  usart_putchar(c1);
  usart_putchar(c2);
}


int8_t decode_l (uint8_t val) {
  if (val==0x01) return -1;
  if (val==0x02) return 1;
  return 0;
}

int8_t decode_r (uint8_t val) {
  if (val==0x01) return 1;
  if (val==0x02) return -1;
  return 0;
}

int main() {
  activate_output_group(IO_LEDS);  // LED bits als Output
  usart_setbaudrate(9600);
  usart_enable();
  motpwm_init();
  odometry_init();
  motpwm_setLeft(0);
  motpwm_setRight(0);
  set_output_group(IO_SENS);       // Pull-ups aktivieren

  while(1) {
    sei();
    _delay_ms(1);
    int16_t speed_l=0;
    int16_t speed_r=0;

    switch (IO_SENS_PIN&0xf0) {
      case COLL_NONE:          speed_l= 500; speed_r= 500; break;
      case COLL_RIGHT_BLOCKED: speed_l=-300; speed_r=-200; break;
      case COLL_LEFT_BLOCKED:  speed_l=-200; speed_r=-300; break;
      case COLL_LEFT_MOVE:     speed_l=-200; speed_r=   0; break;
      case COLL_RIGHT_MOVE:    speed_l=   0; speed_r=-200; break;
      case COLL_BLOCKED:       speed_l=-200; speed_r=-200; break;
      case COLL_TRAPPED:       speed_l=   0; speed_r=   0; break;
      case COLL_LEFT_TURN:     speed_l= 200; speed_r= 300; break;
      case COLL_RIGHT_TURN:    speed_l= 300; speed_r= 200; break;
    }

    motpwm_setLeft(speed_l);
    motpwm_setRight(speed_r);


    if (usart_txempty()) {
      int16_t lval = odometry_getLeft(0);
      int16_t rval = odometry_getRight(0);
      print_hex(HIBYTE(lval));
      print_hex(LOBYTE(lval));
      usart_putchar(':');
      print_hex(HIBYTE(rval));
      print_hex(LOBYTE(rval));
      usart_putchar(10);
      usart_putchar(13);
    }
    
  }


  return 0;                             /* Diese Zeile ist eigentlich nicht
                                           notwendig, da das Programm sie wegen
                                           der Endlosschleife nie erreichen
                                           wird, der Compiler gibt jedoch
                                           ansonsten eine Warnung aus. */
}
