#include <nibobee/iodefs.h>
#include <nibobee/motpwm.h>
#include <nibobee/motpid.h>
#include <nibobee/analog.h>
#include <nibobee/led.h>
#include <nibobee/line.h>
#include <nibobee/delay.h>
#include <nibobee/clock.h>
#include <nibobee/odometry.h>
#include <nibobee/sens.h>


#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

//                      (     ((v)<(l))? (l) :    (   ((v)>(h))? (h) : (v)   )        )
#define INTERVAL(l,v,h) (     ((v)<(l))? (l) :    (   ((v)>(h))? (h) : (v)   )        )

#define BLACK_MAX 10

uint8_t mode;
uint16_t counter;
int16_t lval, rval;
int8_t last_coll;

enum {
  MODE_HALT,
  MODE_GO,
  MODE_GO_L,
  MODE_GO_R,
  MODE_SLOW,
  MODE_SLOW_L,
  MODE_SLOW_R,
  MODE_BACK,
  MODE_BACK_L,
  MODE_BACK_R,
  MODE_BACK_HALT
};

void escape() {
  int16_t fl, fc, fr;
  fl = line_get(LINE_L);
  fc = line_get(LINE_C);
  fr = line_get(LINE_R);    
  
  int8_t dir = (fl<fr)? +1 : -1;
  odometry_reset();
  motpid_stop(1);
  
  while(1) {
    _delay_ms(1);

    if (sens_getLeft() || sens_getRight()) {
      motpid_stop(1);
      continue;
    }
    if (dir==+1) motpid_setSpeed ( +10,  -10);
    if (dir==-1) motpid_setSpeed ( -10,  +10);
    int diff = odometry_getRight(0)-odometry_getLeft(0);
    if ((diff>40) || (diff<-40)) {
      motpid_stop(1);
      return;
    }
  }
}

int main_obst() {
  activate_output_group(IO_LEDS);  // LED bits als Output
  sens_init();
  analog_init();
  odometry_init();
  line_init();

  led_set(LED_L_RD, 1);    
  led_set(LED_R_RD, 1);
  _delay_ms(200);
  led_set(LED_L_RD, 0);
  led_set(LED_R_RD, 0);
  _delay_ms(800);

  analog_setExtToggleMode(ANALOG_EXT2, 1);
  analog_setExtToggleMode(ANALOG_EXT3, 1);
  motpid_init();
  
  mode = MODE_HALT;
  counter = 200;
  int16_t fl=2*BLACK_MAX, fc=2*BLACK_MAX, fr=2*BLACK_MAX;
  
  
  // Hauptschleife:
  while(1) {
    sei();
    _delay_ms(1);
    
    uint8_t sensL = 0;
    uint8_t sensR = 0;    
    uint8_t coll = 0;
    
    int16_t lv, rv;
    
    fl *= 3;  fc *= 3; fr *= 3;
    fl += line_get(LINE_L);
    fc += line_get(LINE_C);
    fr += line_get(LINE_R);    
    fl /= 4;  fc /= 4; fr /= 4;
    
    
    if ((fl<BLACK_MAX) || (fc<BLACK_MAX) || (fr<BLACK_MAX)) {
      escape();
      mode = MODE_HALT;
      counter = 100;
    }    
    
    
    lv = analog_getValue(ANALOG_EXT2H);
    rv = analog_getValue(ANALOG_EXT3H);
    lv -= analog_getValue(ANALOG_EXT2);
    rv -= analog_getValue(ANALOG_EXT3);
    
    lv = 8 * INTERVAL(0, lv, 20);
    rv = 8 * INTERVAL(0, rv, 20);
    
    rval = (31 * rval + rv + 15)/32;
    lval = (31 * lval + lv + 15)/32;
       
    if (sens_getLeft()) {
      coll = 1;
      sensL = 3;      
    } else if (lval>8*6) {
      sensL = 3;
    } else if (lval>8*4) {
      sensL = 2;
    } else if (lval>8*2) {
      sensL = 1;
    }
    
    if (sens_getRight()) {
      coll = 1;
      sensR = 3;
    } else if (rval>8*6) {
      sensR = 3;
    } else if (rval>8*4) {
      sensR = 2;
    } else if (rval>8*2) {
      sensR = 1;
    }
    
    led_set(LED_L_RD, (sensL > 1));
    led_set(LED_R_RD, (sensR > 1));

    led_set(LED_L_YE, ((sensL == 1) || (sensL == 2)));
    led_set(LED_R_YE, ((sensR == 1) || (sensR == 2)));

    if (mode == MODE_BACK_R) {
      int16_t ol = odometry_getLeft(0)-odometry_getRight(0);
      if (ol < -35) {
	counter = 0;
      } 
      if ((ol < -15) && (!sensL && !sensR)) {
	counter = 0;
      }
    }
    if (mode == MODE_BACK_L) {
      int16_t or = odometry_getRight(0)-odometry_getLeft(0);
      if (or < -35) {
	counter = 0;
      }
      if ((or < -15) && (!sensL && !sensR)) {
	counter = 0;
      }
    }
    if (mode == MODE_BACK) {
      if ((odometry_getRight(0) < -40) || (odometry_getLeft(0) < -40)) {
	counter = 0;
      }
    }
        
    if (counter) {
        counter--;
    } else {
      switch (mode) {
        case MODE_HALT:
	  if (sensR>sensL) {
	    mode = MODE_BACK_R;
	    odometry_reset();
            counter=1500;
	  } else if (sensL>sensR) {
	    mode = MODE_BACK_L;
	    odometry_reset();
            counter=1500;
	  } else if (sensL>0) {
	    mode = MODE_BACK;
	    odometry_reset();
	    counter=1500;
	  } else {
	    mode = MODE_SLOW;
	  }
          break;
	
        case MODE_GO:
        case MODE_GO_L:
        case MODE_GO_R:
        case MODE_SLOW:
        case MODE_SLOW_L:
        case MODE_SLOW_R:
	  if (coll) {
	    if (sensR>sensL) {
	      last_coll = +1;
	    
	    } else if (sensL>sensR) {
	      last_coll = -1;
	    
	    } else /* sensL==sensR */ {
	    
	    }
	    mode = MODE_BACK;
	    odometry_reset();
	    counter = 1500;
	    
	  } else if (sensR>sensL) {
	    if (sensR==3) {
	      last_coll = +1;
	      mode = MODE_HALT;
	      counter = 200;
	    } else if (sensR==2) {
	      last_coll = +1;
	      mode = MODE_HALT;
	      counter = 200;
	    } else {
	      mode = MODE_GO;    
	    }
	  } else if (sensL>sensR) {
	    if (sensL==3) {
	      last_coll = -1;
	      mode = MODE_HALT;
	      counter = 200;
	    } else if (sensL==2) {
	      last_coll = -1;
	      mode = MODE_HALT;
	      counter = 200;
	    } else {
	      mode = MODE_GO;
	    }
	  } else /* sensL==sensR */ {
	    if (sensL==3) {
	      mode = MODE_BACK;
	      counter = 200;
	    } else if (sensL==2) {
	      mode = MODE_HALT;
	      counter = 200;
	    } else if (sensL==1) {
	      mode = MODE_SLOW;
	    } else {
	      mode = MODE_GO;
	    }    
	  }
	  break;
	  
        case MODE_BACK:
	  mode = MODE_BACK_HALT;
          counter = 100;
	  break;
	  
        case MODE_BACK_L:
	  mode = MODE_BACK_HALT;
          counter = 100;
	  break;
	  
        case MODE_BACK_R:
	  mode = MODE_BACK_HALT;
          counter = 100;
	  break;
	  
	case MODE_BACK_HALT:
	  if (last_coll==+1) {
	    mode = MODE_BACK_R;
	    last_coll=0;
	  } else if (last_coll==-1) {
	    mode = MODE_BACK_L;
	    last_coll=0;
	  } else {
            mode = MODE_SLOW;
	  }
	  break;
	  
      }
    }
    
    switch (mode) {
        case MODE_HALT:      motpid_stop(1); break;
        case MODE_GO:        motpid_setSpeed ( 17,  17); break;
        case MODE_GO_L:      motpid_setSpeed ( 10,  22); break;
        case MODE_GO_R:      motpid_setSpeed ( 22,  10); break;
        case MODE_SLOW:      motpid_setSpeed ( 10,  10); break;
        case MODE_SLOW_L:    motpid_setSpeed (  0,  10); break;
        case MODE_SLOW_R:    motpid_setSpeed ( 10,   0); break;
        case MODE_BACK:      motpid_setSpeed (-10, -10); break;
        case MODE_BACK_L:    motpid_setSpeed ( -5,  -10); break;
        case MODE_BACK_R:    motpid_setSpeed (-10,   -5); break;
        case MODE_BACK_HALT: motpid_stop(1); break;
    }
    
  }
  return 0;
}
