#include <nibobee/iodefs.h>
#include <nibobee/motpwm.h>
#include <nibobee/motpid.h>
#include <nibobee/analog.h>
#include <nibobee/line.h>
#include <nibobee/led.h>
#include <nibobee/sens.h>
#include <nibobee/delay.h>
#include <nibobee/clock.h>
#include <nibobee/odometry.h>

#include <stdio.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#include "sensoric.h"
#include "util.h"

#define PWM_PC(x) (((x)*1024L)/100)

#define MS(x) ((x)/2)

// prepare colli
uint8_t from_line(int8_t turnticks);
uint16_t start_line;

#define START_LINE_INIT MS(1000)

enum {
  MODE_LINE_STRAIGHT,
  MODE_LINE_CURVE_R,
  MODE_LINE_CURVE_L
};


uint8_t mode;
uint16_t mblock;
uint8_t red_cnt;
uint16_t next_clock;
uint16_t ores;



static void log_odometry() {
  log_str("L=");
  log_hex16(odometry_getLeft(0));
  log_str(" R=");
  log_hex16(odometry_getRight(0));
  log_str("\n");
}

static void logdata(int16_t l, int16_t c, int16_t r, int8_t p) {
  static int cnt=0;
  if (cnt) {
    cnt--;
  } else {
    int16_t a = line_getAbs(LINE_C);
    cnt = MS(1000);
    log_hex8(l);
    log_str(" ");
    log_hex8(c);
    log_str(" ");
    log_hex8(r);
    log_str(" ");
    log_hex8(p);
    log_str(" ");
    log_hex16(a);
    log_str("\n");
  }
}

int main_line(uint8_t arg) {
  start_line = START_LINE_INIT;
  sensoric_init(SENSORIC_MODE_LINE);
  int16_t speed_flt_l=0; // PWM: - 511 ... + 511
  int16_t speed_flt_r=0; // PWM: - 511 ... + 511

  led_set(LED_L_YE, 1);
  led_set(LED_R_YE, 1);
  _delay_ms(1000);
  led_set(LED_L_YE, 0);
  led_set(LED_R_YE, 0);

  sei();
  clock_set_timer(2);
  
  // Hauptschleife:
  while(1) {
    sei();
    clock_wait_timer();
    int16_t speed_l=0;
    int16_t speed_r=0;

    // Sensorwerte aktualisiseren
    int16_t lval = line_get(LINE_L);
    int16_t cval = line_get(LINE_C);
    int16_t rval = line_get(LINE_R);

    int8_t p = sensoric_getLinePosF();
    logdata (lval, cval, rval, p);
       
    if (red_cnt) {
      if (--red_cnt==0) {
        led_set(LED_L_RD, 0);
        led_set(LED_R_RD, 0);
      }
    }

    if (ores) {
      if (--ores==0) {
	odometry_reset();
      }
    }
    
    if (start_line) {
      start_line--;
    }
    
    
    if (!sensoric_line_valid || (line_getAbs(LINE_C)>0x400)) {
      // zu hell -> HALT!
      speed_flt_l = speed_flt_r = speed_r = speed_l = 0;
      
    } else {
      if (mblock) {
        if (--mblock==0) {
          if (mode != MODE_LINE_STRAIGHT) {
            mode = MODE_LINE_STRAIGHT;
            mblock = MS(310);
          } else {
            led_set(LED_L_RD, 1);
            led_set(LED_R_RD, 1);
            red_cnt = MS(100);
            log_odometry();
	    
	    int16_t l = odometry_getLeft(0);
    	    int16_t r = odometry_getRight(0);
	    int16_t d = r-l;
	    if (arg && (d>=30) && (d<=45) && (r>180)) {
	      motpwm_setLeft(0);
	      motpwm_setRight(0);
              led_set(LED_L_RD, 1);
              led_set(LED_R_RD, 0);
              sensoric_init(SENSORIC_MODE_FLOOR);
	      //from_line(77-d);
	      from_line(100-d);
              sensoric_init(SENSORIC_MODE_LINE);
	      start_line=START_LINE_INIT;
	    }
	    if (arg && (d<=-30) && (d>=-45) && (l>180)) {
	      motpwm_setLeft(0);
	      motpwm_setRight(0);
              led_set(LED_L_RD, 0);
              led_set(LED_R_RD, 1);
              sensoric_init(SENSORIC_MODE_FLOOR);
	      //from_line(-77-d);
	      from_line(-100-d);
              sensoric_init(SENSORIC_MODE_LINE);
	      start_line=START_LINE_INIT;
	    }
	    
	    
            odometry_reset();    
	    ores=MS(200);
	    
          }
        }
        switch (mode) {
          case MODE_LINE_CURVE_L: p += 10*mblock/MS(400); break;
          case MODE_LINE_CURVE_R: p -= 10*mblock/MS(400); break;
        }
      } else {
	uint8_t mode_old=mode;
        if (p<-10) {
          mode = (mode==MODE_LINE_CURVE_R)?MODE_LINE_STRAIGHT:MODE_LINE_CURVE_L;
        }
        if (p>10) {
          mode = (mode==MODE_LINE_CURVE_L)?MODE_LINE_STRAIGHT:MODE_LINE_CURVE_R;
        }
        
        if (mode!=mode_old) {
	  mblock = MS(400);
	}
      }
      
      if (p<0) {
        speed_r = PWM_PC(100);
        //speed_l = PWM_PC(100) + p*32;
        speed_l = PWM_PC(100) + p*32;
      } else if (p>0) {
        //speed_r = PWM_PC(100) - p*32;
        speed_r = PWM_PC(100) - p*32;
        speed_l = PWM_PC(100);
      } else {
        speed_r = PWM_PC(100);
        speed_l = PWM_PC(100);
      }
      
      if (mblock) {
        switch (mode) {
          case MODE_LINE_CURVE_L: speed_l += -(PWM_PC(50)*mblock)/MS(400); break;
          case MODE_LINE_CURVE_R: speed_r += -(PWM_PC(50)*mblock)/MS(400); break;
        }
      }
      
      switch (mode) {
        case MODE_LINE_CURVE_L:  led_set(LED_L_YE, 0); led_set(LED_R_YE, 1); break;
        case MODE_LINE_CURVE_R:  led_set(LED_L_YE, 1); led_set(LED_R_YE, 0); break;
        case MODE_LINE_STRAIGHT: led_set(LED_L_YE, 1); led_set(LED_R_YE, 1); break;
      }
      
    }
      
    
    if (sens_getLeft() || sens_getRight()) {
      speed_flt_l = speed_flt_r = speed_r = speed_l = 0;
    }
    
    if (start_line) {
      speed_l /= 2;
      speed_r /= 2;
    }
    
    //if (speed_r<0) speed_r=0;
    //if (speed_l<0) speed_l=0;
    
    // Soll-Geschwindigkeit filtern: 75% alter Wert, 25% neuer Wert
    // alt*75/100 + neu*25/100 = (3*alt+1*neu)/4
    speed_flt_l = (3*speed_flt_l + speed_l + 3) / 4;
    speed_flt_r = (3*speed_flt_r + speed_r + 3) / 4;

    // Soll Werte für Motoren setzen
    //motpwm_setLeft(speed_flt_l);
    //motpwm_setRight(speed_flt_r);
    motpwm_setLeft(speed_l);
    motpwm_setRight(speed_r);

  }
  return 0;
}
