#include <nibobee/iodefs.h>
#include <nibobee/motpwm.h>
#include <nibobee/motpid.h>
#include <nibobee/analog.h>
#include <nibobee/led.h>
#include <nibobee/delay.h>
#include <nibobee/clock.h>
#include <nibobee/line.h>
#include <nibobee/odometry.h>
#include <nibobee/sens.h>
#include <nibobee/usart.h>


#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

int main_obst();
int main_leds();
int main_line(uint8_t arg);
int main_calib();
int main_remote();


void log_hex4(uint8_t value) {
  usart_putchar((value<10)?('0'+value):('a'-10+value));
}

void log_hex8(uint8_t value) {
  log_hex4((value>>4)&0x0f);
  log_hex4((value>>0)&0x0f);
}

void log_hex16(uint16_t value) {
  log_hex4((value>>12)&0x000f);
  log_hex4((value>>8)&0x000f);
  log_hex4((value>>4)&0x000f);
  log_hex4((value>>0)&0x000f);
}

void log_shex16(int16_t value) {
  if (value<0) {
    usart_putchar('-');    
    log_hex16(-value);
  } else if (value>0) {
    usart_putchar('+');
    log_hex16(value);
  } else{
    usart_putchar(' ');
    log_hex16(value);
  }
}

void log_str(const char * txt) {
  char c;
  while ((c = *txt++)) {
    while (usart_txfull()) {
    }
    usart_putchar(c);
  }
}


int main() {
  led_init();
  sens_init();
  analog_init();
  odometry_init();
  line_init();
  usart_setbaudrate(9600);
  usart_enable();
   
  motpwm_init();
  motpid_init();
  motpwm_setLeft(0);
  motpwm_setRight(0);

  line_readPersistent(); // Kalibrierung für Liniensensoren laden

  // BKit XS
  analog_setExtToggleMode(ANALOG_EXT2, 1);
  analog_setExtToggleMode(ANALOG_EXT3, 1);

  
  led_set(LED_L_RD, 1);
  led_set(LED_R_RD, 1);
  led_set(LED_L_YE, 1);
  led_set(LED_R_YE, 1);
  _delay_ms(200);
  int8_t l = sens_getLeft();
  int8_t r = sens_getRight();

  while (sens_getLeft()||sens_getRight()) {
    _delay_ms(100);
  }

  led_set(LED_L_RD, 0);
  led_set(LED_R_RD, 0);
  led_set(LED_L_YE, 0);
  led_set(LED_R_YE, 0);
  
  
  if ((l==1)&&(r==1)) {
    main_calib();
  } else if (l==-1) {
    // PROG 0
    main_line(0);    
  } else if (l==1) {
    // PROG 1
    main_obst();
  } else if (r==1) {
    // PROG 2
    main_line(1);
  } else if (r==-1) {
    // PROG 3
    main_remote();
  } else {
    main_leds();
  }    

  return 0;  
}
