#include <nibobee/iodefs.h>
#include <nibobee/motpwm.h>
#include <nibobee/motpid.h>
#include <nibobee/analog.h>
#include <nibobee/led.h>
#include <nibobee/delay.h>
#include <nibobee/clock.h>
#include <nibobee/odometry.h>

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

uint16_t analog_measure(uint8_t mux) {
  ADMUX = mux;
  ADCSRA = _BV(ADPS2)  // prescale faktor = 128 ADC laeuft 
         | _BV(ADPS1)  // mit 15 MHz / 128 = 120 kHz         
         | _BV(ADPS0)         
         | _BV(ADEN)   // ADC an         
         | _BV(ADSC);  // Beginne mit der Konvertierung  
  while (ADCSRA & _BV(ADSC)) {
  }
  uint16_t result = ADC;
  return result;
}

void showState(uint8_t i) {
  PORTD &= (uint8_t)~0x03;
  switch (i) {
    case 0: PORTD |= 0x00; break;
    case 1: PORTD |= 0x01; break;
    case 2: PORTD |= 0x02; break;
  }
}

uint16_t measure(uint8_t i) {
  PORTC |= 0x0f;
  PORTA &= ~0x03;
  PORTA |= 0x0c;  
  switch (i) {
    case 0: PORTA &= (uint8_t)~0x04; PORTC &= (uint8_t)~0x04; break;
    case 1: PORTA &= (uint8_t)~0x00; PORTC &= (uint8_t)~0x04; break;
    case 2: PORTA &= (uint8_t)~0x08; PORTC &= (uint8_t)~0x04; break;
    case 3: PORTA &= (uint8_t)~0x00; PORTC &= (uint8_t)~0x04; break;
    case 4: PORTA &= (uint8_t)~0x04; PORTC &= (uint8_t)~0x01; break;
    case 5: PORTA &= (uint8_t)~0x00; PORTC &= (uint8_t)~0x01; break;
    case 6: PORTA &= (uint8_t)~0x08; PORTC &= (uint8_t)~0x01; break;
    case 7: PORTA &= (uint8_t)~0x00; PORTC &= (uint8_t)~0x01; break;
    case 8: PORTA &= (uint8_t)~0x04; PORTC &= (uint8_t)~0x02; break;
    case 9: PORTA &= (uint8_t)~0x00; PORTC &= (uint8_t)~0x02; break;
    case 10: PORTA &= (uint8_t)~0x08; PORTC &= (uint8_t)~0x02; break;
    case 11: PORTA &= (uint8_t)~0x00; PORTC &= (uint8_t)~0x02; break;
    case 12: PORTA &= (uint8_t)~0x04; PORTC &= (uint8_t)~0x08; break;
    case 13: PORTA &= (uint8_t)~0x00; PORTC &= (uint8_t)~0x08; break;
    case 14: PORTA &= (uint8_t)~0x08; PORTC &= (uint8_t)~0x08; break;
    case 15: PORTA &= (uint8_t)~0x00; PORTC &= (uint8_t)~0x08; break;
  }
  _delay_ms(1);
  if (i & 0x02) {
    return analog_measure(0 | _BV(REFS0) | _BV(REFS1));
  } else {
    return analog_measure(1 | _BV(REFS0) | _BV(REFS1));
  }
}

int main() {
  DDRD|=0x03;
  DDRA|=0x0c;
  DDRC|=0x0f;
  
  activate_output_group(IO_LEDS);  // LED bits als Output

  
  // Hauptschleife:
  while(1) {
    uint8_t clear=1;
    sei();
    for (uint8_t i=0; i<16; ++i) {
      int16_t value = measure(i++);
      value -= measure(i);
      if (value>8) {
        clear=0;
        showState(1);
      } else {
        if (!clear) {
          showState(0);
        }
      }
      _delay_ms(20);
    }
    if (clear) {
      showState(2);
    }
  }
  return 0;
}
