//#define USE_DISPLAY

#include <nibobee/iodefs.h>
#include <nibobee/motpwm.h>
#include <nibobee/motpid.h>
#include <nibobee/analog.h>
#include <nibobee/line.h>
#include <nibobee/led.h>
#include <nibobee/delay.h>
#include <nibobee/base.h>
#include <nibobee/sens.h>
#ifdef USE_DISPLAY
#include <nibobee/bgx1.h>
#include <nibobee/i2cmaster.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#define ADC_VOLT_FULL 263

#define SPEED(x) x
//#define SLOW

#define WHITE_LINE

#if !defined(WHITE_LINE) && !defined(BLACK_LINE)
#define BLACK_LINE 1
#endif

// MACROS

#define CLIP_MIN(VAL, MINVAL) (((VAL)<(MINVAL))?(MINVAL):(VAL))
#define CLIP_MAX(VAL, MAXVAL) (((VAL)>(MAXVAL))?(MAXVAL):(VAL))
#define CLIP_RANGE(MINVAL, VAL, MAXVAL) (((VAL)<(MINVAL))?(MINVAL):CLIP_MAX((VAL), (MAXVAL)))


char text[16];

void follow_line();
void calculate_position();

enum {
  MODE_STOP,
  MODE_DRIVE,
  MODE_BACK,
  MODE_STEER_R,
  MODE_STEER_L,
  MODE_AVOID_R,
  MODE_AVOID_L,
  MODE_FOLLOW_LINE,
  MODE_TURN_L,
  MODE_TURN_R
};

volatile uint16_t feeling_stress;
volatile uint16_t feeling_huffiness;

volatile uint16_t collision_dead_time; // hunderstel Sekunden

/* line position: left -32 ... +32 right */
int8_t line_position;
/* no line 0 ... 16 line detected */
int8_t line_detection;


int16_t speed_l;
int16_t speed_r;
uint16_t counter_ms;

int16_t speed_flt_l=0;
int16_t speed_flt_r=0;
int8_t dir_wish = 1;


uint8_t perform_check(uint8_t mode);

uint8_t do_stop();
uint8_t do_drive();
uint8_t do_back();
uint8_t do_steer_r();
uint8_t do_steer_l();
uint8_t do_avoid_r();
uint8_t do_avoid_l();

uint16_t randomval;

enum {
  LEDS_STOP    = 0x0,
  LEDS_DRIVE   = 0x9,
  LEDS_BACK    = 0x6,
  LEDS_STEER_R = 0x1,
  LEDS_STEER_L = 0x8,
  LEDS_AVOID_R = 0x2,
  LEDS_AVOID_L = 0x4
};

void leds(uint8_t val) {
  if (val&0x01) IO_LEDS_PORT |= 0x01; else IO_LEDS_PORT &= (uint8_t)~0x01;
  if (val&0x02) IO_LEDS_PORT |= 0x02; else IO_LEDS_PORT &= (uint8_t)~0x02;
  if (val&0x04) IO_LEDS_PORT |= 0x04; else IO_LEDS_PORT &= (uint8_t)~0x04;
  if (val&0x08) IO_LEDS_PORT |= 0x08; else IO_LEDS_PORT &= (uint8_t)~0x08;
}

volatile uint8_t manage_line;

void analog_irq_hook() {
  manage_line=1;
}

volatile uint16_t line_max_t = 500;
volatile uint16_t line_max = 500;

uint8_t disable_dec=0;

void clock_hundredth_irq_hook() {
  if (collision_dead_time) {
    collision_dead_time--;
  }
  
  if (feeling_stress) {
    feeling_stress--;
  }
  
  if (line_max_t>line_max) {
    line_max++;
  } else if (line_max>0) {
    line_max_t=0;
    if (disable_dec) {
      disable_dec--;
      return;
    }
    line_max--;
  }
  
}

void clock_second_irq_hook() {
  if (feeling_huffiness) {
    feeling_huffiness--;
  }
}

void add_stress(uint16_t amount) {
  disable_interrupts();
  feeling_stress += amount;
  enable_interrupts();
}

void add_huffiness(uint16_t amount) {
  disable_interrupts();
  feeling_huffiness += amount;
  enable_interrupts();
}

void set_collision_dead_time(uint16_t amount) {
  disable_interrupts();
  collision_dead_time = amount;
  enable_interrupts(); 
}

int16_t align_voltage(uint16_t volt, int16_t speed) {
  int32_t s = speed;
  s *= volt;
  s /= ADC_VOLT_FULL;
  return s;
}

uint8_t perform_check(uint8_t mode) {
  disable_interrupts();
  if (feeling_stress>1000) {
    collision_dead_time=100;
    if (dir_wish<0) {
      mode = MODE_TURN_L;
    } else {
      mode = MODE_TURN_R;
    }
    feeling_stress=0;
  }
  enable_interrupts();
  
  if (sens_getLeft() && sens_getRight()) {
    set_collision_dead_time(100);
    if ((sens_getLeft()==-1) && (sens_getRight()==-1)) {
      mode = MODE_BACK;
    } else {
      mode = MODE_STOP;
    }
  }
  
  if (collision_dead_time==0) {
    if (line_detection>=50) {
      mode = MODE_FOLLOW_LINE;
    }
  }
  
  return mode;
}


uint8_t do_stop() {
  if ((sens_getLeft()==0) && (sens_getRight()==0)) {
    return MODE_DRIVE;
  }
  return MODE_STOP;
}

uint8_t do_back() {
  if (sens_getLeft()==0) {
    return MODE_AVOID_L;
  }
  if (sens_getRight()==0) {
    return MODE_AVOID_R;
  }
  return MODE_BACK;
}

uint8_t do_drive() {
  if (sens_getRight()==1) {
    dir_wish=-1;
    return MODE_STEER_L;    
  }
  if (sens_getLeft()==1) {
    dir_wish=1;
    return MODE_STEER_R;
  }
  if (sens_getRight()==-1) {
    add_stress(150);
    dir_wish=1;
    set_collision_dead_time(100);
    return MODE_AVOID_L;
  }
  if (sens_getLeft()==-1) {
    add_stress(150);
    dir_wish=-1;
    set_collision_dead_time(100);
    return MODE_AVOID_R;
  }
  return MODE_DRIVE;
}

uint8_t do_follow_line() {
  if (sens_getRight()==1) {
    dir_wish=-1;
    add_stress(400);
    set_collision_dead_time(25);
    return MODE_STEER_L;
  }
  if (sens_getLeft()==1) {
    dir_wish=1;
    add_stress(400);
    set_collision_dead_time(25);
    return MODE_STEER_R;
  }
  if (sens_getRight()==-1) {
    dir_wish=1;
    add_stress(800);
    add_huffiness(20);
    set_collision_dead_time(100);
    return MODE_AVOID_L;
  }
  if (sens_getLeft()==-1) {
    dir_wish=-1;
    add_stress(800);
    add_huffiness(20);
    set_collision_dead_time(100);
    return MODE_AVOID_R;
  }
  return MODE_FOLLOW_LINE;
}


uint8_t do_steer_r() {
  if (sens_getLeft()==0) {
    return MODE_DRIVE;
  }
  return MODE_STEER_R;
}

uint8_t do_steer_l() {
  if (sens_getRight()==0) {
    return MODE_DRIVE;
  }
  return MODE_STEER_L;
}

uint8_t do_avoid_r() {
  if (collision_dead_time) {
    return MODE_AVOID_R;
  } else {
    return MODE_DRIVE;    
  }
}

uint8_t do_avoid_l() {
  if (collision_dead_time) {
    return MODE_AVOID_L;
  } else {
    return MODE_DRIVE;
  }
}

uint8_t do_turn_r() {
  if (collision_dead_time) {
    return MODE_TURN_R;
  } else {
    return MODE_DRIVE;    
  }
}

uint8_t do_turn_l() {
  if (collision_dead_time) {
    return MODE_TURN_L;
  } else {
    return MODE_DRIVE;
  }
}


#ifdef USE_DISPLAY
uint8_t has_bgx;
#endif
void do_blink(uint8_t mask) {
  for (int i=0; i<5; ++i) {
    led_set(LED_L_RD, mask&0x01);
    led_set(LED_R_RD, mask&0x01);
    led_set(LED_L_YE, mask&0x02);
    led_set(LED_R_YE, mask&0x02);
    delay(200);
    led_set(LED_L_RD, 0);
    led_set(LED_R_RD, 0);
    led_set(LED_L_YE, 0);
    led_set(LED_R_YE, 0);
    delay(200);
  }
}


int main_calibration() {
  led_init();
  analog_init();
  activate_output_bit(IO_LINE_EN);

  line_readPersistent();

  while(1==1) {                      // Endlosschleife (1==1 ist immer wahr!)
    enable_interrupts();
    delay(10);
    int16_t speed_l=0;
    int16_t speed_r=0;

    if (sens_getLeft()==-1) {
      while (sens_getLeft()==-1) delay(1);
      line_calibrateWhite();
      do_blink(2);
    }
     
    if (sens_getRight()==-1) {
      while (sens_getRight()==-1) delay(1);
      line_calibrateBlack();
      do_blink(1);
    }

    set_output_groupbitval(IO_LEDS, L_YE, line_get(LINE_L)>0x010);
    set_output_groupbitval(IO_LEDS, L_RD, line_get(LINE_L)>0x380);
    set_output_groupbitval(IO_LEDS, R_YE, line_get(LINE_R)>0x010);
    set_output_groupbitval(IO_LEDS, R_RD, line_get(LINE_R)>0x380);

    if ((sens_getLeft()==1) && (sens_getRight()==1)) {
      while ((sens_getLeft()==1) && (sens_getRight()==1)) {
        delay(1);
      }
      line_writePersistent();
      do_blink(3);
    }

  }
  return 0;
}


int main() {
  activate_output_group(IO_LEDS);  // LED bits als Output
  motpwm_init();
  motpwm_setLeft(0);
  motpwm_setRight(0);
  analog_init();
  motpid_init();
  sens_init();

  if ((sens_getLeft()==-1) && (sens_getRight()==-1)) {
    while ((sens_getLeft()==-1) || (sens_getRight()==-1)) {
      delay(1);
    }
    main_calibration();
  }

  
#ifdef USE_DISPLAY
  i2c_init();
  _delay_ms(10);
  uint8_t i = 20;
  while (i--) {
    has_bgx = bgx1_init();
    if (has_bgx) {
      break;
    }
    _delay_ms(2);
  }
  has_bgx = 1;

  bgx1_syncInterface();
#endif
  
  line_readPersistent();
  set_output_group(IO_SENS);       // Pull-ups aktivieren
  activate_output_bit(IO_LINE_EN);


  // Countdown: LEDs blinken lassen
  for (uint8_t i=0; i<3; ++i) {
    led_set(LED_L_RD, 1);
    led_set(LED_R_RD, 1);
    _delay_ms(10);
    led_set(LED_L_RD, 0);
    led_set(LED_R_RD, 0);
    _delay_ms(990);
  }
  led_set(LED_L_YE, 1);
  led_set(LED_R_YE, 1);
  _delay_ms(1000);
  led_set(LED_L_YE, 0);
  led_set(LED_R_YE, 0);

  sens_init();
  led_init();
  uint8_t mode = MODE_DRIVE;
  
#ifdef USE_DISPLAY
  bgx1_termClear();
#endif

  while(1==1) {
    enable_interrupts();
   
    if (manage_line) {
      manage_line=0;
      calculate_position();
    }
    
    //nibobee_assertMinVoltage(NIBOBEE_ASSERT_VOLTAGE(3.0));
    randomval = random();

    mode = perform_check(mode);

    switch (mode) {
      case MODE_STOP:        mode = do_stop(); break;
      case MODE_DRIVE:       mode = do_drive(); break;
      case MODE_BACK:        mode = do_back(); break;
      case MODE_STEER_R:     mode = do_steer_r(); break;
      case MODE_STEER_L:     mode = do_steer_l(); break;
      case MODE_AVOID_R:     mode = do_avoid_r(); break;
      case MODE_AVOID_L:     mode = do_avoid_l(); break;
      case MODE_FOLLOW_LINE: mode = do_follow_line(); break;
      case MODE_TURN_R:      mode = do_turn_r(); break;
      case MODE_TURN_L:      mode = do_turn_l(); break;
    }

    switch (mode) {
      case MODE_STOP:    speed_l = SPEED(   0); speed_r = SPEED(   0); leds(LEDS_STOP); break;
      case MODE_DRIVE:   speed_l = SPEED( 600); speed_r = SPEED( 600); leds(LEDS_DRIVE); break;
      case MODE_BACK:    speed_l = SPEED(-500); speed_r = SPEED(-500); leds(LEDS_BACK); break;
      case MODE_STEER_R: speed_l = SPEED( 600); speed_r = SPEED( 400); leds(LEDS_STEER_R); break;
      case MODE_STEER_L: speed_l = SPEED( 400); speed_r = SPEED( 600); leds(LEDS_STEER_L); break;
      case MODE_AVOID_R: speed_l = SPEED(-400); speed_r = SPEED(-600); leds(LEDS_AVOID_R); break;
      case MODE_AVOID_L: speed_l = SPEED(-600); speed_r = SPEED(-400); leds(LEDS_AVOID_L); break;
      case MODE_FOLLOW_LINE: follow_line(); leds(LEDS_DRIVE); continue;
      case MODE_TURN_R:  speed_l = SPEED(  20); speed_r = SPEED( -30); leds(LEDS_STEER_R); break;
      case MODE_TURN_L:  speed_l = SPEED( -30); speed_r = SPEED(  20); leds(LEDS_STEER_L); break;
    }
    
    if ((mode==MODE_TURN_R) || (MODE_TURN_L)) {
      motpid_setSpeed(speed_l, speed_r);
    } else {
      motpid_release();
      motpwm_setLeft(speed_l);
      motpwm_setRight(speed_r);
    }
  }
  return 0;
}


uint16_t clip(int16_t val) {
  if (val<0) return 0;
  if (val>1023) return 1023;
  return (uint16_t)val;
}

void calculate_position() {
#ifdef WHITE_LINE
  uint16_t lval = clip(line_get(LINE_L));
  uint16_t cval = clip(line_get(LINE_C));
  uint16_t rval = clip(line_get(LINE_R));
#else  
  uint16_t lval = clip(1024-line_get(LINE_L));
  uint16_t cval = clip(1024-line_get(LINE_C));
  uint16_t rval = clip(1024-line_get(LINE_R));
#endif

  sei();
  uint16_t lmax = line_max;
  if (lval>=line_max) {
    line_max_t = lmax = lval;
    disable_dec = 100;
  }
  if (cval>=line_max) {
    line_max_t = lmax = cval;
    disable_dec = 100;
  }
  if (rval>=line_max) {
    line_max_t = lmax = rval;
    disable_dec = 100;
  }    
  cli();
  
  lval *= 64;
  lval /= lmax;
  cval *= 64;
  cval /= lmax;
  rval *= 64;
  rval /= lmax;  

  int16_t delta = 4*(abs(lval-cval) + abs(rval-cval));
  int8_t newlinepos = line_position;

  if (delta < 64) {
    if (delta<line_detection) {
      line_detection--;
    }
  } else {
    line_detection = 64;
  }

  if (line_position<-8) {
    rval /= 2;
  }
  if (line_position>+8) {
    lval /= 2;
  }

  if (line_detection<16) {
    if (line_position<-10) newlinepos=-32;
    if (line_position>+10) newlinepos=+32;
  } else {
  
    if ((cval>lval) && (cval>rval)) {
      // cval is maximum -> centered, position [-8 ... +8]
      if (lval>rval+1) {
        if (rval<=1) {
          // [-8 ... -5]
          uint8_t delta = cval-lval;
          delta /= 3;
          if (delta>3) {
            delta=3;
          }
          newlinepos = -8 + delta;
        } else {
          // [-4 ... -1]
          uint8_t delta = lval-rval;
          delta /= 16;
          if (delta>3) {
            delta=3;
          }
          newlinepos = -1 - delta;
          
        }
      } else if (rval>lval+1) {
        if (lval<=1) {
          // [+5 ... +8]
          uint8_t delta = cval-rval;
          delta /= 3;
          if (delta>3) {
            delta=3;
          }
          newlinepos = 8 - delta;
        } else {
          // [+1 ... +4]
          uint8_t delta = rval-lval;
          delta /= 16;
          if (delta>3) {
            delta=3;
          }
          newlinepos = 1 + delta;
        }
      } else {
        newlinepos=0;
      }    
    }

    if ((lval>cval) && (lval>rval)) {
      // lval is maximum -> line is left, position [-31 ... -9]
      if (cval<=1) {
        if (lval<32) {
          // [-31 ... -26]
          newlinepos = -28;
        } else {
          // [-25 ... -20]
          newlinepos = -23;
        }
      } else {
        if (cval<32) {
          // [-19 ... -16]
          uint8_t delta = cval;
          delta /= 8;
          newlinepos = -19 + delta;
        } else {
          // [-15 ... -9]
          uint8_t delta = lval-cval;
          delta /= 5;        
          newlinepos = -9 - delta;
        }
      }
    }
    
    if ((rval>cval) && (rval>lval)) {
      // rval is maximum -> line is right, position [+9 ... 31]
      if (cval<=1) {
        if (rval<32) {
          // [+26 ... +31]
          newlinepos = 28;
        } else {
          // [+20 ... +25]
          newlinepos = 23;
        }
      } else {
        if (cval<32) {
          // [+16 ... +19]
          uint8_t delta = cval;
          delta /= 8;
          newlinepos = 19 - delta;
        } else {
          // [+9 ... +15]
          uint8_t delta = rval-cval;
          delta /= 5;        
          newlinepos = +9 + delta;
        }
      }
    }
  }
  
  if (newlinepos>line_position) {
    uint8_t delta = newlinepos-line_position;
    delta /= 4;
    delta += 1;
    line_position += delta;
  } else if (newlinepos<line_position) {
    uint8_t delta = line_position-newlinepos;
    delta /= 4;
    delta += 1;
    line_position -= delta;    
  }
  
#ifdef USE_DISPLAY
  if (has_bgx) {
    bgx1_termGoto(4,0);
    sprintf (text, "%04i ", (int)rval);
    bgx1_termPrint(text);
    sprintf (text, "%04i ", (int)cval);
    bgx1_termPrint(text);
    sprintf (text, "%04i", (int)lval);
    bgx1_termPrint(text);
  
    bgx1_termGoto(9,1);
    sprintf (text, "%04i", (int)line_max);
    bgx1_termPrint(text);
  }
#endif

}


#define PID_SCALE 256

#define PID_KI  0.00
#define PID_KD -0.60
#define PID_KP  1.56
#define PID_CLIP 2

int16_t calculate_pid2 (int16_t setpoint, int16_t reading) { 
  static int16_t d1 = 0;
  static int16_t i1 = 0;
  
  int16_t variance = setpoint -= reading;
  int res = 0;
  
  res += ((int16_t)(PID_KP*PID_SCALE)) * variance;
  res += ((int16_t)(PID_KD*PID_SCALE)) * (variance-d1);
  d1 = variance;
  i1 = i1 + variance;
  if (i1 >= +PID_SCALE*PID_CLIP) i1 = +PID_SCALE*PID_CLIP;
  if (i1 <= -PID_SCALE*PID_CLIP) i1 = -PID_SCALE*PID_CLIP;
  res += ((int16_t)(PID_KI*PID_SCALE)) * (i1);
  return res/PID_SCALE;
}



int8_t calculate_pid () {
  int16_t setpoint = 0;
  int16_t reading = -line_position;
  int16_t res = calculate_pid2(setpoint, reading);
  if (res<-40) return -40;
  if (res>+40) return +40;
  return res;
}
 
 
void follow_line() {
  int16_t speed_l=0;
  int16_t speed_r=0;

#ifdef USE_DISPLAY
  if (has_bgx) {
    bgx1_termGoto(9,2);
    sprintf (text, "%+3i", (int)line_position);
    bgx1_termPrint(text);
  }
#endif

  int lp = calculate_pid ();

  if (lp<0) {
    speed_r = 32;              
    speed_l = 32+lp; // -> 0 - 32
  } else if (lp>0) {
    speed_r = 32-lp; // -> 0 - 32
    speed_l = 32;
  } else {
    speed_l = 32;
    speed_r = 32;
  }
  
  // line_detection = [0 .. +64]
  
  uint8_t ld = line_detection * 2;
  
  if (ld>60) {
    speed_l *= 32;
    speed_r *= 32;
  } else if (ld>45) {
    speed_l *= 26;
    speed_r *= 26;
  } else if (ld>30) {
    speed_l *= 22;
    speed_r *= 22;
  } else {
    speed_l *= 16;
    speed_r *= 16;
  }

  // speed_l = [-1024 .. +1024]
  // speed_r = [-1024 .. +1024]
  
  speed_l += 2*(int16_t)line_detection-100; // += [-100 ... +28]
  speed_r += 2*(int16_t)line_detection-100; // += [-100 ... +28]
    
  uint16_t adc_volt = analog_getValue(ANALOG_VOLT);
    
  //speed_l = align_voltage(adc_volt, speed_l);
  //speed_r = align_voltage(adc_volt, speed_r);

  speed_flt_l*=3; speed_flt_l+=speed_l; speed_flt_l/=4;
  speed_flt_r*=3; speed_flt_r+=speed_r; speed_flt_r/=4;

#ifdef SLOW
  speed_flt_l = (7*speed_flt_l)/8;
  speed_flt_r = (7*speed_flt_r)/8;
#endif
  
  speed_flt_l = CLIP_RANGE(-1024, speed_flt_l, +1024);
  speed_flt_r = CLIP_RANGE(-1024, speed_flt_r, +1024);
  
  motpwm_setLeft(speed_flt_l);
  motpwm_setRight(speed_flt_r);

}

