#include <nibobee/iodefs.h>
#include <nibobee/usart.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

void adc_init(uint8_t channel){
  DDRA &= ~((uint8_t)1 << channel);  // Pin als input
  PORTA &= ~((uint8_t)1 << channel); // Pullup aus.
}

uint16_t adc_read(uint8_t channel){
  uint16_t result = 0x00;

  // Reference voltage: AVCC (5V)
  ADMUX= _BV(REFS0);

  ADMUX |= (channel & 0x07);  // Und jetzt Kanal waehlen, nur single ended

  ADCSRA = _BV(ADPS2)  // prescale faktor = 128 ADC laeuft
         | _BV(ADPS1)  // mit 14,7456MHz/ 128 = 115,2kHz
         | _BV(ADPS0)
         | _BV(ADEN)   // ADC an
         | _BV(ADSC);  // Beginne mit der Konvertierung

  // Warten bis konvertierung beendet, das sollte 25 ADC-Zyklen dauern,
  // also 1/4608 s
  while ( (ADCSRA & _BV(ADSC)) != 0) {
    __asm volatile("nop");
  }

  // Ergebnis zusammenbauen
  result= ADCL;
  result+=(ADCH<<8);

  // Entropiequelle aktualisieren
//  adc_random_seed+=result;

  return result;
}


int8_t decode_l (uint8_t val) {
  if (val==0x01) return -1;
  if (val==0x02) return 1;
  return 0;
}

int8_t decode_r (uint8_t val) {
  if (val==0x01) return 1;
  if (val==0x02) return -1;
  return 0;
}

int main()
{
  set_output_group(IO_MOTOR);      // Motoren aus
  activate_output_group(IO_MOTOR); // Motor bits als Output
  activate_output_group(IO_LEDS);  // LED bits als Output
  set_output_group(IO_SENS);       // Pull-ups aktivieren
  activate_output_bit(IO_LINE_EN);

  adc_init(AN_LINE_R);
  adc_init(AN_LINE_C);
  adc_init(AN_LINE_L);
  usart_setbaudrate(9600);
  usart_enable();

  while(1) {
    sei();
    int8_t left = decode_l((PINC & 0x30)>>4);
    int8_t right = decode_r((PINC & 0xc0)>>6);
    uint8_t motor = 0x00;
    
    switch (left) {
      case -1: motor |= 0x00; break;
      case +1: motor |= 0x40; break;
      case  0: motor |= 0x20; break;
    }
    
    switch (right) {
      case -1: motor |= 0x00; break;
      case +1: motor |= 0x80; break;
      case  0: motor |= 0x10; break;
    }
    
    PORTD = motor;

    clear_output_bit(IO_LINE_EN);
    _delay_ms(1);
    int16_t line_l = adc_read(AN_LINE_L);
    int16_t line_c = adc_read(AN_LINE_C);
    int16_t line_r = adc_read(AN_LINE_R);

    
    set_output_bit(IO_LINE_EN);
    _delay_ms(1);
    line_l -= adc_read(AN_LINE_L);
    line_c -= adc_read(AN_LINE_C);
    line_r -= adc_read(AN_LINE_R);

    set_output_groupbitval(IO_LEDS, L_RD, (line_l>130));
    set_output_groupbitval(IO_LEDS, R_RD, (line_r>200));
    
    set_output_groupbitval(IO_LEDS, L_YE, (PIND & 0x04));
    set_output_groupbitval(IO_LEDS, R_YE, (PIND & 0x08));

    if (usart_txempty()) {
      usart_putchar('N');
      usart_putchar('I');
      usart_putchar('B');
      usart_putchar('O');
      usart_putchar('b');
      usart_putchar('e');
      usart_putchar('e');
    }
    
  }


  return 0;
}
