/**********************************************************
 * Serial Bootloader for Atmel megaAVR Controllers        *
 *                                                        *
 * tested with ATmega16a and ATmega1284                   *
 * should work with other mega's, see code for details    *
 *                                                        *
 * ATmegaBOOT.c                                           *
 *                                                        *
 *                                                        *
 * 20150315: hacked to work with the NIBO robot platforms * 
 *           (removed monitor code, replaced Arduino      *
 *            specific changes below)                     *
 * ------------------------------------------------------ */
/* 20090308: integrated Mega changes into main bootloader */
/*           source by D. Mellis                          */
/* 20080930: hacked for Arduino Mega (with the 1280       */
/*           processor, backwards compatible)             */
/*           by D. Cuartielles                            */
/* 20070626: hacked for Arduino Diecimila (which auto-    */
/*           resets when a USB connection is made to it)  */
/*           by D. Mellis                                 */
/* 20060802: hacked for Arduino by D. Cuartielles         */
/*           based on a previous hack by D. Mellis        */
/*           and D. Cuartielles                           */
/*                                                        */
/* Monitor and debug functions were added to the original */
/* code by Dr. Erik Lins, chip45.com. (See below)         */
/*                                                        */
/* Thanks to Karl Pitrich for fixing a bootloader pin     */
/* problem and more informative LED blinking!             */
/*                                                        */
/* For the latest version see:                            */
/* http://www.chip45.com/                                 */
/*                                                        */
/* ------------------------------------------------------ */
/*                                                        */
/* based on stk500boot.c                                  */
/* Copyright (c) 2003, Jason P. Kyle                      */
/* All rights reserved.                                   */
/* see avr1.org for original file and information         */
/*                                                        */
/* This program is free software; you can redistribute it */
/* and/or modify it under the terms of the GNU General    */
/* Public License as published by the Free Software       */
/* Foundation; either version 2 of the License, or        */
/* (at your option) any later version.                    */
/*                                                        */
/* This program is distributed in the hope that it will   */
/* be useful, but WITHOUT ANY WARRANTY; without even the  */
/* implied warranty of MERCHANTABILITY or FITNESS FOR A   */
/* PARTICULAR PURPOSE.  See the GNU General Public        */
/* License for more details.                              */
/*                                                        */
/* You should have received a copy of the GNU General     */
/* Public License along with this program; if not, write  */
/* to the Free Software Foundation, Inc.,                 */
/* 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA */
/*                                                        */
/* Licence can be viewed at                               */
/* http://www.fsf.org/licenses/gpl.txt                    */
/*                                                        */
/* Target = Atmel AVR m128,m64,m32,m16,m8,m162,m163,m169, */
/* m8515,m8535. ATmega161 has a very small boot block so  */
/* isn't supported.                                       */
/*                                                        */
/* Tested with m168                                       */
/**********************************************************/

/* $Id$ */


/* some includes */
#include <inttypes.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <avr/wdt.h>
#include <util/delay.h>
#include <avr/boot.h>

/* the current avr-libc eeprom functions do not support the ATmega168 */
/* own eeprom write/read functions are used instead */
#if !defined(__AVR_ATmega168__) || !defined(__AVR_ATmega328P__)
#include <avr/eeprom.h>
#endif



#ifdef UCSRA
# define HAS_UART 1
#endif

#ifdef UCSR0A
# define HAS_UART0 1
#endif

#ifdef UCSR1A
# define HAS_UART1 1
#endif


/* Use the F_CPU defined in Makefile */

/* 20060803: hacked by DojoCorp */
/* 20070626: hacked by David A. Mellis to decrease waiting time for auto-reset */
/* set the waiting time for the bootloader */
/* get this from the Makefile instead */
/* #define MAX_TIME_COUNT (F_CPU>>4) */

/* 20070707: hacked by David A. Mellis - after this many errors give up and launch application */
#define MAX_ERROR_COUNT 5

/* set the UART baud rate */
/* 20060803: hacked by DojoCorp */
//#define BAUD_RATE   115200
#ifndef BAUD_RATE
#define BAUD_RATE   19200
#endif


/* SW_MAJOR and MINOR needs to be updated from time to time to avoid warning message from AVR Studio */
/* never allow AVR Studio to do an update !!!! */
#define HW_VER	 0x02
#define SW_MAJOR 0x01
#define SW_MINOR 0x10


/* Adjust to suit whatever pin your hardware uses to enter the bootloader */
/* ATmega128 has two UARTS so two pins are used to enter bootloader and select UART */
/* ATmega1280 has four UARTS, but for Arduino Mega, we will only use RXD0 to get code */
/* BL0... means UART0, BL1... means UART1 */
#ifdef __AVR_ATmega128__
#define BL_DDR  DDRF
#define BL_PORT PORTF
#define BL_PIN  PINF
#define BL0     PINF7
#define BL1     PINF6
#elif defined __AVR_ATmega1280__ 
/* we just don't do anything for the MEGA and enter bootloader on reset anyway*/
#else
/* other ATmegas have only one UART, so only one pin is defined to enter bootloader */
#define BL_DDR  DDRD
#define BL_PORT PORTD
#define BL_PIN  PIND
#define BL      PIND6
#endif


/* onboard LED is used to indicate, that the bootloader was entered (3x flashing) */
/* if monitor functions are included, LED goes on after monitor was entered */
#if defined __AVR_ATmega128__ || defined __AVR_ATmega1280__
/* Onboard LED is connected to pin PB7 (e.g. Crumb128, PROBOmega128, Savvy128, Arduino Mega) */
#define LED_DDR  DDRB
#define LED_PORT PORTB
#define LED_PIN  PINB
#define LED      PINB7
#else
/* Onboard LED is connected to pin PB5 in Arduino NG, Diecimila, and Duomilanuove */ 
/* other boards like e.g. Crumb8, Crumb168 are using PB2 */
#define LED_DDR  DDRB
#define LED_PORT PORTB
#define LED_PIN  PINB
#define LED      PINB2
#endif

#define LED_DBG_DDR  DDRB
#define LED_DBG_PORT PORTB
#define LED_DBG_PIN  PINB
#define LED_DBG      PINB3


// take values from <avr/io.h>
#define SIG1 SIGNATURE_0
#define SIG2 SIGNATURE_1
#define SIG3 SIGNATURE_2
#define PAGE_SIZE SPM_PAGESIZE


/* function prototypes */
void putch(char);
char getch(void);
void getNch(uint8_t);
void putstr_P (const char * s);
void byte_response(uint8_t);
void nothing_response(void);
char gethex(void);
void puthex(char);
void flash_led(uint8_t);

uint16_t get_uint16_BE();
uint16_t get_uint16_LE();

/* some variables */
/*
union address_union {
	uint16_t word;
	uint8_t  byte[2];
} address;

union length_union {
	uint16_t word;
	uint8_t  byte[2];
} length;
*/
uint16_t address, length;


struct flags_struct {
	unsigned eeprom : 1;
	unsigned rampz  : 1;
} flags;

uint8_t buff[256];
uint8_t address_high;

uint8_t pagesz=0x80;

uint8_t i;
uint8_t bootuart = 0;
uint8_t pgmmode_cnt = 0;
uint8_t special_sig = 0;

uint8_t error_count = 0;

void (*app_start)(void) = 0x0000;


/* return 0 for app start,
 * 1, 2 for uart bootloader
 */
static uint8_t check_boot() {
#ifdef WATCHDOG_MODS
	ch = MCUSR;
	MCUSR = 0;

	WDTCSR |= _BV(WDCE) | _BV(WDE);
	WDTCSR = 0;

	// Check if the WDT was used to reset, in which case we dont bootload and skip straight to the code. woot.
	if (! (ch &  _BV(EXTRF))) {
		// if its a not an external reset...
		return 0;  // skip bootloader
	}
#else
	asm volatile("nop\n\t");
#endif

	
#ifdef SYSTEM_ROBO2
	// start ADC
	ADMUX = _BV(REFS1)|_BV(REFS0)|4;
	ADCSRA = _BV(ADPS2)  // prescale faktor = 128 ADC laeuft
         | _BV(ADPS1)  // mit 15 MHz / 128 = 120 kHz
         | _BV(ADPS0)
         | _BV(ADEN)   // ADC an
         | _BV(ADSC);  // Beginne mit der Konvertierung

	_delay_ms(5);
	// measure KEY
	ADCSRA |= _BV(ADSC);
	while (ADCSRA & _BV(ADSC)) {
	}
	// if all keys are pressed, enter BL
	if (ADC<323) {
	  return 1; // enter bootloader
	}
	ADMUX = ADCSRA = 0;
#endif
	
	/* set pin direction for bootloader pin and enable pullup */
	/* for ATmega128, two pins need to be initialized */
#ifdef __AVR_ATmega128__
	BL_DDR &= ~_BV(BL0);
	BL_DDR &= ~_BV(BL1);
	BL_PORT |= _BV(BL0);
	BL_PORT |= _BV(BL1);
#else
	/* We run the bootloader regardless of the state of this pin.  Thus, don't
	put it in a different state than the other pins.  --DAM, 070709
	This also applies to Arduino Mega -- DC, 080930
	BL_DDR &= ~_BV(BL);
	BL_PORT |= _BV(BL);
	*/
#endif


#ifdef __AVR_ATmega128__
	/* check which UART should be used for booting */
	if(bit_is_clear(BL_PIN, BL0)) {
		return 1;
	}
	else if(bit_is_clear(BL_PIN, BL1)) {
		return 2;
	}
#endif

#if defined __AVR_ATmega1280__
	/* the mega1280 chip has four serial ports ... we could eventually use any of them, or not? */
	/* however, we don't wanna confuse people, to avoid making a mess, we will stick to RXD0, TXD0 */
	return 1;
#endif

	/* check if flash is programmed already, if not start bootloader anyway */
	if (pgm_read_word_near(0x0000) == 0xffff) {
		return 1;
	}

    return 0;
}



/* main program starts here */
int main(void)
{
	uint8_t ch,ch2;
	uint16_t w;

	bootuart = check_boot();
	
	if (!bootuart) {
		app_start();
	}

#ifdef HAS_MULTI_UARTS
	if (bootuart == 1) {
#endif

#ifdef HAS_UART
# ifdef DOUBLE_SPEED
	UCSRA = (1<<U2X0); //Double speed mode USART0
	UBRRL = (uint8_t)(F_CPU/(BAUD_RATE*8L)-1);
	UBRRH = (F_CPU/(BAUD_RATE*8L)-1) >> 8;
# else
	UBRRL = (uint8_t)(F_CPU/(BAUD_RATE*16L)-1);
	UBRRH = (F_CPU/(BAUD_RATE*16L)-1) >> 8;
# endif
	UCSRA = 0x00;
	UCSRB = _BV(TXEN)|_BV(RXEN);	
#endif

#ifdef HAS_UART0
# ifdef DOUBLE_SPEED
	UCSR0A = (1<<U2X0); //Double speed mode USART0
	UBRR0L = (uint8_t)(F_CPU/(BAUD_RATE*8L)-1);
	UBRR0H = (F_CPU/(BAUD_RATE*8L)-1) >> 8;
# else
	UBRR0L = (uint8_t)(F_CPU/(BAUD_RATE*16L)-1);
	UBRR0H = (F_CPU/(BAUD_RATE*16L)-1) >> 8;
# endif
	UCSR0C = 0x06;
	UCSR0B = _BV(TXEN0)|_BV(RXEN0);
#endif

#ifdef HAS_MULTI_UARTS
	} else if (bootuart == 2) {
#endif

#ifdef HAS_UART1
# ifdef DOUBLE_SPEED
	UCSR1A = (1<<U2X1); //Double speed mode USART1
	UBRR1L = (uint8_t)(F_CPU/(BAUD_RATE*8L)-1);
	UBRR1H = (F_CPU/(BAUD_RATE*8L)-1) >> 8;
# else
	UBRR1L = (uint8_t)(F_CPU/(BAUD_RATE*16L)-1);
	UBRR1H = (F_CPU/(BAUD_RATE*16L)-1) >> 8;
# endif
	UCSR1C = 0x06;
	UCSR1B = _BV(TXEN1)|_BV(RXEN1);
#endif

#ifdef HAS_MULTI_UARTS
	}
#endif



#if defined __AVR_ATmega1280__
	/* Enable internal pull-up resistor on pin D0 (RX), in order
	to supress line noise that prevents the bootloader from
	timing out (DAM: 20070509) */
	/* feature added to the Arduino Mega --DC: 080930 */
	DDRE &= ~_BV(PINE0);
	PORTE |= _BV(PINE0);
#endif

	// Enable D0 (RxD) Pull Up
	PORTD |= _BV(0);
	

	/* set LED pin as output */
	LED_DDR |= _BV(LED);

	/* set LED_DBG pin as output */
	LED_DBG_DDR |= _BV(LED_DBG);

	/* flash onboard LED to signal entering of bootloader */
#if defined(__AVR_ATmega128__) || defined(__AVR_ATmega1280__)
	// 4x for UART0, 5x for UART1
	flash_led(NUM_LED_FLASHES + bootuart);
#else
	flash_led(NUM_LED_FLASHES);
#endif

	/*
	putch('O');
	putch('K');
	putch('\n');
	putch('\r');
	while (1) {
	  putch(getch());
	}
	*/
	
	/* enable access to application code */
	boot_rww_enable();	
	/* 20050803: by DojoCorp, this is one of the parts provoking the
		 system to stop listening, cancelled from the original */
	//putch('\0');

	/* forever loop */
	for (;;) {

	/* get character from UART */
	ch = getch();

	/* A bunch of if...else if... gives smaller code than switch...case ! */

	if (special_sig) {
	  special_sig++;
	  if (special_sig>2) {
	    special_sig=0;
	  }
	}
	
	
	/* Hello is anyone home ? */ 
	if(ch=='0') {
		nothing_response();
	}


	/* Request programmer ID */
	/* Not using PROGMEM string due to boot block in m128 being beyond 64kB boundry  */
	/* Would need to selectively manipulate RAMPZ, and it's only 9 characters anyway so who cares.  */
	else if(ch=='1') {
		if (getch() == ' ') {
			putch(0x14);
			putstr_P(PSTR("AVR ISP"));
			if (special_sig==2) {
				putstr_P(PSTR("@NIBO-burger-m16-15"));
			} else {
				special_sig = 1;
			}
			putch(0x10);
		} else {
			if (++error_count == MAX_ERROR_COUNT)
				app_start();
		}
	}


	/* AVR ISP/STK500 board commands  DON'T CARE so default nothing_response */
	else if(ch=='@') {
		ch2 = getch();
		if (ch2>0x85) getch();
		nothing_response();
	}


	/* AVR ISP/STK500 board requests */
	else if(ch=='A') {
		ch2 = getch();
		if(ch2==0x80) byte_response(HW_VER);		// Hardware version
		else if(ch2==0x81) byte_response(SW_MAJOR);	// Software major version
		else if(ch2==0x82) byte_response(SW_MINOR);	// Software minor version
		else if(ch2==0x98) byte_response(0x03);		// Unknown but seems to be required by avr studio 3.56
		else byte_response(0x00);				// Covers various unnecessary responses we don't care about
	}


	/* Device Parameters  DON'T CARE, DEVICE IS FIXED  */
	else if(ch=='B') {
		getNch(20);
		nothing_response();
	}


	/* Parallel programming stuff  DON'T CARE  */
	else if(ch=='E') {
		getNch(5);
		nothing_response();
	}

	/* R: Erase device, don't care as we will erase one page at a time anyway.  */
	else if(ch=='R') {
		nothing_response();
	}

	/* P: Enter programming mode  */
	else if(ch=='P') {
		pgmmode_cnt++;
		nothing_response();
	}

	/* Leave programming mode  */
	else if(ch=='Q') {
		if (pgmmode_cnt==0) {
			app_start();
		}
		pgmmode_cnt--;
		nothing_response();
#ifdef WATCHDOG_MODS
		// autoreset via watchdog (sneaky!)
		WDTCSR = _BV(WDE);
		while (1); // 16 ms
#endif
	}


	/* Set address, little endian. EEPROM in bytes, FLASH in words  */
	/* Perhaps extra address bytes may be added in future to support > 128kB FLASH.  */
	/* This might explain why little endian was used here, big endian used everywhere else.  */
	else if(ch=='U') {
		address = get_uint16_LE();
		nothing_response();
	}


	/* Universal SPI programming command, disabled.  Would be used for fuses and lock bits.  */
	else if(ch=='V') {
		if (getch() == 0x30) {
			getch();
			ch = getch();
			getch();
#ifdef SIGRD
			byte_response(boot_signature_byte_get(ch));
#else			
			if (ch == 0) {
				byte_response(SIG1);
			} else if (ch == 1) {
				byte_response(SIG2); 
			} else {
				byte_response(SIG3);
			} 
#endif
		} else {
			getNch(3);
			byte_response(0x00);
		}
	}


	/* Write memory, length is big endian and is in bytes  */
	else if(ch=='d') {
		length = get_uint16_BE();

		flags.eeprom = 0;
		if (getch() == 'E') flags.eeprom = 1;
		for (w=0;w<length;w++) {
			buff[w] = getch();	                        // Store data in buffer, can't keep up with serial data stream whilst programming pages
		}
		if (getch() == ' ') {
			if (flags.eeprom) {		                //Write to EEPROM one byte at a time
				address <<= 1;
				for(w=0;w<length;w++) {
#if defined(__AVR_ATmega168__)  || defined(__AVR_ATmega328P__)
					while(EECR & (1<<EEPE));
					EEAR = (uint16_t)(void *)address;
					EEDR = buff[w];
					EECR |= (1<<EEMPE);
					EECR |= (1<<EEPE);
#else
					eeprom_write_byte((void *)address,buff[w]);
#endif
					address++;
				}			
			}
			else {					        //Write to FLASH one page at a time
				if (address&0x8000) address_high = 0x01;	//Only possible with m128, m256 will need 3rd address byte. FIXME
				else address_high = 0x00;
#if defined(__AVR_ATmega128__) || defined(__AVR_ATmega1280__) || defined(__AVR_ATmega1281__)
				RAMPZ = address_high;
#endif
				address = address << 1;	        //address * 2 -> byte location
				/* if ((length.byte[0] & 0x01) == 0x01) length.word++;	//Even up an odd number of bytes */
				if (length & 0x01) length++;	//Even up an odd number of bytes
				cli();				//Disable interrupts, just to be sure

				LED_DBG_PORT |= _BV(LED_DBG);
				uint16_t * bufp = (uint16_t *)buff;
				while (length) {
				  uint16_t paddress = address;
				  boot_page_erase_safe(paddress);
				  boot_spm_busy_wait();
				  uint8_t cnt = (length>PAGE_SIZE)? PAGE_SIZE : length;
				  length -= cnt;
				  cnt /= 2;
				  while (cnt--) {
				    boot_page_fill(address, *bufp++);
				    address += 2;
				  }
				  boot_page_write(paddress);
				  boot_spm_busy_wait();
				}
				boot_rww_enable();
				LED_DBG_PORT &= ~_BV(LED_DBG);
			}
			
			putch(0x14);
			putch(0x10);
		} else {
			if (++error_count == MAX_ERROR_COUNT)
				app_start();
		}		
	}


	/* Read memory block mode, length is big endian.  */
	else if(ch=='t') {
		length = get_uint16_BE();
#if defined(__AVR_ATmega128__) || defined(__AVR_ATmega1280__)
		if (address>0x7FFF) flags.rampz = 1;		// No go with m256, FIXME
		else flags.rampz = 0;
#endif
		address = address << 1;	        // address * 2 -> byte location
		if (getch() == 'E') flags.eeprom = 1;
		else flags.eeprom = 0;
		if (getch() == ' ') {		                // Command terminator
			putch(0x14);
			for (w=0;w < length;w++) {		        // Can handle odd and even lengths okay
				if (flags.eeprom) {	                        // Byte access EEPROM read
#if defined(__AVR_ATmega168__)  || defined(__AVR_ATmega328P__)
					while(EECR & (1<<EEPE));
					EEAR = (uint16_t)(void *)address;
					EECR |= (1<<EERE);
					putch(EEDR);
#else
					putch(eeprom_read_byte((void *)address));
#endif
					address++;
				}
				else {

					if (!flags.rampz) putch(pgm_read_byte_near(address));
#if defined(__AVR_ATmega128__) || defined(__AVR_ATmega1280__)
					else putch(pgm_read_byte_far(address + 0x10000));
					// Hmmmm, yuck  FIXME when m256 arrvies
#endif
					address++;
				}
			}
			putch(0x10);
		}
	}


	/* Get device signature bytes  */
	else if(ch=='u') {
		if (getch() == ' ') {
			putch(0x14);
#ifdef SIGRD
			putch(boot_signature_byte_get(0));
			putch(boot_signature_byte_get(1));
			putch(boot_signature_byte_get(2));
#else		
			putch(SIGNATURE_0);
			putch(SIGNATURE_1);
			putch(SIGNATURE_2);
#endif
			putch(0x10);
		} else {
			if (++error_count == MAX_ERROR_COUNT)
				app_start();
		}
	}


	/* Read oscillator calibration byte */
	else if(ch=='v') {
		byte_response(0x00);
	}



	else if (++error_count == MAX_ERROR_COUNT) {
		app_start();
	}
	} /* end of forever loop */

}


char gethexnib(void) {
	char a;
	a = getch(); putch(a);
	if(a >= 'a') {
		return (a - 'a' + 0x0a);
	} else if(a >= '0') {
		return(a - '0');
	}
	return a;
}


char gethex(void) {
	return (gethexnib() << 4) + gethexnib();
}


void puthex(char ch) {
	char ah;

	ah = ch >> 4;
	if(ah >= 0x0a) {
		ah = ah - 0x0a + 'a';
	} else {
		ah += '0';
	}
	
	ch &= 0x0f;
	if(ch >= 0x0a) {
		ch = ch - 0x0a + 'a';
	} else {
		ch += '0';
	}
	
	putch(ah);
	putch(ch);
}

#if FLASHEND > 0x0FFFF
void putstr_P (const char * s) {
  char c;
  do {
    c = pgm_read_byte_far((uint32_t)(s++) | 0x10000);
    if (c) {
      putch(c);
    }
  } while (c);
}

#else
void putstr_P (const char * s) {
  char c;
  do {
    c = pgm_read_byte_near(s++);
    if (c) {
      putch(c);
    }
  } while (c);
}
#endif

void putch (char ch) {
#ifdef HAS_MULTI_UART
	if (bootuart == 1) {
#endif

#ifdef HAS_UART
	while (!(UCSRA & _BV(UDRE)));
	UDR = ch;
#endif

#ifdef HAS_UART0
	while (!(UCSR0A & _BV(UDRE0)));
	UDR0 = ch;
#endif
	  
#ifdef HAS_MULTI_UART
	} else if (bootuart == 2) {
#endif

#ifdef HAS_UART1
	while (!(UCSR1A & _BV(UDRE1)));
	UDR1 = ch;
#endif
  
#ifdef HAS_MULTI_UART
	}
#endif

  
}


char getch(void) {
#ifdef USE_TIMEOUT
	uint32_t count = 0;
#endif

#ifdef HAS_MULTI_UART
	if (bootuart == 1) {
#endif
  
#ifdef HAS_UART
	while (!(UCSRA & _BV(RXC))) {
#ifdef USE_TIMEOUT		
		count++;
		if (count > MAX_TIME_COUNT)
			app_start();
#endif
	}
	return UDR;
#endif  

#ifdef HAS_UART0
	while (!(UCSR0A & _BV(RXC0))) {
#ifdef USE_TIMEOUT		
		count++;
		if (count > MAX_TIME_COUNT)
			app_start();
#endif
	}
	return UDR0;
#endif  

#ifdef HAS_MULTI_UART
	} else if (bootuart == 2) {
#endif
  
#ifdef HAS_UART1
	while (!(UCSR1A & _BV(RXC1))) {
#ifdef USE_TIMEOUT		
		count++;
		if (count > MAX_TIME_COUNT)
			app_start();
#endif
	}
	return UDR1;
#endif

#ifdef HAS_MULTI_UART
	}
#endif

}


void getNch (uint8_t count) {
	while(count--) {
		getch();
	}
}

uint16_t get_uint16_BE() {
  uint16_t val = ((uint16_t)(uint8_t)getch())<<8;
  val |= (uint8_t)getch();
  return val;
}

uint16_t get_uint16_LE() {
  uint16_t val = (uint8_t)getch();
  val |= ((uint16_t)(uint8_t)getch())<<8;
  return val;
}


void byte_response(uint8_t val)
{
	if (getch() == ' ') {
		putch(0x14);
		putch(val);
		putch(0x10);
	} else {
		if (++error_count == MAX_ERROR_COUNT)
			app_start();
	}
}


void nothing_response(void)
{
	if (getch() == ' ') {
		putch(0x14);
		putch(0x10);
	} else {
		if (++error_count == MAX_ERROR_COUNT)
			app_start();
	}
}

void flash_led(uint8_t count)
{
	while (count--) {
		LED_PORT |= _BV(LED);
		_delay_ms(100);
		LED_PORT &= ~_BV(LED);
		_delay_ms(100);
	}
}


/* end of file ATmegaBOOT.c */
