/*
  Programm zum ersten Testen des NIBObee Roboters
*/


#include <niboburger/base.h>
#include <niboburger/iodefs.h>
#include <niboburger/usart.h>
#include <niboburger/motpwm.h>
#include <niboburger/motpid.h>
#include <niboburger/analog.h>
#include <niboburger/odometry.h>
#include <niboburger/led.h>
#include <niboburger/key.h>
#include <niboburger/delay.h>
#include <niboburger/surface.h>
#include <niboburger/color.h>

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <string.h>
#include <stdlib.h>

//#define SPEED 500
#define SPEED 1023


#define MAROON_CLEAR() "\33c"
#define MAROON_BRIGHT(x) "\33b" #x
#define MAROON_DIM(x) "\33d" #x
#define MAROON_TXBACK(c) "\33t" c
#define MAROON_LOAD() "\33l"

#define MAROON_BAR(bar) "\33B" bar " "
#define MAROON_GFX(gfx) "\33G" gfx " "
#define MAROON_PAUSE(x) "\33P" #x " "
#define MAROON_STIME(x) "\33S" #x " "
#define MAROON_DTIME(x) "\33D" #x " "
#define MAROON_TRANSITION(n, x) "\33T" #n #x " "

#define MAROON_IMM_CLEAR() "\20c"
#define MAROON_IMM_DIM(x) "\20d" #x



#ifndef min
#define min(a,b) \
   ({ __typeof__ (a) _a = (a); \
       __typeof__ (b) _b = (b); \
     _a < _b ? _a : _b; })
#endif

#define min3(a,b,c) min(min(a,b),c) 


uint8_t progmode = 0;
uint8_t has_maroon = 0;
uint8_t follow_speed = 1;
enum {
  EVENT_NONE              =  0,
  EVENT_KEY1              =  1,
  EVENT_KEY2              =  2,
  EVENT_KEY3              =  3,
  EVENT_TIMEOUT           =  4,
  
  EVENT_OBSTACLE_DETECTED = 16,
  EVENT_OBSTACLE_CLEAR    = 17,
  EVENT_OBSTACLE_CLEAR_L  = 18,
  EVENT_OBSTACLE_CLEAR_R  = 19,
  
  EVENT_NO_HAND           = 20,
  EVENT_TO_CLOSE          = 21,
  EVENT_HAND_C            = 22,
  EVENT_HAND_L            = 23,
  EVENT_HAND_R            = 24,  
  EVENT_HAND_LL           = 25,
  EVENT_HAND_RR           = 26,
  
  EVENT_NICK              = 31
  
};


uint8_t cops1[8] = {
  0x10,
  0x20,
  0x40,
  0x80,
  0x00,
  0x00,
  0x00,
  0x00
};

uint8_t cops2[8] = {
  0x30,
  0x30,
  0x30,
  0xc0,
  0xc0,
  0xc0,
  0x00,
  0x00
};

uint8_t key_getEvent();
  

void blink_led(uint8_t led, uint8_t count) {
  while (count--) {
    led_set(led, 1);
    delay(80);
    led_set(led, 0);
    delay(120);
  }
}

void blink_led_fast(uint8_t led, uint8_t count) {
  while (count--) {
    led_set(led, 1);
    delay(40);
    led_set(led, 0);
    delay(60);
  }
}


void set_led_byte(uint8_t byte) {
  IO_LEDS_PORT = (IO_LEDS_PORT & ~IO_LEDS_MASK) | ((byte << IO_LEDS_BIT_1) & IO_LEDS_MASK);
}


void blink_byte(uint8_t byte) {
  IO_LEDS_PORT = (IO_LEDS_PORT & ~IO_LEDS_MASK) | ((byte << IO_LEDS_BIT_1) & IO_LEDS_MASK);
  delay(50);  
  IO_LEDS_PORT = (IO_LEDS_PORT & ~IO_LEDS_MASK) | ((byte >> (4-IO_LEDS_BIT_1)) & IO_LEDS_MASK);
  delay(50);
}

uint8_t do_cops1() {
  uint8_t i, e;
  for (i=0; i<4; i++) {
    blink_byte(cops1[i]);
    e = key_getEvent();
    if ((e>=EVENT_KEY1)&&(e<=EVENT_KEY3)) {
      blink_byte(0);
      return e;
    }
  }
  return EVENT_NONE;
}

uint8_t do_cops2() {
  uint8_t i, e;
  for (i=0; i<7; i++) {
    blink_byte(cops2[i]);
    e = key_getEvent();
    if ((e>=EVENT_KEY1)&&(e<=EVENT_KEY3)) {
      blink_byte(0);
      return e;
    }
  }
  return EVENT_NONE;
}

void do_cops() {
  uint8_t i;
  while (1) {
    for (i=0; i<2; i++) {
      if (do_cops2() != EVENT_NONE) {
        blink_byte(0);
        return;
      }
    }
    for (i=0; i<3; i++) {
      if (do_cops1() != EVENT_NONE) {
        blink_byte(0);
        return;
      }
    }
  }
  blink_byte(0);
}

uint8_t check_nick() {
  uint16_t l = analog_getValueExt(ANALOG_BL, 2);
  uint16_t r = analog_getValueExt(ANALOG_BR, 2);
  uint16_t c = analog_getValueExt(ANALOG_BC, 2);
  if (l>280) return 1;
  if (r>280) return 1;
  if (c>280) return 1;
  return 0;
}


/* OBSTACLE */

int8_t obstacle_pos;
uint8_t obstacle_val;

uint8_t getObstSensorValue(uint8_t sensor) {
  uint16_t val = analog_getValueExt(sensor, 2);
  if (val&0xff00) return 0xff;
  return val;
}

uint8_t follow_getEvent() {
  static uint16_t f_ll = 0;
  static uint16_t f_rr = 0;
  static uint16_t f_cc = 0;
  static uint8_t state = 0;
  uint8_t new_follow_speed = follow_speed;
  uint8_t nstate = 0;
  uint8_t l = getObstSensorValue(ANALOG_FL)>>1;
  uint8_t r = getObstSensorValue(ANALOG_FR)>>1;
  uint8_t _ll = getObstSensorValue(ANALOG_FLL)>>1;
  uint8_t _rr = getObstSensorValue(ANALOG_FRR)>>1;
  uint8_t _cc = l + r + 2;
  _ll += l;
  _rr += r;
  
  f_ll = (3*f_ll+_ll+_ll+_ll+_ll + 2)/4;
  f_rr = (3*f_rr+_rr+_rr+_rr+_rr + 2)/4;
  f_cc = (3*f_cc+_cc+_cc+_cc+_cc + 2)/4;

  uint8_t ll = f_ll/4;
  uint8_t rr = f_rr/4;
  uint8_t cc = f_cc/4;
  
 
  obstacle_val = (ll>rr)?ll:rr;
  if (cc>obstacle_val) {
    obstacle_val = cc;
  }
  
  if ((obstacle_val==cc) || (ll==rr)) {
    if (ll>rr) {
      obstacle_pos = -1;
      nstate = EVENT_HAND_L;
    } else if (ll<rr) {
      obstacle_pos = +1;
      nstate = EVENT_HAND_R;
    } else {
      obstacle_pos = 0;
      nstate = EVENT_HAND_C;
    }
  } else if (ll>rr) {
    obstacle_pos = -2;
    nstate = EVENT_HAND_LL;
    obstacle_val = 50;
  } else {
    obstacle_pos = +2;
    nstate = EVENT_HAND_RR;
    obstacle_val = 50;
  }

  if (obstacle_val>50) {
    nstate = EVENT_TO_CLOSE;
  } else if (obstacle_val<5) {
    nstate = EVENT_NO_HAND;
  } else if (obstacle_val>15) {
    follow_speed=1;
  } else if (obstacle_val>10) {
    follow_speed=2;
  } else {
    follow_speed=3;    
  }

  if (check_nick()) {
    nstate = EVENT_NICK;
  }  
  
  if (state!=nstate) {
    state = nstate;
    return state;
  }
  
  if (new_follow_speed!=follow_speed) {
    return state;    
  }
 
  return EVENT_NONE;
}


uint8_t obstacle_getEvent() {
  static uint8_t event_last = EVENT_NONE;
  uint8_t event = EVENT_NONE;
  uint8_t l = getObstSensorValue(ANALOG_FL)>>1;
  uint8_t r = getObstSensorValue(ANALOG_FR)>>1;
  uint8_t ll = getObstSensorValue(ANALOG_FLL)>>1;
  uint8_t rr = getObstSensorValue(ANALOG_FRR)>>1;
  uint8_t cc = l + r;
  ll += l;
  rr += r;
 
  obstacle_val = (ll>rr)?ll:rr;
  if (cc>obstacle_val) {
    obstacle_val = cc;
  }
  
  if ((obstacle_val==cc) || (ll==rr)) {
    if (ll>rr) {
      obstacle_pos = -1;
    } else if (ll<rr) {
      obstacle_pos = +1;
    } else {
      obstacle_pos = 0;
    }
  } else if (ll>rr) {
    obstacle_pos = -2;
  } else {
    obstacle_pos = +2;
  }
 

  if (obstacle_val>25) {
    event = EVENT_OBSTACLE_DETECTED;
  }
  
  if (obstacle_val<20) {
    event = EVENT_OBSTACLE_CLEAR;
    if (obstacle_val>5) {
      if (obstacle_pos<0) {
        event = EVENT_OBSTACLE_CLEAR_R;
      } else if (obstacle_pos>0) {
        event = EVENT_OBSTACLE_CLEAR_L;
      }
    }
  } 
  
  if (check_nick()) {
    event = EVENT_NICK;
  }
  
  if (event!=event_last) {
    event_last = event;
    return event;
  }
  
  return EVENT_NONE;
}

/* KEY */

uint8_t key_getEvent() {
  static uint8_t key = 0;
  uint8_t act = key_get_state();
  if (act==0) {
    act = key;
    key = 0;
    if (act==0x01) return EVENT_KEY1;
    if (act==0x02) return EVENT_KEY2;
    if (act==0x04) return EVENT_KEY3;
    return EVENT_NONE;
  }
  if (act!=KEY_STATE_INVALID) {
    key |= act;  
  }
  return EVENT_NONE;
}


uint8_t getEvent() {
  uint8_t event = EVENT_NONE;
  event = key_getEvent();
  if (event) return event;
  
  if (progmode==2) {
    event = obstacle_getEvent();
  } else if (progmode==1) {
    event = follow_getEvent();
  }
  return event;
}


int8_t decode_l (uint8_t val) {
  if (val==0x01) return -1;
  if (val==0x02) return 1;
  return 0;
}

int8_t decode_r (uint8_t val) {
  if (val==0x01) return -1;
  if (val==0x02) return 1;
  return 0;
}


void print_color (uint16_t val) {
  char r = (val>>8)&0x0f;
  char g = (val>>4)&0x0f;
  char b = (val>>0)&0x0f;
  if (r<10) r+='0'; else r+='a'-10;
  if (g<10) g+='0'; else g+='a'-10;
  if (b<10) b+='0'; else b+='a'-10;
  usart_putchar('#');
  usart_putchar(r);
  usart_putchar(g);
  usart_putchar(b);
}


void print_hex (uint8_t val) {
  char c1=val/16;
  char c2=val%16;
  if (c1<10) c1+='0'; else c1+='a'-10;
  if (c2<10) c2+='0'; else c2+='a'-10;
  usart_putchar(c1);
  usart_putchar(c2);
}


void print_decimal2(uint8_t val) {
  uint8_t t=0;
  while (val>9) {
    t++;
    val-=10;
  }
  usart_putchar('0'+t);
  usart_putchar('0'+val);
}


void print_color_hsv2 (uint16_t color) {
  int8_t h = (int8_t)((color >> 10) & 0x1f); // [0..31]
  int8_t s = (int8_t)((color >>  5) & 0x1f); // [0..31]
  int8_t v = (int8_t)((color >>  0) & 0x1f); // [0..31]

  usart_putchar('!');
  print_decimal2(h);
  usart_putchar('.');
  print_decimal2(s);
  usart_putchar('.');
  print_decimal2(v);
}

void print_color_rgb (uint32_t rgb) {
  uint8_t r = (uint8_t)((rgb >> 16) & 0xff);
  uint8_t g = (uint8_t)((rgb >>  8) & 0xff);
  uint8_t b = (uint8_t)((rgb >>  0) & 0xff);
  usart_putchar('#');
  print_hex(r);
  print_hex(g);
  print_hex(b);
}



void print_text (char * text) {
  char c;
  while ((c=*text++)) {
    while (usart_txfull()) {
      /* wait */
    }
    usart_putchar(c);
  }
}


uint16_t get_sensor(uint8_t s) {
  int16_t val = analog_getValueExt(s, 1);
  val -= analog_getValueExt(s, 0);
  if (val<0) val = 0;
  if (val>1023) val = 1023;
  return val;
}


uint8_t run = 0;
uint16_t counter = 0;


void setup_follow_me();
void setup_avoid_obst();
void setup_calibrate();

void loop_startup();
void loop_follow_me();
void loop_avoid_obst();
void loop_calibrate();

void sendText(const char * txt) {
  char c;
  while ((c=*txt++)) {
    while (usart_txfull()) {
      // wait...
    }
    usart_putchar(c);
  }
}

int main() {
  activate_output_group(IO_LEDS);  // LED bits als Output
  //usart_setbaudrate(9600);
  usart_setbaudrate(38400);
  usart_enable();
  motpwm_init();
  motpid_init();
  odometry_init();
  //motpwm_setLeft(0);
  //motpwm_setRight(0);
  analog_init();
  sei();
  surface_init();
  
  analog_setExtToggleMode(7, 1);
  
  delay(100);
  delay(100);
  sendText(MAROON_TXBACK(":") MAROON_TXBACK(":") "Please select:\n" MAROON_TXBACK("#"));
  //sendText(MAROON_IMM_CLEAR() "Hello!\nPlease select:\n" MAROON_TXBACK("#"));
  //sendText(MAROON_IMM_CLEAR() "Hello!\n");
  //motpwm_setLeft(-500);
  //motpwm_setRight(500);
  //motpid_setSpeed(+20,+20);  
  
  while(1) {
    nibo_checkMonitorVoltage();
    switch (progmode) {
      case 0:
        loop_startup();
        break;
      case 1:
        loop_follow_me();
        break;
      case 2:
        loop_avoid_obst();
        break;
      case 3:
        loop_calibrate();
        break;
    }
  }
}


void loop_startup() {
    if (counter==10) {
      led_set(1,1);
    } else if (counter==20) {
      led_set(1,0);
      led_set(2,1);      
    } else if (counter==30) {
      led_set(2,0);
      led_set(3,1);      
    } else if (counter==40) {
      led_set(3,0);
      led_set(4,1);            
    } else if (counter==50) {
      led_set(4,0);
    } else if (counter==60) {
      led_set(1,1);
    } else if (counter==70) {
      led_set(1,0);
      led_set(2,1);      
    } else if (counter==80) {
      led_set(2,0);
      led_set(3,1);      
    } else if (counter==90) {
      led_set(3,0);
      led_set(4,1);            
    } else if (counter==100) {
      led_set(4,0);
      
    } else if (counter==410) {
      led_set(4,1);
    } else if (counter==420) {
      led_set(4,0);
      led_set(3,1);      
    } else if (counter==430) {
      led_set(3,0);
      led_set(2,1);      
    } else if (counter==440) {
      led_set(2,0);
      led_set(1,1);            
    } else if (counter==450) {
      led_set(1,0);
    } else if (counter==460) {
      led_set(4,1);
    } else if (counter==470) {
      led_set(4,0);
      led_set(3,1);      
    } else if (counter==480) {
      led_set(3,0);
      led_set(2,1);      
    } else if (counter==490) {
      led_set(2,0);
      led_set(1,1);            
    } else if (counter==500) {
      led_set(1,0);
      
    } else if (counter>=800) {
      counter = has_maroon ? 800 : 0;
    }
    counter++;
    analog_wait_update();
    uint8_t event = getEvent();
    if (event==EVENT_KEY1) {
      setup_follow_me();
    } else if (event==EVENT_KEY2) {
      setup_avoid_obst();
    } else if (event==EVENT_KEY3) {
      setup_calibrate();
    }
    if (!usart_rxempty()) {
      uint8_t c = usart_getchar();
      if (c==':') {
        has_maroon = 1;
      } else if (c=='#') {
        sendText(MAROON_BRIGHT(0) MAROON_LOAD() "1" MAROON_BAR("00") MAROON_DIM(*) MAROON_PAUSE(100) MAROON_DIM(5) " Follow me\n"
                 MAROON_BRIGHT(0) MAROON_LOAD() "2" MAROON_BAR("0") MAROON_DIM(*) MAROON_PAUSE(100) MAROON_DIM(5) " Fraidy-cat\n"
                 MAROON_BRIGHT(0) MAROON_LOAD() "3" MAROON_BAR("00") MAROON_DIM(*) MAROON_PAUSE(100) MAROON_DIM(5) " Colour detection\n"
                 MAROON_TXBACK("#"));
      }
    }
}

char getSensorChar(int16_t val) {
  if (val<5) return 'a';
  if (val<10) return 'b';
  if (val<15) return 'c';
  if (val<25) return 'd';
  if (val<35) return 'e';
  if (val<50) return 'f';
  if (val<70) return 'g';
  if (val<100) return 'h';
  return 'i';
}

uint16_t maroon_cnt;
uint8_t maroon_mode;
char maroon_gfxdata[] = MAROON_LOAD() MAROON_BAR("00000000");
#define MAROON_BAR_OFFSET 4

static void do_maroon() {
  if (!usart_rxempty()) {
    char c = usart_getchar();
    if (c=='.') {
      maroon_mode = 1;
    }
  }
  if (maroon_mode) {
    if (maroon_cnt) {
      maroon_cnt--;
      return;
    }
    maroon_cnt=20;
    if (usart_txempty()) {
      int16_t l = analog_getValueExt(ANALOG_FL, 2);
      int16_t r = analog_getValueExt(ANALOG_FR, 2);
      int16_t ll = analog_getValueExt(ANALOG_FLL, 2);
      int16_t rr = analog_getValueExt(ANALOG_FRR, 2);   
      
      maroon_gfxdata[MAROON_BAR_OFFSET+0] = getSensorChar(rr);
      maroon_gfxdata[MAROON_BAR_OFFSET+1] = '0';//getSensorChar(rr);
      maroon_gfxdata[MAROON_BAR_OFFSET+2] = getSensorChar(r);
      maroon_gfxdata[MAROON_BAR_OFFSET+3] = '0';//getSensorChar(r);
      maroon_gfxdata[MAROON_BAR_OFFSET+4] = '0';//getSensorChar(l);
      maroon_gfxdata[MAROON_BAR_OFFSET+5] = getSensorChar(l);
      maroon_gfxdata[MAROON_BAR_OFFSET+6] = '0';//getSensorChar(ll);
      maroon_gfxdata[MAROON_BAR_OFFSET+7] = getSensorChar(ll);
      print_text(maroon_gfxdata);
    }
  }
}

char getSensorChar2(int16_t val) {
  if (val<114) return 'a';
  if (val<227) return 'b';
  if (val<341) return 'c';
  if (val<454) return 'd';
  if (val<568) return 'e';
  if (val<682) return 'f';
  if (val<796) return 'g';
  if (val<909) return 'h';
  return 'i';
}

char getSensorChar3(int16_t val) {
  if (val<-300) return '1';
  if (val<-200) return '2';
  if (val<-100) return '3';
  if (val<0) return '4';
  if (val<100) return '5';
  if (val<200) return '6';
  if (val<300) return '7';
  return '8';
}


static void do_maroon_col() {
  if (!usart_rxempty()) {
    char c = usart_getchar();
    if (c=='.') {
      maroon_mode = 1;
    }
  }
  if (maroon_mode) {
    if (maroon_cnt) {
      maroon_cnt--;
      return;
    }
    maroon_cnt=20;
    if (usart_txempty()) {
      //uint16_t l = analog_getValueExt(ANALOG_BL, 2);
      //uint16_t r = analog_getValueExt(ANALOG_BR, 2);
      //uint16_t c = analog_getValueExt(ANALOG_BC, 2);      
      
      int16_t r = surface_get(SURFACE_R);
      int16_t g = surface_get(SURFACE_C);
      int16_t b = surface_get(SURFACE_L);
      
      int16_t rg = (r-g);
      int16_t by = (b-(r+g)/2);
      
      maroon_gfxdata[MAROON_BAR_OFFSET+0] = getSensorChar2(r);
      maroon_gfxdata[MAROON_BAR_OFFSET+1] = getSensorChar2(r);
      maroon_gfxdata[MAROON_BAR_OFFSET+2] = getSensorChar3(rg);
      maroon_gfxdata[MAROON_BAR_OFFSET+3] = getSensorChar2(g);
      maroon_gfxdata[MAROON_BAR_OFFSET+4] = getSensorChar2(g);
      maroon_gfxdata[MAROON_BAR_OFFSET+5] = getSensorChar3(by);
      maroon_gfxdata[MAROON_BAR_OFFSET+6] = getSensorChar2(b);
      maroon_gfxdata[MAROON_BAR_OFFSET+7] = getSensorChar2(b);
      print_text(maroon_gfxdata);
    }
  }
}

uint8_t do_maroon_process(uint8_t pos) {
  if (!usart_rxempty()) {
    char c = usart_getchar();
    if (c=='.') {
      maroon_mode = 1;
    }
  }
  if (!maroon_mode) {
    return 0;
  }
  
  if (maroon_cnt) {
    maroon_cnt--;
    return 0;
  }
  maroon_cnt=20;

  memset(maroon_gfxdata+MAROON_BAR_OFFSET, '0', 8);
  if (pos==0) {
    ;
  } else if (pos==1) {
    maroon_gfxdata[MAROON_BAR_OFFSET+0] = 'M';
    maroon_gfxdata[MAROON_BAR_OFFSET+7] = 'M';    
  } else if (pos<8) {
    pos-=2;
    maroon_gfxdata[MAROON_BAR_OFFSET+1+pos] = '1';
    maroon_gfxdata[MAROON_BAR_OFFSET+7] = '2'+pos;
    maroon_gfxdata[MAROON_BAR_OFFSET+6-pos] = '8';
    maroon_gfxdata[MAROON_BAR_OFFSET+0] = '7'-pos;
  } else if (pos==8) {
    maroon_gfxdata[MAROON_BAR_OFFSET+1] = 'L';
    maroon_gfxdata[MAROON_BAR_OFFSET+6] = 'L';
  } else if (pos==9) {
    maroon_gfxdata[MAROON_BAR_OFFSET+2] = 'K';
    maroon_gfxdata[MAROON_BAR_OFFSET+5] = 'K';
  } else if (pos==10) {
    maroon_gfxdata[MAROON_BAR_OFFSET+3] = 'J';
    maroon_gfxdata[MAROON_BAR_OFFSET+4] = 'J';
  }     
    
    
  if (pos==1) {
    print_text(MAROON_BRIGHT(1));
  } else if (pos==8) {
    print_text(MAROON_BRIGHT(6));
  } else if (pos==9) {
    print_text(MAROON_BRIGHT(9));
  } else if (pos==10) {
    print_text(MAROON_BRIGHT(*));
  }
  print_text(maroon_gfxdata);
  return 1;
}


void denick_procedure() {
  motpid_release(1);
  _delay_ms(100);
  led_setall(1, 0, 0, 1);
  motpwm_setLeft(-1024);
  motpwm_setRight(-1024);
  _delay_ms(400);
  led_setall(0, 1, 1, 0);
  motpwm_setLeft(+1024);
  motpwm_setRight(+1024);
  _delay_ms(450);
  led_setall(1, 1, 1, 1);
  motpwm_setLeft(0);
  motpwm_setRight(0);  
  _delay_ms(200);
  motpid_init();
}

void setup_follow_me() {
  maroon_mode=0;
  sendText(MAROON_IMM_CLEAR() MAROON_BRIGHT(0) MAROON_LOAD() "1" MAROON_BAR("00") MAROON_DIM(*) MAROON_PAUSE(100) MAROON_DIM(5) MAROON_STIME(60) " Follow\n" MAROON_TXBACK("."));
  //sendText(MAROON_IMM_CLEAR() MAROON_LOAD() MAROON_BAR("AAABBAAA") MAROON_PAUSE(500) MAROON_STIME(60) "Follow\n" MAROON_TXBACK("."));
  motpid_stop(1);
  blink_led(1, 4);
  progmode = 1;  
  run = 0;
  counter = 0;
}


static void follow_setSpeed(int16_t l, int16_t r) {
  if (follow_speed==1) {
    l/=2;
    r/=2;
  } else if (follow_speed==2) {
    
  } else if (follow_speed==3) {
    l+=l/2;
    r+=r/2;
  }
  motpid_setSpeed(l, r);
}


void loop_follow_me() {
  while(1) {
    analog_wait_update();
    do_maroon();
    uint8_t event = getEvent();
    
    int8_t left = 0;
    int8_t right = 0;
    
    uint16_t val;   
    
    if (event==EVENT_KEY1) {
        run = 0;
	//print_text("KEY1");
	motpid_stop(1);
	sendText(MAROON_IMM_CLEAR() MAROON_BRIGHT(0) MAROON_LOAD() MAROON_GFX("48500ce0e00c5048") MAROON_DIM(*));
	do_cops();
	sendText(MAROON_IMM_CLEAR() MAROON_BRIGHT(5));
    }

    if (event==EVENT_KEY2) {
        setup_avoid_obst();
	return;
    }

    if (event==EVENT_KEY3) {
	if (run) {
	  run = 0;
          motpid_stop(1);
	} else {
	  run = 1;
          odometry_reset();
          motpid_setSpeed(+20,+20);
	}
	//print_text("KEY3");
    }
      
      
    if (event==EVENT_TO_CLOSE) {
          motpid_stop(1);
          if (has_maroon) led_setall(1, 0, 0, 1);
          if (!has_maroon) led_setall(1, 1, 1, 1);
	
    } else if (event==EVENT_NO_HAND) {
          motpid_stop(0);
          led_setall(0, 0, 0, 0);
	
    } else if (event==EVENT_HAND_L) {
	follow_setSpeed(+25,+35);
        if (has_maroon) led_setall(0, 0, 0, 0);
        if (!has_maroon) led_setall(0, 1, 0, 0);
	
    } else if (event==EVENT_HAND_C) {
	follow_setSpeed(+35,+35);
        if (has_maroon) led_setall(0, 0, 0, 0);
        if (!has_maroon) led_setall(0, 1, 1, 0);
	
    } else if (event==EVENT_HAND_R) {
	follow_setSpeed(+35,+25);
        if (has_maroon) led_setall(0, 0, 0, 0);
        if (!has_maroon) led_setall(0, 0, 1, 0);
	
    } else if (event==EVENT_HAND_LL) {
	motpid_setSpeed(-30,+40);
        if (has_maroon) led_setall(0, 0, 0, 0);
        if (!has_maroon) led_setall(1, 1, 0, 0);
	
    } else if (event==EVENT_HAND_RR) {
	motpid_setSpeed(+40,-30);
        if (has_maroon) led_setall(0, 0, 0, 0);
        if (!has_maroon) led_setall(0, 0, 1, 1);
      
    } else if (event==EVENT_NICK) {
          motpid_stop(0);
          led_setall(1, 1, 1, 1);
          if (!has_maroon) {
            denick_procedure();
          } else {
            maroon_mode = 0;
            sendText(MAROON_IMM_CLEAR() MAROON_LOAD() MAROON_GFX("080cfa0101fa0c08") MAROON_PAUSE(800) " Help me!\n" MAROON_TXBACK("."));
          }
    }
      

    if (run==1) {
        if (odometry_getRight(0) >= TICKS_PER_METER/5) {
          motpid_stop(1);
          run = 0;
        }
    }
      
    if (run==2) {
      if (odometry_getRight(0)-odometry_getLeft(0) >= TICKS_PER_TURN/4) {
        motpid_stop(1);
        run = 0;
      }
    }
          
  }

}

void setup_avoid_obst() {
  maroon_mode=0;
  sendText(MAROON_IMM_CLEAR() MAROON_BRIGHT(0) MAROON_LOAD() "2" MAROON_BAR("0") MAROON_DIM(*) MAROON_PAUSE(100) MAROON_DIM(5) MAROON_STIME(60) " Fraidy\n" MAROON_TXBACK("."));
  //sendText(MAROON_IMM_CLEAR() MAROON_LOAD() MAROON_BAR("ABBAABBA") MAROON_PAUSE(500) MAROON_STIME(60) "Fraidy\n" MAROON_TXBACK("."));
  motpid_stop(1);
  blink_led(2, 4);
  progmode = 2;
  run = 0;
  counter = 0;
}


void loop_avoid_obst() {
  while(1) {
    analog_wait_update();
    do_maroon();
    uint8_t event = getEvent();
    
    int8_t left = 0;
    int8_t right = 0;
    
    uint16_t val;   
    
    if (event==EVENT_KEY1) {
      setup_follow_me();
      return;
    }

    if (event==EVENT_KEY2) {
      motpid_stop(1);
	sendText(MAROON_IMM_CLEAR() MAROON_BRIGHT(0) MAROON_LOAD() MAROON_GFX("48500ce0e00c5048") MAROON_DIM(*));
	do_cops();
	sendText(MAROON_IMM_CLEAR() MAROON_BRIGHT(5));
      run = 0;
    }

    if (event==EVENT_KEY3) {
	if (run) {
	  run = 0;
          motpid_stop(1);
	} else {
	  run = 1;
          odometry_reset();
          motpid_setSpeed(+20,+20);
	}
    }
      
    if (event==EVENT_NICK) {
        motpid_stop(1);
        led_setall(1, 1, 1, 1);
        maroon_mode = 0;
        sendText(MAROON_IMM_CLEAR() MAROON_LOAD() MAROON_GFX("080cfa0101fa0c08") MAROON_PAUSE(800) " Help me!\n" MAROON_TXBACK("."));
    }      
      
    if (event==EVENT_OBSTACLE_DETECTED) {
	led_set(LED_1, obstacle_pos<=0);
	led_set(LED_2, 0);
        led_set(LED_3, 0);
        led_set(LED_4, obstacle_pos>=0);
	if (obstacle_pos>0) {
	  motpid_setSpeed(-45,+35);
	} else if (obstacle_pos<0) {
	  motpid_setSpeed(+35,-45);
	} else {
	  motpid_setSpeed(-30,-30);
	}
    }
      
    if (event==EVENT_OBSTACLE_CLEAR) {
        led_setall(0, 0, 0, 0);
        motpid_setSpeed(+55,+55);
    }

    if (event==EVENT_OBSTACLE_CLEAR_L) {
        led_setall(0, 0, 1, 0);
        motpid_setSpeed(+30,+55);
    }
    
    if (event==EVENT_OBSTACLE_CLEAR_R) {
        led_setall(0, 1, 0, 0);
        motpid_setSpeed(+55,+30);
    }
      
    if (run==1) {
        if (odometry_getRight(0) >= TICKS_PER_METER/5) {
          motpid_stop(1);
	  run = 0;
        }
    }
      
    if (run==2) {
        if (odometry_getRight(0)-odometry_getLeft(0) >= TICKS_PER_TURN/4) {
          motpid_stop(1);
          run = 0;
        }
    }
    
  }

}

uint8_t cal_lastColor;
uint16_t cal_detCount;



enum {
  COL_NONE,
  COL_BLACK,
  COL_WHITE,
  COL_RED,
  COL_BLUE,
  COL_GREEN,
  COL_YELLOW
};

void setup_calibrate() {
  sendText(MAROON_IMM_CLEAR() MAROON_BRIGHT(0) MAROON_LOAD() "3" MAROON_BAR("00") MAROON_DIM(*) MAROON_PAUSE(100) MAROON_DIM(5) MAROON_STIME(60) " Colour\n" MAROON_TXBACK(".") MAROON_STIME(125));
  //sendText(MAROON_IMM_CLEAR() MAROON_LOAD() MAROON_BAR("BBABBABB") MAROON_PAUSE(500) MAROON_STIME(60) "Colour\n" MAROON_STIME(125) MAROON_TXBACK("."));
  blink_led(3, 4);
  progmode = 3;
}



void loop_calibrate() {
  static uint8_t pos=1;
  analog_wait_update();
  
  //do_maroon_col();
  if (do_maroon_process(pos)) {
    pos++;    
    if (cal_detCount==100) {
      /*
      if (pos==8) pos=34;
      if (pos==15) pos=39;
      if (pos==22) pos=44;
      if (pos==29) pos=29;
      */
      if (pos==11) {
        maroon_mode = 0;
        if (cal_lastColor==COL_BLACK) {
          sendText(MAROON_BRIGHT(5) " black\n" MAROON_TXBACK("."));
        } else if (cal_lastColor==COL_WHITE) {
          sendText(MAROON_BRIGHT(5) " white\n" MAROON_TXBACK("."));
        } else if (cal_lastColor==COL_RED) {
          sendText(MAROON_BRIGHT(5) " red\n" MAROON_TXBACK("."));
        } else if (cal_lastColor==COL_BLUE) {
          sendText(MAROON_BRIGHT(5) " blue\n" MAROON_TXBACK("."));
        } else if (cal_lastColor==COL_GREEN) {
          sendText(MAROON_BRIGHT(5) " green\n" MAROON_TXBACK("."));
        } else if (cal_lastColor==COL_YELLOW) {
          sendText(MAROON_BRIGHT(5) " yellow\n" MAROON_TXBACK("."));
        }
      }
      if (pos>11) {
        pos=11;
      }
    } else if (pos>7) {
      pos=1;
    }
  }
  uint8_t event = getEvent();
  
  if (event==EVENT_KEY1) {
    blink_byte(0);
    surface_calibrateBlack();
    surface_writePersistent();
    maroon_mode = 0;
    sendText(MAROON_IMM_CLEAR() MAROON_DIM(5) MAROON_STIME(60) "BLACK calibrated!\n"  MAROON_STIME(125) MAROON_TXBACK("."));
    blink_led_fast(1, 10);
  }

  if (event==EVENT_KEY2) {
    blink_byte(0);
    surface_calibrateWhite();
    surface_writePersistent();
    maroon_mode = 0;
    sendText(MAROON_IMM_CLEAR() MAROON_DIM(5) MAROON_STIME(60) "WHITE calibrated!\n" MAROON_STIME(125) MAROON_TXBACK("."));
    blink_led_fast(2, 10);
  }

  uint8_t col = COL_NONE;
  uint8_t leds = 0;
  
  uint32_t rgb = surface_getColorRGB();
  uint32_t hsv = surface_getColorHSV();
  
  /*
  uint16_t dcol_bk = color_diff_hsv_bal(hsv, COLOR_HSV_CAL_BLACK, 20);
  uint16_t dcol_wt = color_diff_hsv_bal(hsv, COLOR_HSV_CAL_WHITE, 20);
  uint16_t dcol_rd = color_diff_hsv_bal(hsv, COLOR_HSV_CAL_RED, 20);
  uint16_t dcol_bl = color_diff_hsv_bal(hsv, COLOR_HSV_CAL_BLUE, 20);
  uint16_t dcol_gn = color_diff_hsv_bal(hsv, COLOR_HSV_CAL_GREEN, 20);
  uint16_t dcol_ye = color_diff_hsv_bal(hsv, COLOR_HSV_CAL_YELLOW, 20);
  */
  
  int16_t r = surface_get(SURFACE_R);
  int16_t g = surface_get(SURFACE_C);
  int16_t b = surface_get(SURFACE_L);
      
  int16_t rg = (r-g);
  int16_t by = (b-(r+g)/2);
  int16_t i = r+g+b;
  
  if ((abs(rg)<100) && (abs(by)<100)) {
    if (i<200) col = COL_BLACK;
    if (i>2500) col = COL_WHITE;
  }
  if (rg>500) {
    if (g<200) col = COL_RED; 
    else col = COL_YELLOW;
  }
  if ((rg<100)&&(by>200)) {
    col = COL_BLUE;
  }
  if ((rg<100)&&(by<-300)) {
    col=COL_GREEN;
  }
    
  //if (dcol_min<900) {
    if (col==COL_BLACK) {
      leds = 0x01;
    } else if (col==COL_WHITE) {
      leds = 0x02;
    } else if (col==COL_RED) {
      leds = 0x09;
    } else if (col==COL_BLUE) {
      leds = 0x06;
    } else if (col==COL_GREEN) {
      leds = 0x03;
    } else if (col==COL_YELLOW) {
      leds = 0x0c;
    }
  //}
  
  if ((col!=COL_NONE) && (cal_lastColor==col)) {
    if ((cal_detCount<100) && maroon_mode){
      cal_detCount++;
    }
  } else {
    cal_lastColor = col;
    cal_detCount=0;
  }
  
  set_led_byte(leds);

  if (event==EVENT_KEY3) {
    maroon_mode = 0;
    sendText(MAROON_IMM_CLEAR() MAROON_DIM(5) MAROON_STIME(125));
    print_color_rgb(rgb);
    sendText("\n" MAROON_TXBACK("."));
    delay(100);
  }
}
