/*
  Programm zum Kalibrieren der Boden-Sensoren
*/


#include <robo2/iodefs.h>
#include <robo2/usart.h>
#include <robo2/motpwm.h>
#include <robo2/motpid.h>
#include <robo2/analog.h>
#include <robo2/odometry.h>
#include <robo2/led.h>
#include <robo2/key.h>
#include <robo2/storage.h>
#include <robo2/surface.h>

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>


void blink_all() {
  for (uint8_t i=0; i<3; ++i) {
    led_set(1, 1);
    led_set(2, 1);
    led_set(3, 1);
    led_set(4, 1);
    delay(100);
    led_set(1, 0);
    led_set(2, 0);
    led_set(3, 0);
    led_set(4, 0);
    delay(100);
  }
}

void blink_led(uint8_t led) {
  led_set(1, 0);
  led_set(2, 0);
  led_set(3, 0);
  led_set(4, 0);
  for (uint8_t i=0; i<3; ++i) {
    led_set(led, 1);
    delay(200);
    led_set(led, 0);
    delay(200);
  }
}


void calibrate() {
  uint8_t mode = 1;
  blink_led(mode);
  surface_init();

  for (uint8_t i=0; i<7; i++) {  
    analog_setExtToggleMode(i, i<3);
  }
  analog_setExtToggleMode(7, 1);
  
  
  while (1) {    
    uint8_t key = key_read();
    uint16_t val_l, val_c, val_r;

    if (mode==2) {
      val_l = surface_get(SURFACE_CL);
      val_c = surface_get(SURFACE_C);
      val_r = surface_get(SURFACE_CR);
    } else {
      val_l = surface_get(SURFACE_L);
      val_c = surface_get(SURFACE_C);
      val_r = surface_get(SURFACE_R);
    }

    if (mode<=2) {
      // show 
      led_set(LED_1, (val_l>50));
      led_set(LED_2, (val_c>50));
      led_set(LED_3, (val_c>50));
      led_set(LED_4, (val_r>50));
      
    } else if (mode==3) {
      led_set(LED_1, (val_c<5));
      led_set(LED_2, (val_c>=5)&&(val_c<10));
      led_set(LED_3, (val_c>=10)&&(val_c<20));
      led_set(LED_4, (val_c>=20));
      
    } else if (mode==4) {
      led_set(LED_1, (val_c<1024-16));
      led_set(LED_2, (val_c>=1024-16)&&(val_c<1024));
      led_set(LED_3, (val_c>=1024)&&(val_c<1024+16));
      led_set(LED_4, (val_c>=1024+16));
    }

    if (key==KEY_1) {
      // mode key
      mode++;
      if (mode>4) {
        mode=1;
      }
      blink_led(mode);
      if (mode==2) {
        analog_setExtToggleMode(0, 0);
        analog_setExtToggleMode(2, 0);
      } else if (mode==3) {
        analog_setExtToggleMode(0, 1);
        analog_setExtToggleMode(2, 1);
      }      
    }
    
    
    if (key==KEY_2) {
      // calibrate key
      if (mode==3) {
        surface_calibrateBlack();
        blink_all();
      }
      if (mode==4) {
        surface_calibrateWhite();
        blink_all();
      }
    }
    
    if (key==KEY_3) {
      // store key
      //storage_format();
      surface_writePersistent();
      blink_all();
    }
    
  }
}


