/*
  Programm zum ersten Testen des NIBObee Roboters
*/


#include <robo2/iodefs.h>
#include <robo2/usart.h>
#include <robo2/motpwm.h>
#include <robo2/motpid.h>
#include <robo2/analog.h>
#include <robo2/odometry.h>
#include <robo2/led.h>
#include <robo2/key.h>
#include <robo2/storage.h>
#include <robo2/surface.h>
#include <robo2/delay.h>

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

//#define SPEED 500
#define SPEED 1023

#define STORAGE_USER1 STORAGE_ENTRY(STORAGE_USER1_ID, 0x01, 1, 1)

void calibrate();


uint8_t pers_data;

static void load_persistant() {
  if (storage_read(STORAGE_USER1, &pers_data)==0) {
    pers_data=0;
  }  
}

static void save_persistant() {
  storage_write(STORAGE_USER1, &pers_data);
}

#define STORAGE_USER_BASE 0x80

#define STORAGE_ENTRY(owner, type, version, size) ((((uint32_t)(owner))<<24) || (((uint32_t)(type))<<16) || (((uint32_t)(version))<<8) || ((uint32_t)(size)))



int8_t decode_l (uint8_t val) {
  if (val==0x01) return -1;
  if (val==0x02) return 1;
  return 0;
}

int8_t decode_r (uint8_t val) {
  if (val==0x01) return -1;
  if (val==0x02) return 1;
  return 0;
}


void print_hex (uint8_t val) {
  char c1=val/16;
  char c2=val%16;
  if (c1<10) c1+='0'; else c1+='a'-10;
  if (c2<10) c2+='0'; else c2+='a'-10;
  usart_putchar(c1);
  usart_putchar(c2);
}

void print_text (char * text) {
  //bgx1_termPrint(text);
  char c;
  while ((c=*text++)) {
    while (usart_txfull()) {
      // wait 
    }
    usart_putchar(c);
  }
}

uint16_t get_sensor(uint8_t s) {
  int16_t val = analog_getValue2(s, 1);
  val -= analog_getValue2(s, 0);
  if (val<0) val = 0;
  if (val>1023) val = 1023;
  return val;
}


uint8_t run = 0;
uint8_t sensor = 7;
int main() {
  
  load_persistant();
  
  activate_output_group(IO_LEDS);  // LED bits als Output
  
  if (pers_data==2) {
    led_set(LED_2, 1);
    delay(200);
  }

  if (pers_data==3) {
    led_set(LED_3, 1);
    delay(200);    
  }
  
  
  activate_output_bit(IO_EN_SL);
  activate_output_bit(IO_EN_RL);
  activate_output_bit(IO_EN_FL);
  activate_output_bit(IO_EN_SC);
  activate_output_bit(IO_EN_FR);
  activate_output_bit(IO_EN_RR);
  activate_output_bit(IO_EN_SR);
  
  delay(200);
  set_output_bit(IO_EN_SL);
  set_output_bit(IO_EN_RL);
  set_output_bit(IO_EN_FL);
  set_output_bit(IO_EN_SC);
  set_output_bit(IO_EN_FR);
  set_output_bit(IO_EN_RR);
  set_output_bit(IO_EN_SR);
   
  usart_setbaudrate(38400);
  usart_enable();
  motpwm_init();
  motpid_init();
  odometry_init();
  //motpwm_setLeft(0);
  //motpwm_setRight(0);
  analog_init();
  //bgx1_init();
  surface_init();
  sei();
  
  analog_setExtToggleMode(7, 1);
  
  delay(100);

  uint8_t key = key_read();


  if (key&0x01) {
    calibrate();
  }
  
  //bgx1_termClear();
  
  //motpwm_setLeft(-500);
  //motpwm_setRight(500);
  //motpid_setSpeed(+20,+20);
  
  int16_t delta = 0;

  while(1) {
    analog_wait_update();
    int8_t left = 0;
    int8_t right = 0;
    int16_t speed_l = 0;
    int16_t speed_r = 0;
    
    int16_t val_l, val_c, val_r;
    key = key_read();
        
    if (key&0x01) {
       //motpid_stop(1);
    }
    if (key&0x04) {
       //motpid_setSpeed(5,5);
    }

    
    if ((speed_l==0) && (speed_r==0)) {
   
      if (key == KEY_2) {
	//motpwm_setLeft(0);
	//motpwm_setRight(0);
	pers_data = 2;
        save_persistant();
      }

      if (key == KEY_3) {
	  run = 2;
          pers_data = 3;
          save_persistant();
      }
      
      
      //led_set(LED_2, (odometry_getLeft(0)<-250));
      //led_set(LED_3, (odometry_getRight(0)>250));
      
      /*
      if (odometry_getLeft(0)<-255) {
	motpwm_setLeft(0);
      } else if (odometry_getLeft(0)<-240) {
	motpwm_setLeft(-300);
      }
      
      if (odometry_getRight(0)>255) {
	motpwm_setRight(0);	
      } else if (odometry_getRight(0)>240) {
	motpwm_setRight(300);	
      }
      */

      val_l = surface_get(SURFACE_CL);
      val_c = surface_get(SURFACE_C);
      val_r = surface_get(SURFACE_CR);
      
      
      if (run==2) {
        if (val_c<50) {
          delta = val_l - val_r;
          motpwm_setLeft(800 + delta);
          motpwm_setRight(800 - delta);
        } else {
          if (delta<0) {
            motpwm_setLeft(0);
            motpwm_setRight(1024);
          }
          if (delta>0) {
            motpwm_setLeft(1024);
            motpwm_setRight(0);
          }
        }
        print_hex(val_l>>2);
        usart_putchar(' ');
        print_hex(val_c>>2);
        usart_putchar(' ');
        print_hex(val_r>>2);
        usart_putchar('\n');
      }

      led_set(LED_1, (val_c<250));
      led_set(LED_2, (val_l<val_r+50));
      led_set(LED_3, (val_r<val_l+50));
      led_set(LED_4, (val_c<250));

    } else {
      /*
      led_set(LED_L_RD, (speed_l>0));
      led_set(LED_R_RD, (speed_r>0));
      led_set(LED_L_YE, (speed_l<0));
      led_set(LED_R_YE, (speed_r<0));
      */
    }
    
  }

  return 0;                             /* Diese Zeile ist eigentlich nicht
                                           notwendig, da das Programm sie wegen
                                           der Endlosschleife nie erreichen
                                           wird, der Compiler gibt jedoch
                                           ansonsten eine Warnung aus. */
}
