/*
  Programm zum ersten Testen des NIBObee Roboters
*/


#include <robo2/iodefs.h>
#include <robo2/usart.h>
#include <robo2/motpwm.h>
#include <robo2/motpid.h>
#include <robo2/analog.h>
#include <robo2/odometry.h>
#include <robo2/led.h>
#include <robo2/key.h>

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

//#define SPEED 500
#define SPEED 1023

enum {
  EVENT_NONE              =  0,
  EVENT_KEY1              =  1,
  EVENT_KEY2              =  2,
  EVENT_KEY3              =  3,
  EVENT_TIMEOUT           =  4,
  
  EVENT_LINE_LOST         =  8,
  EVENT_LINE_FOUND        =  9,
  
  EVENT_OBSTACLE_DETECTED = 16,
  EVENT_OBSTACLE_CLEAR    = 17
};

/* OBSTACLE */

int8_t obstacle_pos;
int8_t obstacle_val;

uint8_t obstacle_getEvent() {
  static uint8_t state = 0;
  int8_t l = analog_getValue2(ANALOG_FL, 2)>>3;
  int8_t r = analog_getValue2(ANALOG_FR, 2)>>3;
  int8_t ll = analog_getValue2(ANALOG_SL, 2)>>3;
  int8_t rr = analog_getValue2(ANALOG_SR, 2)>>3;
  int8_t cc = l + r;
  ll += l;
  rr += r;
 
  obstacle_val = (ll>rr)?ll:rr;
  if (cc>obstacle_val) {
    obstacle_val = cc;
  }
  
  if ((obstacle_val==cc) || (ll==rr)) {
    if (ll>rr) {
      obstacle_pos = -1;
    } else if (ll<rr) {
      obstacle_pos = +1;
    } else {
      obstacle_pos = 0;
    }
  } else if (ll>rr) {
    obstacle_pos = -2;
  } else {
    obstacle_pos = +2;
  }
 

  if ((obstacle_val>6) && (state!=1)) {
    state=1;
    return EVENT_OBSTACLE_DETECTED;
  }
  
  if ((obstacle_val<5) && (state!=0)) {
    state=0;
    return EVENT_OBSTACLE_CLEAR;
  }  
  
  return EVENT_NONE;
}

/* KEY */

uint8_t key_getEvent() {
  static uint8_t key = 0;
  uint8_t act = key_get();
  if (act==0) {
    act = key;
    key = 0;
    if (act==0x01) return EVENT_KEY1;
    if (act==0x02) return EVENT_KEY2;
    if (act==0x04) return EVENT_KEY3;
    return EVENT_NONE;
  }
  if (act!=KEY_INVALID) {
    key |= act;  
  }
  return EVENT_NONE;
}


uint8_t getEvent() {
  uint8_t event = EVENT_NONE;
  event = key_getEvent();
  if (event) return event;
  event = obstacle_getEvent();
  return event;
}


int8_t decode_l (uint8_t val) {
  if (val==0x01) return -1;
  if (val==0x02) return 1;
  return 0;
}

int8_t decode_r (uint8_t val) {
  if (val==0x01) return -1;
  if (val==0x02) return 1;
  return 0;
}


void print_hex (uint8_t val) {
  char c1=val/16;
  char c2=val%16;
  if (c1<10) c1+='0'; else c1+='a'-10;
  if (c2<10) c2+='0'; else c2+='a'-10;
  usart_putchar(c1);
  usart_putchar(c2);
}

void print_text (char * text) {
  char c;
  while ((c=*text++)) {
    while (usart_txfull()) {
      /* wait */
    }
    usart_putchar(c);
  }
}


uint16_t get_sensor(uint8_t s) {
  int16_t val = analog_getValue2(s, 1);
  val -= analog_getValue2(s, 0);
  if (val<0) val = 0;
  if (val>1023) val = 1023;
  return val;
}


int16_t value_pwm = 0;
uint8_t count = 0;
uint8_t run = 0;
uint8_t sensor = 7;

int main() {
  activate_output_group(IO_LEDS);  // LED bits als Output
  //usart_setbaudrate(9600);
  usart_setbaudrate(38400);
  usart_enable();
  motpwm_init();
  motpid_init();
  odometry_init();
  //motpwm_setLeft(0);
  //motpwm_setRight(0);
  analog_init();
  sei();
  
  analog_setExtToggleMode(7, 1);
  
  delay(100);

  
  //motpwm_setLeft(-500);
  //motpwm_setRight(500);
  //motpid_setSpeed(+20,+20);

  clock_set_timer(100);

  while(1) {
    clock_wait_timer();
    // every 100 ms...

    uint8_t event = getEvent();
    
    int8_t left = 0;
    int8_t right = 0;
    int16_t speed_l = 0;
    int16_t speed_r = 0;
    
    uint16_t val;   
    
     
    if (event==EVENT_KEY1) {
      print_text("KEY1\n");
      run=1;
    }

    if (event==EVENT_KEY2) {
      print_text("KEY2\n");
      run=2;
    }

    if (event==EVENT_KEY3) {
      print_text("KEY3\n");      
      run=0;
      motpwm_setLeft(0);
      motpwm_setRight(0);
    }
      
    if (run!=0) {
      if (++count>=5) {
        count = 0;
	if (value_pwm < 1024) {
	  value_pwm += 16;
	} else {
	  value_pwm = 0;
	  run = 0;
	}
	uint16_t val = 0;
	if (run==1) {
	  val = odometry_getLeft(1);
          motpwm_setLeft(value_pwm);
	} else if (run==2) {
	  val = odometry_getRight(1);
          motpwm_setRight(value_pwm);
	} else {
	  motpwm_setLeft(0);
	  motpwm_setRight(0);
	}
	print_hex(value_pwm>>4);
	usart_putchar(' ');
	print_hex(val & 0xff);
	usart_putchar('\n');
      }
    }    
  }

  return 0;                             /* Diese Zeile ist eigentlich nicht
                                           notwendig, da das Programm sie wegen
                                           der Endlosschleife nie erreichen
                                           wird, der Compiler gibt jedoch
                                           ansonsten eine Warnung aus. */
}
