/*
  Programm zum ersten Testen des NIBObee Roboters
*/


#include <niboburger/iodefs.h>
#include <niboburger/usart.h>
#include <niboburger/motpwm.h>
#include <niboburger/motpid.h>
#include <niboburger/analog.h>
#include <niboburger/odometry.h>
#include <niboburger/led.h>
#include <niboburger/delay.h>
#include <niboburger/key.h>

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

//#define SPEED 500
#define SPEED 1023

enum {
  EVENT_NONE              =  0,
  EVENT_KEY1              =  1,
  EVENT_KEY2              =  2,
  EVENT_KEY3              =  3,
  EVENT_TIMEOUT           =  4,

  EVENT_LINE_LOST         =  8,
  EVENT_LINE_FOUND        =  9,

  EVENT_OBSTACLE_DETECTED = 16,
  EVENT_OBSTACLE_CLEAR    = 17
};

/* OBSTACLE */

int8_t obstacle_pos;
int8_t obstacle_val;

uint8_t obstacle_getEvent() {
  static uint8_t state = 0;
  int8_t l = analog_getValueExt(ANALOG_FL, 2)>>3;
  int8_t r = analog_getValueExt(ANALOG_FR, 2)>>3;
  int8_t ll = analog_getValueExt(ANALOG_FLL, 2)>>3;
  int8_t rr = analog_getValueExt(ANALOG_FRR, 2)>>3;
  int8_t cc = l + r;
  ll += l;
  rr += r;

  obstacle_val = (ll>rr)?ll:rr;
  if (cc>obstacle_val) {
    obstacle_val = cc;
  }

  if ((obstacle_val==cc) || (ll==rr)) {
    if (ll>rr) {
      obstacle_pos = -1;
    } else if (ll<rr) {
      obstacle_pos = +1;
    } else {
      obstacle_pos = 0;
    }
  } else if (ll>rr) {
    obstacle_pos = -2;
  } else {
    obstacle_pos = +2;
  }


  if ((obstacle_val>6) && (state!=1)) {
    state=1;
    return EVENT_OBSTACLE_DETECTED;
  }

  if ((obstacle_val<5) && (state!=0)) {
    state=0;
    return EVENT_OBSTACLE_CLEAR;
  }

  return EVENT_NONE;
}

/* KEY */

uint8_t key_getEvent() {
  uint8_t key = key_get_char();
  if (key) {
    if (key=='A') return EVENT_KEY1;
    if (key=='B') return EVENT_KEY2;
    if (key=='C') return EVENT_KEY3;
  }
  return EVENT_NONE;
}

uint8_t getEvent() {
  uint8_t event = EVENT_NONE;
  event = key_getEvent();
  if (event) return event;
  event = obstacle_getEvent();
  return event;
}


int8_t decode_l (uint8_t val) {
  if (val==0x01) return -1;
  if (val==0x02) return 1;
  return 0;
}

int8_t decode_r (uint8_t val) {
  if (val==0x01) return -1;
  if (val==0x02) return 1;
  return 0;
}


void print_hex (uint8_t val) {
  char c1=val/16;
  char c2=val%16;
  if (c1<10) c1+='0'; else c1+='a'-10;
  if (c2<10) c2+='0'; else c2+='a'-10;
  usart_putchar(c1);
  usart_putchar(c2);
}

void print_text (char * text) {
  char c;
  while ((c=*text++)) {
    while (usart_txfull()) {
      /* wait */
    }
    usart_putchar(c);
  }
}


uint16_t get_sensor(uint8_t s) {
  int16_t val = analog_getValueExt(s, 1);
  val -= analog_getValueExt(s, 0);
  if (val<0) val = 0;
  if (val>1023) val = 1023;
  return val;
}


uint8_t run = 0;
uint8_t sensor = 7;
int main() {
  activate_output_group(IO_LEDS);  // LED bits als Output
  //usart_setbaudrate(9600);
  usart_setbaudrate(38400);
  usart_enable();
  motpwm_init();
  motpid_init();
  odometry_init();
  //motpwm_setLeft(0);
  //motpwm_setRight(0);
  analog_init();
  sei();

  analog_setExtToggleMode(7, 1);

  delay(100);


  //motpwm_setLeft(-500);
  //motpwm_setRight(500);
  //motpid_setSpeed(+20,+20);


  while(1) {
    analog_wait_update();
    uint8_t event = getEvent();

    int8_t left = 0;
    int8_t right = 0;
    int16_t speed_l = 0;
    int16_t speed_r = 0;

    uint16_t val;

    if (event==EVENT_KEY1) {
       //motpid_stop(1);
    }
    if (event==EVENT_KEY2) {
       //motpid_setSpeed(5,5);
    }


    if ((speed_l==0) && (speed_r==0)) {
      //int16_t lval = analog_getValue(ANALOG_L1)-(int16_t)analog_getValue(ANALOG_L0) - 40;
      //int16_t cval = (3*(analog_getValue(ANALOG_C1)-(int16_t)analog_getValue(ANALOG_C0)))/4 - 60;
      //int16_t rval = analog_getValue(ANALOG_R1)-(int16_t)analog_getValue(ANALOG_R0) - 40;

      //if (lval<0) lval=0;
      //if (cval<0) cval=0;
      //if (rval<0) rval=0;
   /*
      if (usart_txempty()) {
        print_hex(LOBYTE(lval>>4));
        usart_putchar(':');
        print_hex(LOBYTE(cval>>4));
        usart_putchar(':');
        print_hex(LOBYTE(rval>>4));
        usart_putchar(10);
        usart_putchar(13);
      }
      uint8_t rule_l = 0;
      uint8_t rule_c = 0;
      uint8_t rule_r = 0;


      if (lval+lval > rval+cval) rule_l=1;
      if (cval+cval > lval+rval) rule_c=1;
      if (rval+rval > lval+cval) rule_r=1;

      if (lval > 300) rule_l=1;
      if (cval > 300) rule_c=1;
      if (rval > 300) rule_r=1;

      if (lval < 100) rule_l=0;
      if (cval < 100) rule_c=0;
      if (rval < 100) rule_r=0;

      led_set(LED_L_RD, rule_l);
      led_set(LED_R_RD, rule_r);
*/

      if (event==EVENT_KEY1) {
	/*
	if (led_get(LED_2)) {
          led_set(LED_2, 0);
          led_set(LED_3, 0);
	} else {
          led_set(LED_2, 1);
          led_set(LED_3, 1);
	}
	*/
	print_text("KEY1");
      }

      if (event==EVENT_KEY2) {
	if (run) {
	  run = 0;
          motpid_stop(1);
	} else {
	  run = 2;
          odometry_reset();
          motpid_setSpeed(-20,+20);
	}
	print_text("KEY2");
      }

      if (event==EVENT_KEY3) {
	if (run) {
	  run = 0;
          motpid_stop(1);
	} else {
	  run = 1;
          odometry_reset();
          motpid_setSpeed(+20,+20);
	}
	print_text("KEY3");
      }

      if (event==EVENT_OBSTACLE_DETECTED) {
	led_set(LED_2, obstacle_pos<=0);
        led_set(LED_3, obstacle_pos>=0);
	if (obstacle_pos>0) {
	  motpid_setSpeed(-20,+20);
	} else if (obstacle_pos<0) {
	  motpid_setSpeed(+20,-20);
	} else {
	  motpid_setSpeed(-20,-20);
	}
      }

      if (event==EVENT_OBSTACLE_CLEAR) {
	led_set(LED_2, 0);
        led_set(LED_3, 0);
	motpid_setSpeed(+25,+25);
      }

      val = get_sensor(sensor);

      if (run==1) {
        if (odometry_getRight(0) >= TICKS_PER_METER/5) {
          motpid_stop(1);
	  run = 0;
        }
        print_hex((val>>8)&0xff);
        print_hex((val>>0)&0xff);
        usart_putchar('\n');
      }

      if (run==2) {
        if (odometry_getRight(0)-odometry_getLeft(0) >= TICKS_PER_TURN/4) {
          motpid_stop(1);
          run = 0;
        }
      }

      /*
      if (sensor<7) {
        led_set(LED_1, (val>15));
        led_set(LED_2, (val>50));
        led_set(LED_3, (val>100));
        led_set(LED_4, (val>200));
      } else {
	int16_t val2 = get_sensor(8);
        led_set(LED_1, (val>5));
        led_set(LED_2, (val>10));
        led_set(LED_3, (val2>10));
        led_set(LED_4, (val2>5));
      }
      */

      //led_set(LED_1, (PIND & 0x04));
      //led_set(LED_4, (PIND & 0x08));

    } else {
      /*
      led_set(LED_L_RD, (speed_l>0));
      led_set(LED_R_RD, (speed_r>0));
      led_set(LED_L_YE, (speed_l<0));
      led_set(LED_R_YE, (speed_r<0));
      */
    }

  }

  return 0;                             /* Diese Zeile ist eigentlich nicht
                                           notwendig, da das Programm sie wegen
                                           der Endlosschleife nie erreichen
                                           wird, der Compiler gibt jedoch
                                           ansonsten eine Warnung aus. */
}
