/*
  Programm zum ersten Testen des NIBObee Roboters
*/


#include <niboburger/base.h>
#include <niboburger/iodefs.h>
#include <niboburger/usart.h>
#include <niboburger/motpwm.h>
#include <niboburger/motpid.h>
#include <niboburger/analog.h>
#include <niboburger/odometry.h>
#include <niboburger/led.h>
#include <niboburger/key.h>
#include <niboburger/delay.h>
#include <niboburger/surface.h>
#include <niboburger/color.h>

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <string.h>
#include <stdlib.h>

//#define SPEED 500
#define SPEED 1023


#define MAROON_CLEAR() "\33c"
#define MAROON_BRIGHT(x) "\33b" #x
#define MAROON_DIM(x) "\33d" #x
#define MAROON_TXBACK(c) "\33t" c
#define MAROON_LOAD() "\33l"
#define MAROON_FONT_ECO() "\33e1"
#define MAROON_FONT_NORM() "\33e0"

#define MAROON_BAR(bar) "\33B" bar " "
#define MAROON_GFX(gfx) "\33G" gfx " "
#define MAROON_PAUSE(x) "\33P" #x " "
#define MAROON_STIME(x) "\33S" #x " "
#define MAROON_DTIME(x) "\33D" #x " "
#define MAROON_TRANSITION(n, x) "\33T" #n #x " "

#define MAROON_IMM_CLEAR() "\20c"
#define MAROON_IMM_DIM(x) "\20d" #x

uint8_t act_sensor;
uint8_t act_value;


void print (char * txt) {
  char c;
  while ((c=*txt++)) {
    while (usart_txfull()) {
      /* wait */
    }
    usart_putchar(c);
  }  
}


static uint8_t hexnibble(uint8_t x) {
  return (x<10)?(x+'0'):(x+'A'-10);
}

char displaytext[] = MAROON_BRIGHT(0) MAROON_LOAD() "-" MAROON_LOAD() "-" MAROON_BRIGHT(5) MAROON_TXBACK(".");

void display_data() {
  uint16_t val = analog_getValueExt(act_sensor, act_value)>>2;
  uint8_t c0 = 'X';
  uint8_t c1 = 'X';
  
  if (val<256) {
    c0 = (val&0x000f);
    c1 = (val&0x00f0)>>4;
    c0 = hexnibble(c0);
    c1 = hexnibble(c1);
  }
  displaytext[5]=c1;
  displaytext[8]=c0;
  print(displaytext);
}

void select_next_sensor() {
  if (++act_sensor>=ANALOG_CNT) {
    act_sensor=0;
  }
  print (MAROON_IMM_CLEAR() MAROON_BRIGHT(5) MAROON_STIME(125));
  
  switch (act_sensor) {
    case ANALOG_BL: print("BL"); analog_setExtToggleMode(7, 0); analog_setExtToggleMode(0, 1); break;
    case ANALOG_BC: print("BC"); analog_setExtToggleMode(0, 0); analog_setExtToggleMode(1, 1);  break;
    case ANALOG_BR: print("BR"); analog_setExtToggleMode(1, 0); analog_setExtToggleMode(2, 1);  break;
    case ANALOG_FL: print("FL"); analog_setExtToggleMode(2, 0); analog_setExtToggleMode(3, 1);  break;
    case ANALOG_FR: print("FR"); analog_setExtToggleMode(3, 0); analog_setExtToggleMode(4, 1);  break;
    case ANALOG_FLL: print("FLL"); analog_setExtToggleMode(4, 0); analog_setExtToggleMode(5, 1);  break;
    case ANALOG_FRR: print("FRR"); analog_setExtToggleMode(5, 0); analog_setExtToggleMode(6, 1);  break;
    case ANALOG_BCL: print("BCL"); analog_setExtToggleMode(6, 0); analog_setExtToggleMode(7, 1);  break;
    case ANALOG_BCR: print("BCR"); break;
    case ANALOG_KEY: print("KEY"); analog_setExtToggleMode(7, 0); break;
    case ANALOG_VOLT: print("VCC"); break;
  }
  print (MAROON_STIME(1) MAROON_TXBACK("."));
  if (act_sensor<ANALOG_KEY) {
    act_value=ANALOG_DIFFERENTIAL;
  } else {
    act_value=ANALOG_PASSIVE;
  }
}

void select_next_value() {
  if (++act_value>ANALOG_DIFFERENTIAL) {
    act_value=0;
  }
  print (MAROON_IMM_CLEAR() MAROON_BRIGHT(5) MAROON_STIME(125));
  switch (act_value) {
    case ANALOG_PASSIVE: print("pass"); break;
    case ANALOG_ACTIVE: print("act"); break;
    case ANALOG_DIFFERENTIAL: print("diff"); break;
  }  
  print (MAROON_STIME(1) MAROON_TXBACK("."));
}

void setup() {
  activate_output_group(IO_LEDS);
  usart_setbaudrate(38400);
  usart_enable();
  analog_init();
  sei();
  surface_init();
  
  for (uint8_t i=0; i<8; i++) {
    analog_setExtToggleMode(i, 0);
  }
  
  delay(100);
  delay(100);
  print(MAROON_FONT_ECO() "sensormon\n" MAROON_STIME(1) MAROON_TXBACK("."));
  act_sensor=ANALOG_CNT;
  select_next_sensor();
}

void loop() {
  if (!usart_rxempty()) {
    char c = usart_getchar();
    if (c=='a') {
      // FN pressed
      select_next_sensor();
    }
    if (c=='b') {
      // Enter pressed
      select_next_value();
    }
    if (c=='.') {
      display_data();
    }
  }
  delay(50);
}






int main () {
  sei();
  setup();
  while(1) {
    loop();
  }
}