
LIBDIR = $(BASEDIR)/lib/$(PLATFORM)
HEXDIR = $(BASEDIR)/hex/$(PLATFORM)

INCLUDEDIR = $(BASEDIR)/src

OBJ_DIR = obj-$(PLATFORM)
   
OBJS = $(SRC:%.c=$(OBJ_DIR)/%.o)
DEPS = $(SRC:%.c=$(OBJ_DIR)/%.d)  

# avrdude configuration 
PROGRAMMER = stk500v2
AVRDUDE_TIME_PROG = 2
AVRDUDE_TIME_FUSE = 10

AVRDUDE_FLAGS = -c $(PROGRAMMER) -P /dev/ttyACM0 -p $(DEVICE)

# AVR compiler setup
PREFIX =
CC = $(PREFIX)avr-gcc
OBJCOPY = $(PREFIX)avr-objcopy
OBJDUMP = $(PREFIX)avr-objdump
AVRDUDE = avrdude


# AVR compiler and linker flags
CFLAGS += -Os -ffunction-sections -DAVR -I$(INCLUDEDIR) -mmcu=$(DEVICE) -std=c99
CFLAGS += -DF_CPU=$(F_CPU) -DVERSION="\"$(VERSION)\"" -D_NIBO_2_
CLDFLAGS += -v -mmcu=$(DEVICE) -L$(LIBDIR)

# build intel hex files
all: $(OBJ_DIR)/$(TARGET).hex $(OBJ_DIR)/$(TARGET).xhex


$(OBJ_DIR)/%.o: %.c Makefile
	$(CC) $(CFLAGS) -c $< -o $@

$(OBJ_DIR)/%.d: %.c
	@mkdir -p $(OBJ_DIR)
	@echo "calculating dependencies for $<"
	@set -e; $(CC) -MM $(CFLAGS) $< \
	| sed 's#\($*\)\.o[ :]*#$(OBJ_DIR)/\1.o $@ : #g' > $@ ; \
	[ -s $@ ] || rm -f $@

# avr specific entries
$(OBJ_DIR)/%.elf: $(OBJS)
	$(CC) $(CLDFLAGS) -o $@ $(OBJS) $(LIBS) $(MATH)
	avr-size -A $(OBJ_DIR)/$(TARGET).elf

$(OBJ_DIR)/%.hex: $(OBJ_DIR)/%.elf
	$(OBJCOPY) -j .text -j .data -O ihex $< $@

$(OBJ_DIR)/$(TARGET).lss: $(OBJ_DIR)/$(TARGET).elf
	avr-objdump -h -S $(OBJ_DIR)/$(TARGET).elf >$(OBJ_DIR)/$(TARGET).lss

$(OBJ_DIR)/%.xhex: $(OBJ_DIR)/%.hex
	@echo "building $@"
	@echo >$@ "<?xml version=\"1.0\"?>"
	@echo >>$@ "<xhex version=\"1.0\">"
	@echo >>$@ "<platform>$(subst nibo2-,NIBO2-,$(PLATFORM))</platform>"
	@echo >>$@ "<programmer type=\"$(PROGRAMMER)\"/>"
	@echo >>$@ "<device part=\"$(DEVICE)\" erase=\"yes\">"
	@echo >>$@ "<segment id=\"hfuse\" verify=\"yes\"><data>$(HFUSE)</data></segment>"
	@echo >>$@ "<segment id=\"lfuse\" verify=\"yes\"><data>$(LFUSE)</data></segment>"
	@echo >>$@ "<segment id=\"flash\" format=\"ihex\">"
	@cat >>$@ $<
	@echo >>$@ "</segment>"
	@echo >>$@ "</device>"
	@echo >>$@ "</xhex>"

lss: $(OBJ_DIR)/$(TARGET).lss


avrdude: $(OBJ_DIR)/$(TARGET).hex
	$(AVRDUDE) $(AVRDUDE_FLAGS) -B $(AVRDUDE_TIME_FUSE) -U lfuse:w:$(LFUSE):m -U hfuse:w:$(HFUSE):m
	$(AVRDUDE) $(AVRDUDE_FLAGS) -B $(AVRDUDE_TIME_PROG) -U flash:w:$(OBJ_DIR)/$(TARGET).hex

clean:
	@rm -rf *~ obj-*

tarball: $(OBJ_DIR)/$(TARGET).hex $(OBJ_DIR)/$(TARGET).xhex
	@mkdir -p $(HEXDIR)
	cp $^ $(HEXDIR)


.PHONY: clean lss avrdude
.SECONDARY: $(OBJ_DIR)/$(TARGET).hex $(OBJS) $(LOBJS)
