#include <avr/interrupt.h>
#include "condition.h"

/*
  cdll_t waiting_threads;
*/

void _mutext_lockCondition(mutex_t * mutex, uint8_t value);

uint8_t _mutext_unlockCondition(mutex_t * mutex);


void condition_init(condition_t * condition) {
  cdll_reset(&(condition->waiting_threads));
}


void condition_delete(condition_t * condition) {
}

void _mutext_lockCondition(mutex_t * mutex, uint8_t value);


uint8_t condition_wait(condition_t * condition, mutex_t * mutex, uint16_t timeout) {
  uint8_t result;
  cli();
  thread_t * thread = thread_getCurrent();
  
  if (timeout==TO_POLL) {
    // conditions can't be polled - return false
    result = 0;
    
  } else {
    // unlock mutex, save spinlock variable
    uint8_t mutexCnt = _mutext_unlockCondition(mutex); 

    // wait for signal...
    cdll_push_back(&(condition->waiting_threads), &(thread->rivals_list));
    if (timeout == TO_INF) {
      thread_wait();
      result = 1;      
    } else {
      result = thread_waitTimeout(timeout);
    }
    cdll_remove(&(thread->rivals_list));

    // lock mutex, restore spinlock variable
    _mutext_lockCondition(mutex, mutexCnt);
    // mutex is always locked at this position!
  }
  sei();
  return result;    
}


void condition_broadcast(condition_t * condition) {
  cli();
  cdll_t * next = condition->waiting_threads.next;
  while (next != &condition->waiting_threads) {
    thread_t * t = THREAD_FROM_RIVALS_CDLL(next);
    thread_signalISR(t);
  }
  sei();
}


void condition_broadcastISR(condition_t * condition) {
  cdll_t * next = condition->waiting_threads.next;
  while (next != &condition->waiting_threads) {
    thread_t * t = THREAD_FROM_RIVALS_CDLL(next);
    thread_signalISR(t);
  }
}


void condition_signal(condition_t * condition) {
  cli();
  cdll_t * next = condition->waiting_threads.next;
  if (next != &condition->waiting_threads) {
    thread_t * t = THREAD_FROM_RIVALS_CDLL(next);
    thread_signalISR(t);
  }
  sei();
}


void condition_signalISR(condition_t * condition) {
  cdll_t * next = condition->waiting_threads.next;
  if (next != &condition->waiting_threads) {
    thread_t * t = THREAD_FROM_RIVALS_CDLL(next);
    thread_signalISR(t);
  }
}
