#ifndef CONDITION_H
#define CONDITION_H

#include "thread.h"
#include "mutex.h"

/*! @file    condition.h
 *  @brief   Eine Condition dient zum geregelten (Deadlock-freien) Warten auf ein Ereignis.
 * 
 *  Mit einer Condition ist es möglich, mehrere Thrads gleichzeitig über ein Ereignis zu 
 *  informieren. Das Signalisieren einer Condition wecke einen Thread auf, das Broadcasten
 *  einer Condition weckt (nacheinander) alle wartenden Threads auf!
 */



typedef struct _condition_s {
  cdll_t waiting_threads;
} condition_t;



/**
  * Condition initialisieren
  */
void condition_init(condition_t * condition);


/**
  *
  */
void condition_delete(condition_t * condition);


/**
  * Der Funktionsaufruf gibt den Mutex frei und wartet auf die Signalisierung. Bevor die Funktion zurückkehrt wird der Mutex wieder gelockt!
  */
uint8_t condition_wait(condition_t * condition, mutex_t * mutex, uint16_t timeout);


/**
  * Einen wartenden Thread informieren
  */
void condition_signal(condition_t * condition);


/**
  * Alle wartenden Threads informieren
  */
void condition_broadcast(condition_t * condition);


/**
  * Einen wartenden Thread informieren
  */
void condition_signalISR(condition_t * condition);


/**
  * Alle wartenden Threads informieren
  */
void condition_broadcastISR(condition_t * condition);



#endif
