#include <avr/interrupt.h>
#include "mutex.h"
#include "cdll.h"
#include <assert.h>

/*
  cdll_t waiting_threads;
  thread_t * locker;
  uint16_t counter;
*/


void mutext_init(mutex_t * mutex) {
  cdll_reset(&(mutex->waiting_threads));
  mutex->locker = 0;
  mutex->counter = 0;
}


void mutext_delete(mutex_t * mutex) {
}


uint8_t mutext_lock(mutex_t * mutex, uint16_t timeout) {
  uint8_t result;
  cli();
  thread_t * thread = thread_getCurrent();
  
  if ((mutex->counter==0) || (mutex->locker==thread)) {
    // mutex is free or we are already the owner
    mutex->counter++;
    mutex->locker = thread;
    result = 1;
  
  } else if (timeout==TO_POLL) {
    // not yet available - return false
    result = 0;
    
  } else {
    // wait for mutex...
    cdll_push_back(&(mutex->waiting_threads), &(thread->rivals_list));
    if (timeout == TO_INF) {
      thread_wait();
    } else {
      thread_waitTimeout(timeout);
    }
    cdll_remove(&(thread->rivals_list));
    if (mutex->counter==0) {
      mutex->counter++;
      mutex->locker = thread;
      result = 1;
    } else {
      result = 0;
    }  
  }
  sei();
  return result;
}


void mutext_unlock(mutex_t * mutex) {
  // assert (mutex->locker==thread_getCurrent())
  // assert (mutex->counter>0)
  cli();
  mutex->counter--;
  if (mutex->counter==0) {
    cdll_t * next = cdll_peek_front(&(mutex->waiting_threads));
    if (next) {
      thread_t * t = THREAD_FROM_RIVALS_CDLL(next);
      thread_signalISR(t);
    }
  }
  sei();
}


uint8_t mutext_lockISR(mutex_t * mutex) {
  if ((mutex->counter==0) || (mutex->locker==0)) {
    mutex->counter++;
    mutex->locker=0;
    return 1;
  } else {
    return 0;
  }
}


void mutext_unlockISR(mutex_t * mutex) {
  // assert (mutex->locker==0)
  // assert (mutex->counter>0)
  mutex->counter--;
  if (mutex->counter==0) {
    cdll_t * next = cdll_peek_front(&(mutex->waiting_threads));
    if (next) {
      thread_t * t = THREAD_FROM_RIVALS_CDLL(next);
      thread_signalISR(t);
    }
  }
}


void _mutext_lockCondition(mutex_t * mutex, uint8_t value) {
  if ((mutex==0) || (value==0)) {
    return;
  }
  thread_t * thread = thread_getCurrent();
  while (1) {
    if (mutex->counter==0) {
      // mutex is free
      mutex->counter = value;
      mutex->locker = thread;
      return;
    }
    // wait for mutex...
    cdll_push_back(&(mutex->waiting_threads), &(thread->rivals_list));
    thread_wait();
    cdll_remove(&(thread->rivals_list));
  }
}


uint8_t _mutext_unlockCondition(mutex_t * mutex) {
  if (mutex==0) {
    return 0;
  }
  uint8_t res = mutex->counter;
  mutex->counter = 0;
  cdll_t * next = cdll_peek_front(&(mutex->waiting_threads));
  if (next) {
    thread_t * t = THREAD_FROM_RIVALS_CDLL(next);
    thread_signalISR(t);
  }
  return res;
}
