#include <avr/interrupt.h>
#include "semaphore.h"

/*
  cdll_t waiting_threads;
  uint16_t counter;
*/

void semaphore_init(semaphore_t * semaphore) {
  cdll_reset(&(semaphore->waiting_threads));
  semaphore->counter = 0;  
}

void semaphore_delete(semaphore_t * semaphore) {
}

uint8_t semaphore_acquire(semaphore_t * semaphore, uint16_t timeout) {
  uint8_t result;
  cli();
  thread_t * thread = thread_getCurrent();
  
  if (semaphore->counter>0) {
    // semaphore is available
    semaphore->counter--;
    result = 1;
  
  } else if (timeout==TO_POLL) {
    // not yet available - return false
    result = 0;
    
  } else {
    // wait for mutex...
    cdll_push_back(&(semaphore->waiting_threads), &(thread->rivals_list));
    if (timeout == TO_INF) {
      thread_wait();
    } else {
      thread_waitTimeout(timeout);
    }
    cdll_remove(&(thread->rivals_list));
    if (semaphore->counter>0) {
      semaphore->counter--;
      result = 1;
    } else {
      result = 0;
    }  
  }
  sei();
  return result;    
}

void semaphore_signal(semaphore_t * semaphore) {
  cli();
  semaphore->counter++;
  cdll_t * next = cdll_peek_front(&(semaphore->waiting_threads));
  if (next) {
    thread_t * t = THREAD_FROM_RIVALS_CDLL(next);
    thread_signalISR(t);
    }
  sei();
}

uint8_t semaphore_acquireISR(semaphore_t * semaphore) {
  if (semaphore->counter>0) {
    // semaphore is available
    semaphore->counter--;
    return 1;
  } else {
    return 0;
  }
}

void semaphore_signalISR(semaphore_t * semaphore) {
  semaphore->counter++;
  cdll_t * next = cdll_peek_front(&(semaphore->waiting_threads));
  if (next) {
    thread_t * t = THREAD_FROM_RIVALS_CDLL(next);
    thread_signalISR(t);
  }
}
