/*  BSD-License:

Copyright (c) 2007 by Nils Springob, nicai-systems, Germany

All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.
  * Neither the name nicai-systems nor the names of its contributors may be
    used to endorse or promote products derived from this software without
    specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

/*! @file    base.h
 *  @brief   \~english Supply voltage functions and internal definitions for the NIBObee Library
 *           \~german Funktionen für die Versorgungsspannung und interne Definitionen der NIBObee Library
 *  @author  Nils Springob (nils@nicai-systems.de)
 *  @date    2009-09-26
 */

#ifndef _NIBOBEE_BASE_H_
#define _NIBOBEE_BASE_H_


#include <stdint.h>

#ifdef __cplusplus
extern "C" {
#endif


/*!
 * \internal
 * \~english Initialization state values for the library modules
 * \~german Werte für die Initialisierungszustände der Module
 */
enum {
  NIBOBEE_ANALOG_INITIALIZED = 0x01,
  NIBOBEE_MOTPWM_INITIALIZED = 0x02,
  NIBOBEE_I2C_INITIALIZED    = 0x04,
};

/*!
 * \internal
 * \~english Initialization state of NIBObee Library modules
 * \~german Initialisierungszustand der NIBObee Library Module
 */
extern uint8_t nibobee_initialization;


/*!
 * \~english generate value for nibobee_assertMinVoltage() function
 * @return value for nibobee_assertMinVoltage() function
 * \~german generiert den Wert für die nibobee_assertMinVoltage() funktion
 * @return Wert für die nibobee_assertMinVoltage() funktion
 */
#define NIBOBEE_ASSERT_VOLTAGE(x) ((uint16_t)(1248/x))




/*!
 * \~english return accumulator voltage, requiers call to function analog_init()
 * @return accumulator voltage in millivolt
 * \~german Akkuspannung zurückgeben, erfordert den Aufruf der Funktion analog_init()
 * @return Akkuspannung in Millivolt
 */
uint16_t nibobee_getMillivolt();


/*!
 * \~english check accumulator voltage, blink SOS signal if voltage is below.
 * \par value result from NIBOBEE_ASSERT_VOLTAGE() macro
 * \~german Akkuspannung prüfen, SOS Blinken wenn die Akkuspannung niedriger ist.
 * \par value Ergebnis des NIBOBEE_ASSERT_VOLTAGE() makros
 */
void nibobee_assertMinVoltage(uint16_t value);


#ifdef __cplusplus
} // extern "C"
#endif

#endif // _NIBOBEE_BASE_H_

