/*  BSD-License:

Copyright (c) 2007 by Nils Springob, nicai-systems, Germany

All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.
  * Neither the name nicai-systems nor the names of its contributors may be
    used to endorse or promote products derived from this software without
    specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

/*!
 *  @file    led.h
 *  \~german
 *  @brief   Routinen zur Ansteuerung der LEDs
 *  \~english
 *  @brief   functions for LED control
 *  \~
 *  @author  Nils Springob (nils@nicai-systems.de)
 *  @date    2009-09-26
 */

#ifndef _NIBOBEE_LED_H_
#define _NIBOBEE_LED_H_

#ifdef __cplusplus
extern "C" {
#endif


enum {
  LED_L_YE = IO_LEDS_BIT_L_YE,
  LED_L_RD = IO_LEDS_BIT_L_RD,
  LED_R_RD = IO_LEDS_BIT_R_RD,
  LED_R_YE = IO_LEDS_BIT_R_YE
};

/*!
 * \~german
 * Initialisierung der LEDs.
 * Die Funktion muss vor Aufruf der anderen LED-Funktion aufgerufen werden um
 * das Datenrichtungs-Register (DDRB) für die Ausgabe zu konfigurieren.
 * \~english
 * This function will configure the LED bits of the data direction register DDRB for output.
 * It must be called once before a call to led_set() is made.
 */
void led_init();

/*!
 * \~german
 * Ein / Ausschalten der LEDs.
 * @param led ist die Nummer der LED (0-3) bzw. eine der Konstanten
 * LED_L_YE, LED_L_RD, LED_R_RD oder LED_R_YE
 * @param status Beim Wert 0 wird die LED ausgeschaltet anderenfalls
 * wird die LED eingeschaltet.
 * \~english
 * Switch on/off the LEDs.
 * @param led number of the LED (0-3) (or one of the LED-constant values
 * LED_L_YE, LED_L_RD, LED_R_RD oder LED_R_YE)
 * @param status if the value is 0 the LED will be switched off, otherwise it will be switched on.
 */
void led_set(uint8_t led, uint8_t status);

/*!
 * \~german
 * Zustand der LED abfragen.
 * @param led ist die Nummer der LED (0-3) bzw. eine der Konstanten
 * LED_L_YE, LED_L_RD, LED_R_RD oder LED_R_YE
 * @result Liefert den Wert 0 bei ausgeschalteter LED und den Wert 1 bei eingeschalteter LED
 * \~english
 * Get status of LED.
 * @param led number of the LED (0-3) (or one of the LED-constant values
 * LED_L_YE, LED_L_RD, LED_R_RD oder LED_R_YE)
 * @result returns 0 if the LED is switched off, otherwise it will return the value 1
 */
uint8_t led_get(uint8_t led);

#ifdef __cplusplus
} // extern "C"
#endif



#endif // _NIBOBEE_LED_H_
