/*  BSD-License:

Copyright (c) 2007 by Nils Springob, nicai-systems, Germany

All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.
  * Neither the name nicai-systems nor the names of its contributors may be
    used to endorse or promote products derived from this software without
    specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

/*! @file    sens.h
 *  @brief   Funktionen zur Auswertung der Tastsensoren/Fühler 
 *  @author  Nils Springob (nils@nicai-systems.de)
 *  @date    2009-08-19
 */

#ifndef _NIBOBEE_SENS_H_
#define _NIBOBEE_SENS_H_

#include "feeler.h"

#define SENS_LB FEELER_LB
#define SENS_LF FEELER_LF
#define SENS_RB FEELER_RB
#define SENS_RF FEELER_RF

/*!
 * Initialisierung der Fühler-Sensoren.
 * Die Pullups werden durch Aufruf dieser Funktion aktiviert.
 */
#define sens_init() feeler_init()


/*!
 * Liefert den Status des linken Fühlers zurück.
 * der Wert 0 bedeutet keine Betätigung, +1 bedeutet Betätigung nach
 * vorn, -1 Betätigung nach hinten.
 */
#define sens_getLeft() feeler_getLeft()


/*!
 * Liefert den Status des rechten Fühlers zurück.
 * der Wert 0 bedeutet keine Betätigung, +1 bedeutet Betätigung nach
 * vorn, -1 Betätigung nach hinten.
 */
#define sens_getRight() feeler_getRight()

/*!
 * Liefert den Status von beiden Fühlern als binäre Zahl zurück.
 */
#define sens_getAll() feeler_getAll()

/*!
 * Wartet auf betätigung der Fühler und kehrt nach dem loslassen mit dem Status von beiden Fühlern als binäre Zahl zurück.
 * Wurde nur ein Taster betätigt, entspricht der Rückgabewert diesem Taster.
 * Wurden mehrere Taster betätigt, dann entsprechen die 4 niedrigsten Bits dem zuerst betätigtem Taster, in dden 4 oberen Bits 
 * Sind alle betätigten Taster ODER verknüpft.
 * Bsp.: SENS_LB wird gehalten und dabei SENS_RB und SENS_RF ausgeöst. Der Rückgabewert ist dann 1 + 16*(4+8) = 193.
 */
#define sens_waitInput() feeler_waitInput()


#endif // _NIBOBEE_SENS_H_

